c---+----1----+----2----+----3----+----4----+----5----+----6----+----7--
c
c **********************************************************************
c * ****************************************************************** *
c * *                                                                * *
c * *       FFFFFFFFFFF     AA       NNN      NNNN     SSSSSS        * *
c * *        FF      FF    A AA       NNN      NN    SS      SS      * *
c * *        FF       F    A AA       NN N     NN   SS               * *
c * *        FF    F      A   AA      NN  N    NN    SS              * *
c * *        FFFFFFF      A   AA      NN   N   NN      SSSSS         * *
c * *        FF    F     AAAAAAAA     NN    N  NN           SS       * *
c * *        FF          A     AA     NN     N NN            SS      * *
c * *        FF         A       AA    NN      NNN   SS      SS       * *
c * *       FFFF       AAA     AAAA  NNNN      NNN    SSSSSS         * *
c * *                                                                * *
c * ****************************************************************** *
c **********************************************************************
c
c
c                 FANS, VERSION 1.1, RELEASE OF 15JUN19
c
c                          David Alan Clarke
c                  Department of Astronomy and Physics
c                  Saint Mary's University, Halifax NS
c
c=============== SIMPLIFIED PUBLIC VERSION USING 1DPLOT ================
c
c  PREAMBLE: This program computes profiles for the primitive variables
c  rho, p, v_x, v_y, v_z, B_y, B_z across an MHD rarefaction fan (RF) of
c  specified family type (1 => left-moving fast RF, 3 => left-moving
c  slow RF, 5 => right-moving slow RF, and 7 => right-moving fast RF)
c  with the specified upwind primitive variable values (including B_x)
c  and a specified fan "depth".
c
c  Fast fans can "saturate":
c
c    - bperp > 0, RF saturates when bperp --> 0 (switch-off)
c
c    - bperp = 0, alphax < 1, fast Euler RF saturates when alphax --> 1
c      (triple umbilic)
c
c  If specified depth ("slim") is greater than the saturation depth, the
c  program reports the saturation depth, and only plots the data to that
c  point.
c
c  For bperp = 0 and alphax .leq. 1, there is no fast RF.
c
c  Slow fans don't saturate, but rather asymptote to when p --> 0, but
c  with ever increasing width.
c
c  Once computed, two data files describing the RF are written to disc:
c
c  1. ascii data file (fans.dat) suitable for use by GNUPLOT.  Columns
c     left to right are line number n, generalised coordinate s, self-
c     coordinate x/t = v_x \pm c_w (wave speed), rho, p, v_x, v_y, v_z,
c     B_y, B_z, alpha_x, alpha_perp, and c_w.  To view the solution of
c     the ith variable with GNUPLOT, issue the command:
c
c     plot "fans.dat" u 2:4 w lines   (p vs. s)
c     plot "fans.dat" u 2:9 w lines   (by vs. s)
c     plot "fans.dat" u 3:9 w lines   (by vs x/t)
c     plot "fans.dat" u 11:12 w lines (alpha_perp vs alpha_x)
c
c  2. a 3 x 3 frame psplot (fans.ps) of the variables listed above
c     plotted as functions of the self-similar coordinate, x/t.
c
c
c  This program is based on discussions in section 6.2 in the MHD text.
c
c=======================================================================
c
c    \\\\\\\\\\         B E G I N   P R O G R A M         //////////
c    //////////                  F A N S                  \\\\\\\\\\
c
c=======================================================================
c
       program fans
c
c    dac:fans <--------------- integrates and plots MHD rarefaction fans
c                                                         november, 2018
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE:  This program uses 6th-order Runge-Kutta to integrate across
c  any flavour of MHD rarefaction fan.  See program banner for complete
c  functionality.
c
c  EXTERNALS:
c    NMLSTS
c    RK
c    PLOTIT
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       real*8        maxerr  , ds0
       parameter     ( maxerr=1.0d-7, ds0=0.001d0 )
c
       logical       satur8  , nofastrf, noslowrf, norf
       character*5   dir     , fors
       character*18  frmt
       character*15  dirrf
       character*128 phdr
       integer       node    , n       , ipts    , i       , ndir
     1             , ii
       real*8        pm1     , ds      , sleft   , sright  , dshalf
     1             , dptwid  , dpmid   , dpstar  , errn    , err
     1             , smax    , xlim    , bperp   , alphax  , alphaf
     1             , alphas  , cssq    , disc    , frds    , dpacc
     1             , xmax    , delx    , delxmin
c
       real          x4      (imax), alfp4   (imax), amag4   (imax)
       real*8        s       (imax), x       (imax), alfx    (imax)
     1             , amag    (imax), alfp    (imax)
     1             , yleft   (pvar), ytwid   (pvar), ymid    (pvar)
     1             , ystar   (pvar), yright  (pvar)
c
       real          y4      (imax,pvar)
       real*8        y       (imax,pvar)
c
       data          delxmin  / 1.0d-6 /
c
       external      nmlsts  , rk      , plotit
c
c-----------------------------------------------------------------------
c
c------opening procedures-----------------------------------------------
c
c      Read namelists.
c
       call nmlsts
c
c      Opening banner.
c
       write ( iotty, 2010 )
c
c      Initialise local variables.
c
       node     = pvar
       smax     = 0.0d0
       satur8   = .false.
       nofastrf = .false.
       noslowrf = .false.
       if (family .lt. 4) then
         pm1    =-1.0d0
         dir    = 'Left'
         ndir   = 4
         dirrf  = 'left-moving RF '
       else
         pm1    = 1.0d0
         dir    = 'Right'
         ndir   = 5
         dirrf  = 'right-moving RF'
       endif
       if ( (family .eq. 3) .or. (family .eq. 5) ) fors = 'slow '
       if ( (family .eq. 1) .or. (family .eq. 7) ) fors = 'fast '
c
c      Compute "bperp" and components of perpendicular unit vector.
c
       coschi = 0.8d0
       sinchi = 0.6d0
       bperp  = sqrt ( y0(6)**2 + y0(7)**2 )
       if (bperp .gt. zero) then
         coschi = y0(6) / bperp
         sinchi = y0(7) / bperp
       endif
c
c      Compute "alphax" (axsq/cssq).
c
       alphax = bx0**2 * gammai / y0(2)
       if (abs(alphax-1.0d0) .lt. zero) alphax = 1.0d0
c
c      For bx0 = 0, there is no slow fan; for bperp = 0, alphax .ge. 1,
c  there is no fast fan.  Take care of these special cases now before
c  the Runge-Kutta loop.
c
       noslowrf = (fors .eq. 'slow') .and. (bx0    .eq. 0.0d0)
       nofastrf = (fors .eq. 'fast') .and. (bperp  .eq. 0.0d0)
     1                               .and. (alphax .ge. 1.0d0)
       norf     = noslowrf .or. nofastrf
c
       if ( norf ) then
         i    = 2
         s(i) = slim
         do 10 n=1,node
           y(i,n) = y0(n)
10       continue
         go to 80
       endif
c
c      Initialise parameters for next Runge-Kutta integration.
c
       i     = 1
       ds    = ds0
       sleft = 0.0d0
       s(1)  = sleft
c
c      Set upwind state for rarefaction wave.
c
       do 30 n=1,node
         yleft(n) = y0(n)
         y  (1,n) = y0(n)
30     continue
c
c------start of adaptive loop-------------------------------------------
c
40     continue
       sright = min ( sleft + ds, slim )
       ds     = sright - sleft
       dshalf = 0.5d0 * ds
c
c      Runge-Kutta integrates "yleft" by a full step to "ytwid", then
c  again by two half steps to "ystar".
c
       call rk ( yleft, ytwid, node, ds    , dptot, dptwid )
       call rk ( yleft, ymid , node, dshalf, dptot, dpmid  )
       dpacc = dptot + dpmid
       call rk ( ymid , ystar, node, dshalf, dpacc, dpstar )
c
c      Evaluate "yright" ("y" at the right side of the interval) using
c  Richardson extrapolation on "ytwid" and "ystar", then estimate the
c  error.
c
       err = 0.0d0
       do 50 n=1,node
         yright(n) = ( 16.0d0 * ystar(n) - ytwid(n) ) / 15.0d0
         errn      = abs ( ( yright(n) - ystar(n) )
     1                   / ( ( yright(n) + ystar(n) ) + tiny ) ) * 2.0d0
         err       = max ( err, errn )
50     continue
c
c      If error is too large, repeat iteration with a half step size.
c
       if (err .gt. maxerr) then
         ds = dshalf
         go to 40
       endif
c
c      For fast fans only, if either or both components of "bperp" have
c  changed sign from the previous step (which analytically, should not
c  happen), the solution has overshot the terminus of a switch-off fan.
c  In this case, use a bisection approach by repeating last iteration
c  with a half step size.  This allows b_y and b_z to approach zero
c  without ever changing sign, and converges within 100 steps or fewer.
c
       if (fors .eq. 'fast') then
         if ( ( yright(6) * y(i,6) .lt. 0.0d0 ) .or.
     1        ( yright(7) * y(i,7) .lt. 0.0d0 ) ) then
           ds = dshalf
           go to 40
         endif
       endif
c
c      If error is much smaller than needed, increase step size for next
c  time (but don't redo integration).
c
       if (err .lt. 0.1d0*maxerr) ds = 1.5d0 * ds
c
c      If execution arrives here, "yright" has been integrated to within
c  error tolerance and is stored in the solution array "y".  "sright" is
c  correspondingly stored in the independent variable array "s".  The
c  left-values ("sleft", "yleft") are then shifted to the right-values
c  ("sright", "yright") in preparation for the next iteration.
c
c  Other quantities set here:
c
c    dptot   accumulated deviation from critical pressure (B_x**2/gamma)
c            is updated using the same Richardson extrapolation step as
c            used above to calculate "yright".
c
c    frds    fractional change in generalised coordinate, s, used below
c            to detect whether a fast fan has been saturated.
c
       i = i + 1
       if (i .le. imax) then
c
         do 60 n=1,node
           y  (i,n) = yright(n)
           yleft(n) = yright(n)
60       continue
         dptot = dptot
     1         + ( 16.0d0 * ( dpmid + dpstar ) - dptwid ) / 15.0d0
         s (i) = sright
         frds  = ( sright - sleft ) / max ( sleft, sright )
         sleft = sright
c
c      For fast fans only, we test for "saturation".
c
c  If frds < maxerr, declare fan to be "saturated", which occurs when:
c
c  1.  for alphax > 1 and bperp > 0, fast RF is a switch-off fan, and
c      bperp has fallen essentially to zero; or
c
c  2.  for alphax < 1 and bperp = 0, RF is a fast Euler fan that has
c      reached the triple umbilic (bperp = 0, alphax = 1) through which
c      it cannot pass.
c
c  In either case, redirect execution to where a saturated fan is
c  identified.
c
         if ( (fors .eq. 'fast') .and. (abs(frds) .lt. maxerr) )
     1     go to 70
c
c      If execution gets here and desired limit of integration has not
c  yet been reached, do another RK integration step.  Otherwise, exit.
c
         if (sleft .lt. slim) go to 40
         go to 80
c
       else
c
c      If execution gets here, maximum number of steps has been
c  exceeded.  Issue warning message and exit.
c
         write ( iotty, 2020 ) sleft, imax
         i = imax
         go to 80
c
       endif
c
c------end of adaptive loop---------------------------------------------
c
c      If execution is redirected here, the fast RF was saturated before
c  reaching specified depth "slim".  Report value of "smax" (maximum fan
c  depth) along with the nature of the RF ("switch-off" or "saturated
c  Euler").
c
c  FORTRAN77 note: // means "concatenate".
c
70     continue
       satur8 = .true.
       smax   = sright
       ipts   = i
       write ( iotty, 2030 ) smax, 1.0d2*frds
       if (bperp .ne. 0.0d0) then
         write ( iotty, 2040 ) dir(1:ndir), ipts
         phdr = 'Switch-off (fast) ' // dirrf
       else
         write ( iotty, 2050 ) dir(1:ndir), ipts
         phdr = 'Saturated fast Euler ' // dirrf
       endif
       go to 90
c
c      If execution is redirected here, the RF has been integrated
c  through to "slim", and is not a saturated fast RF.  Report the RF
c  type.
c
80     continue
       ipts = i
       if ( noslowrf ) then
         write ( iotty, 2060 )
         phdr = 'No slow RF for |F10|B|B|x|NF5| = 0'
       else
         if (bperp .ne. 0.0d0) then
           write ( iotty, 2070 ) dir(1:ndir), fors, ipts
           phdr = 'Ordinary ' // fors // dirrf
         else
           if (fors .eq. 'fast ') then
             if (alphax .lt. 1.0d0) then
               write ( iotty, 2080 ) dir(1:ndir), ipts
               phdr = 'Ordinary Euler (fast) ' // dirrf
             endif
             if (alphax .eq. 1.0d0) then
               write ( iotty, 2090 )
               phdr = 'No fast RF for |F5|a|BF10|x|NF5| = 1 |F0| '
     1                // '(triple umbilic)'
             endif
             if (alphax .gt. 1.0d0) then
               write ( iotty, 2100 )
               phdr = 'No fast RF for |F5|a|BF10|x|NF5| < 1'
             endif
           else
             if ( (alphax .lt. 1.0d0) .or. (dp0 .gt. 0.0d0) ) then
               write ( iotty, 2110 ) dir(1:ndir), ipts
               phdr = 'Switch-on (slow) ' // dirrf
             endif
             if ( (alphax .eq. 1.0d0) .and. (dp0 .eq. 0.0d0) ) then
               write ( iotty, 2120 ) dir(1:ndir), ipts
               phdr = 'Slow Euler (triple umbilic) ' // dirrf
             endif
             if ( (alphax .gt. 1.0d0) .or. (dp0 .lt. 0.0d0) ) then
               write ( iotty, 2130 ) dir(1:ndir), ipts
               phdr = 'Slow Euler ' // dirrf
             endif
           endif
         endif
       endif
c
90     continue
c
c      We now need to compute:
c
c  x       self-similar coordinate x/t = wave speed (v +- af or as),
c          proportional to physical distance through RF.
c  amag    magnetosonic speed (fast or slow)
c
       do 100 i=1,ipts
         alphap = ( y(i,6)**2 + y(i,7)**2 ) * gammai / y(i,2)
         alphax = bx0**2 * gammai / y(i,2)
         disc   = sqrt ( ( alphax - 1.0d0 + alphap )**2
     1                 + 4.0d0 * alphap )
         alphaf = 1.0d0 + 0.5d0 * ( alphax - 1.0d0 + alphap + disc )
         alphas = alphax / alphaf
         cssq   = gamma * y(i,2) / y(i,1)
         if (fors .eq. 'fast') amag(i) = sqrt ( alphaf * cssq )
         if (fors .eq. 'slow') amag(i) = sqrt ( alphas * cssq )
         x(i)   = y(i,3) + pm1 * amag(i)
         if ( norf ) x(i) = slim * 0.5d0 * ( 1.0d0 + pm1 )
         if (i .gt. 1) then
           if (pm1 .lt. 0.0d0) x(i) = max ( x(i), x(i-1) )
           if (pm1 .gt. 0.0d0) x(i) = min ( x(i), x(i-1) )
           if ( norf ) x(i) = slim * 0.5d0 * ( 1.0d0 - pm1 )
         endif
100    continue
c
c      Execution arrives here only if Fig. 6.4 is *not* being made.  In
c  that case, only one RF is being integrated.  Store its solution in a
c  columnar format suitable for gnuplot including "s", "x", the seven
c  primitive variables in "y", plus "alfx" [bx0**2/(gamma p)], "alfp"
c  [bperp**2/(gamma p)], and "amag".
c
       write ( frmt, 2140 ) node + 5
       open  ( 10, file='fans.dat', status='unknown' )
       do 120 i=1,ipts
         alfx(i) = bx0**2 * gammai / y(i,2)
         alfp(i) = ( y(i,6)**2 + y(i,7)**2 ) * gammai / y(i,2)
         write ( 10, frmt ) i, s(i), x(i), (y(i,n),n=1,node), alfx(i)
     1                    , alfp(i), amag(i)
120    continue
       close ( 10 )
c
c      For the PSPLOT file, all plotting variables need to be converted
c  to real*4.  Index "ii" used for right-moving fans to reverse order
c  of arrays since, unlike GNUPLOT, the GRFX03 graphics library is not
c  smart enough to detect and deal with a decreasing independent
c  variable.
c
c  If a fast RF is started too close to its saturation point (e.g., fast
c  Euler fans saturate at alphax=1), it could be sufficiently narrow
c  that converting variables to single precision may make all values of
c  the independent coordinate "x4" equal, triggering FPE messages from
c  the graphics module.
c
c  Thus, if the domain of "x" is less than 10**-6 of the maximum value
c  in "x", the case is treated the same as though bperp=0 and alphax>1
c  where no fast RF solution exists, and where a two-point plot of all
c  variables set constant to the upwind values is generated.
c
       xmax = max ( abs ( x(1) ), abs( x(ipts) ) )
       delx = 0.0d0
       if (xmax .gt. 0.0d0) delx = abs ( x(ipts) - x(1) ) / xmax
c
       if (delx .gt. delxmin) then
c
c      The domain in "x" is sufficient to plot in single precision.
c  Convert all needed variables to single precision.
c
         do 140 i=1,ipts
           ii       = i
           if (pm1 .gt. 0.0d0) ii = ipts - i + 1
           x4   (i) = sngl ( x    (ii) )
           alfp4(i) = sngl ( alfp (ii) )
           amag4(i) = sngl ( amag (ii) )
           do 130 n=1,node
             y4(i,n) = sngl ( y(ii,n) )
130        continue
140      continue
c
       else
c
c    The domain in "x" is insufficient to plot in single precision.
c  For the purpose of plotting something, create a two-point constant
c  profile for all variables over domain [0,slim], similar to what is
c  done before the adaptive loop for bperp=0 and alphax>1 (no fast RF).
c
         x4(1) = 0.0
         x4(2) = sngl ( slim )
         ipts  = 2
         do 160 i=1,ipts
           alfp4(i) = sngl ( alfp (1) )
           amag4(i) = sngl ( amag (1) )
           do 150 n=1,node
             y4(i,n) = sngl ( y0(n) )
150        continue
160      continue
         write ( iotty, 2150 ) x(1), x(ipts)
c
       endif
c
c      Create PSPLOT file.
c
       call plotit ( x4, y4, alfp4, amag4, ipts, phdr )
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-------------0----+----1----+----2----+----3----+----4----+----5----+--
c
2010   format(///,12x,'FFFFF   A   N   N  SSSS',/
     1           ,12x,'F      A A  NN  N S',/
     2           ,12x,'FFFF   A A  N N N  SSS'
     3           , 5x,'FANS1.1 - RELEASE OF 15JUN19',/
     4           ,12x,'F     AAAAA N  NN     S',/
     5           ,12x,'F     A   A N   N SSSS',///)
2020   format('FANS    : |---WARNING---> Execution terminated before '
     1       ,'specified domain completed.',/
     2       ,'FANS    : Solution stored to s = ',1pg10.3,' only.  '
     3       ,'Recompile with imax > ',i6,'.')
2030   format('FANS    : "slim" exceeds fast RF depth; smax = '
     1       ,1pg15.7,' +/- ',g9.2,'%.')
2040   format('FANS    : ',a,'-moving switch-off RF generated with ',i4
     1       ,' data points stored in',/
     2       ,'FANS    : file "fans.dat".')
2050   format('FANS    : ',a,'-moving saturated Euler RF generated with'
     1       ,i4,' data points stored',/
     2       ,'FANS    : in file "fans.dat".')
2060   format('FANS    : No slow RF for bx0 = 0.  Data file generated '
     1       ,'with 2 data points stored',/
     2       ,'FANS    : in file "fans.dat".')
2070   format('FANS    : ',a,'-moving ordinary ',a,'RF generated with '
     1       ,i4,' data points stored',/
     2       ,'FANS    : in file "fans.dat".')
2080   format('FANS    : alphax < 1; ',a,'-moving fast Euler RF '
     1       ,'generated with ',i4,' data points',/
     2       ,'FANS    : stored in file "fans.dat".')
2090   format('FANS    : No fast RF for alphax = 1 (triple umbilic). '
     1       ,' Data file generated with',/
     2       ,'FANS    : 2 data points stored in file "fans.dat".')
2100   format('FANS    : No fast RF for alphax > 1.  Data file '
     1       ,'generated with 2 data points',/
     2       ,'FANS    : stored in file "fans.dat".')
2110   format('FANS    : alphax < 1; ',a,'-moving switch-on RF '
     1       ,'generated with ',i4,' data',/
     2       ,'FANS    : points stored in file "fans.dat".')
2120   format('FANS    : alphax = 1 (triple umbilic); ',a,'-moving '
     1       ,'slow Euler RF generated with',/
     2       ,'FANS    : ',i4,' data points stored in file "fans.dat".')
2130   format('FANS    : alphax > 1; ',a,'-moving slow Euler RF '
     1       ,'generated with ',i4,' data points',/
     2       ,'FANS    : stored in file "fans.dat".')
2140   format('(i5,2x,1p',i2.2,'g23.15)')
2150   format('FANS    : |---WARNING---> Domain (',1pg18.10,1x,g18.10
     1       ,') is too',/
     1       ,'FANS    : narrow to resolve in real*4.  Plots made as '
     1       ,'if there were no fast RF.')
c
       stop
       end
c
c=======================================================================
c
c    \\\\\\\\\\           E N D   P R O G R A M           //////////
c    //////////                  F A N S                  \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                D E R I V S                \\\\\\\\\\
c
c=======================================================================
c
       subroutine derivs ( y, dpacc, yp )
c
c    dac:fans.derivs <----------------------------- evaluate derivatives
c                                                         november, 2018
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE:  For the 7 coupled 1st-order ODEs describing the rarefaction
c  fan, this routine returns the values of all function derivatives, and
c  thus the corresponding components in the right eigenket.
c
c  INPUT VARIABLES:
c    y           1-D array of the "node" dependent variables.
c    dpacc       accumulated difference between pressure and critical
c                pressure, pcr = bx0**2/gamma
c
c  OUTPUT VARIABLES:
c    yp          1-D array of derivatives needed by RK.
c
c  EXTERNALS:
c    SPEEDS
c    MUNU
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       real*8        dpacc   , mu      , nu      , pm1
c
       real*8        y       (   *), yp      (   *)
c
       external      speeds  , munu
c
c-----------------------------------------------------------------------
c
       call speeds ( y(1), dpacc )
c
c      Compute "mu" and "nu", the "normalisation" factors for the fast
c  and slow eigenkets.
c
       call munu ( mu, nu )
c
c      The only difference between the left- and right- waves is a sign
c  in some terms, embodied in "pm1".
c
       pm1  = 1.0d0
       if (family .lt. 4 ) pm1 =-1.0d0
c
       if ( (family .eq. 1) .or. (family .eq. 7) ) then
c
c----- FAST RAREFACTION FANS -------------------------------------------
c
c      The fact there is no fast wave for bperp = 0 and alphax .ge. 1 is
c  taken care of in FANS.  Thus, here we set the fast eigenket for a
c  fast regular fan (bperp > 0, any alphax), a switch-off fan (saturated
c  fast regular fan), or a fast Euler fan (bperp = 0 and alphax < 1)
c  both unsaturated and saturated.
c
         yp(1) =-mu * y(1)
         yp(2) =-mu * gamma * y(2)
         yp(3) =-mu * pm1 * af
         yp(4) = nu * pm1 * sgnbx0 * as * coschi
         yp(5) = nu * pm1 * sgnbx0 * as * sinchi
         yp(6) =-nu * sqrt ( gamma * y(2) ) * coschi
         yp(7) =-nu * sqrt ( gamma * y(2) ) * sinchi
c
       else
c
c----- SLOW RAREFACTION FANS -------------------------------------------
c
c      The fact there is no slow wave for bx0 = 0 is taken care of in
c  FANS.  Thus, here we set the slow eigenket for a slow regular fan
c  (bperp > 0, any alphax), a switch-on fan (bperp = 0, alphax < 1), or
c  a slow Euler fan (bperp = 0, alphax .ge. 1).
c
         yp(1) =-nu * y(1)
         yp(2) =-nu * gamma * y(2)
         yp(3) =-nu * pm1 * as
         yp(4) =-mu * pm1 * sgnbx0 * af * coschi
         yp(5) =-mu * pm1 * sgnbx0 * af * sinchi
         yp(6) = mu * sqrt ( gamma * y(2) ) * coschi
         yp(7) = mu * sqrt ( gamma * y(2) ) * sinchi
c
       endif
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                D E R I V S                \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                  M U N U                  \\\\\\\\\\
c
c=======================================================================
c
       subroutine munu ( mu, nu )
c
c    dac:fans.munu <---------------------- computes mu and nu for derivs
c                                                          january, 2019
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This routine is called by DERIVS and computes the factors
c  "mu" and "nu" for the fast and slow RF.  By strategic use of binomial
c  expansions, "mu" and "nu" are returned with at least nine significant
c  figures, regardless of how small they may be.  The input variables
c  "alphap" and "deltax" are set by the latest call to SPEEDS, which
c  should be immediately before this routine is called.
c
c  INPUT VARIABLES (from common block comvarr):
c    alphap      apsq / cssq
c    deltax      axsq / cssq - 1
c
c  OUTPUT VARIABLES:
c    mu, nu      factors used in DERIVS
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [none]
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       real*8        mu      , nu      , nusq    , musq    , dxpap
     1             , eps     , qty
c
c-----------------------------------------------------------------------
c
       if (alphap .eq. 0.0d0) then
c
c      Take care of the alphap = 0 (bperp = 0) case first.
c
c  Programming notes:
c
c  1.  There is no fast RF for alphap = 0 and deltax .ge. 0, and thus
c      execution never gets here in that stead.  Thus, the second case
c      is only for slow waves.
c
c  2.  Algebraically, the limiting case for deltax = alphap = 0 (triple
c      umbilic; TU) is musq = nusq = 1/2.  Further, "alphax" ("deltax")
c      necessarily increases across an RF.  Thus, when starting from the
c      TU, musq = 1/2 for the very first step only seeding "bperp" (and
c      "alphap") with a small but not infinitesimal value.  Then, on the
c      second step, since "alphap" is not strictly zero, calculation of
c      "musq" is done by the "else" portion below which does its best to
c      keep "musq" at zero but, not being exactly zero, allows "bperp"
c      to grow like a switch-on fan.  As calculated by this program
c      (by setting musq = 1/2 at the TU), "bperp" grows to ~ 1.0d-16.
c
c      Now, as dp0 -> 0, maximum "bperp" for switch-on fans asymptotes
c      to 0 (this program can follow switch-on fans for "dp0" as small
c      as ~2.5d-95 resulting in a maximum "bperp" of ~2.0d-38).  Thus, a
c      slow fan originating from the TU should be an Euler fan, not a
c      switch-on fan, and the observation that "bperp" grows from the
c      TU to ~1.0d-16 is a result of round-off error.
c
c      A practical way to get around this problem is to initialise
c      "musq" to 0 at the TU rather than 0.5 (as implemented below), so
c      that no seed value is assigned to "bperp".  Thus, "alphap" stays
c      0 and "musq" continues to be set to 0 by this upper portion of
c      the routine maintaining "bperp" at exactly zero across the RF.
c
         if (deltax .lt. 0.0d0) then
           musq = 1.0d0
           nusq = 0.0d0
         else
           musq = 0.0d0
           nusq = 1.0d0
         endif
c
       else
c
c      For alphap > 0, "musq" and "nusq" can be computed from the full
c  expression most of the time.  The exception is when either expression
c  becomes a difference between 1 and a number very close to 1 resulting
c  in values dominated by round-off errors.
c
c  The quantity monitored is eps = alphap / (deltax + alphap)**2.  If
c  eps < emin, "musq" (for deltax + alphap > 0) or "nusq" (for deltax
c  + alphap < 0) is evaluated by a three-term binomial expansion.  See
c  notes "Limiting behaviour for mu**2 and nu**2", January, 2019.
c
         dxpap = deltax + alphap
         musq  = 0.5d0
     1         * ( 1.0d0 - dxpap / sqrt ( dxpap**2 + 4.0d0 * alphap ) )
         nusq  = 1.0d0 - musq
c
         if (dxpap .ne. 0.0d0) then
           eps = alphap / dxpap**2
           if (eps .lt. emin) then
             qty = eps - 3.0d0 * eps**2 + 10.0d0 * eps**3
             if (dxpap .gt. 0.0d0) musq = qty
             if (dxpap .lt. 0.0d0) nusq = qty
           endif
         endif
c
       endif
c
       mu = sqrt ( musq )
       nu = sqrt ( nusq )
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                  M U N U                  \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                N M L S T S                \\\\\\\\\\
c
c=======================================================================
c
       subroutine nmlsts
c
c    dac:fans.nmlsts <------------------ reads namelists from input deck
c                                                          january, 2019
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: Reads input parameters from namelist FANCON.  The default
c  RF is a left-moving fast fan with upwind values given by the "Sod
c  shock tube", modified to include both bx and bperp:
c
c  (d, p, vx, vy, vz, by, bz) = (3, 3, 0, 0, 0, 1, 0), with bx0 = 2.
c
c  INPUT VARIABLES:
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [none]
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       integer       n
       real*8        upd     , upp     , upvx    , upvy    , upvz
     1             , upby    , upbz
       character*32  namelist (   2,1000)
       external      nlsdac01, nlsdac02, nlsdac03, nlsdac04, nlsdac05
     1             , nlsdac06, nlsdac07, nlsdac08, nlsdac09, nlsdac10
     2             , nlsdac11, nlsdac12, nlsdac13, nlsdac14, nlsdac15
     3             , nlsdac16, nlsdac17, nlsdac18, nlsdac19, nlsdac20
     4             , nlsdac21, nlsdac22, nlsdac23, nlsdac24
c
c-----------------------------------------------------------------------
c
c      Set logical units.
c
       iotty = 6
       ioin  = 59
       ioplt = 99
c
c      Open input file.
c
       open ( unit=ioin , file='infans', status='old' )
c
c----------------------  RAREFACTION FAN CONTROL  ----------------------
c
c  family          family type
c                  1 =>  left-moving fast fan (default)
c                  3 =>  left-moving slow fan
c                  5 => right-moving slow fan
c                  7 => right-moving fast fan
c                  anything else => 1.
c  upd             upwind density                          default = 3.0
c  upp             upwind pressure                         default = 3.0
c  upvx            upwind x-velocity                       default = 0.0
c  upvy            upwind y-velocity                       default = 0.0
c  upvz            upwind z-velocity                       default = 0.0
c  upby            upwind y-magnetic field                 default = 1.0
c  upbz            upwind z-magnetic field                 default = 0.0
c  bx0             uniform x-magnetic field                default = 2.0
c  slim            depth of fan                            default = 1.0
c  dp0             upp - bx0**2/gamma                      default = 0.0
c
c  If non-zero, "dp0" seeds "dptot", the accumulated pressure difference
c  from pcr = bx0**2/gamma ("critical pressure" where alphax = 1), used
c  to calculate "deltax" until dptot/pcr becomes significant (SPEEDS).
c
       call nlsdac01 ( namelist, 1000 )
       namelist (1,   1) = 'family                       i s'
       namelist (1,   2) = 'upd                          r s'
       namelist (1,   3) = 'upp                          r s'
       namelist (1,   4) = 'upvx                         r s'
       namelist (1,   5) = 'upvy                         r s'
       namelist (1,   6) = 'upvz                         r s'
       namelist (1,   7) = 'upby                         r s'
       namelist (1,   8) = 'upbz                         r s'
       namelist (1,   9) = 'bx0                          r s'
       namelist (1,  10) = 'slim                         r s'
       namelist (1,  11) = 'dp0                          r s'
c
c      Default values
c
       family = 1
       upd    = 3.0d0
       upp    = 3.0d0
       upvx   = 0.0d0
       upvy   = 0.0d0
       upvz   = 0.0d0
       upby   = 1.0d0
       upbz   = 0.0d0
       bx0    = 2.0d0
       slim   = 1.0d0
       dp0    = 0.0d0
c
c      Read namelist
c
       call nlsdac02 ( ioin    , 'fancon  ', namelist, 1000,   11 )
       call nlsdac04 ( family  , 'family  ', namelist, 1000 )
       call nlsdac05 ( upd     , 'upd     ', namelist, 1000 )
       call nlsdac05 ( upp     , 'upp     ', namelist, 1000 )
       call nlsdac05 ( upvx    , 'upvx    ', namelist, 1000 )
       call nlsdac05 ( upvy    , 'upvy    ', namelist, 1000 )
       call nlsdac05 ( upvz    , 'upvz    ', namelist, 1000 )
       call nlsdac05 ( upby    , 'upby    ', namelist, 1000 )
       call nlsdac05 ( upbz    , 'upbz    ', namelist, 1000 )
       call nlsdac05 ( bx0     , 'bx0     ', namelist, 1000 )
       call nlsdac05 ( slim    , 'slim    ', namelist, 1000 )
       call nlsdac05 ( dp0     , 'dp0     ', namelist, 1000 )
c
c      Initialise global variables.  If "dptot" is not set to 0 when it
c  is within "zero" of 0, the triple umbilic (TU) can be missed.
c
       sgnbx0 = sign ( 1.0d0, bx0 )
       blank  = ' '
       gammai = 0.6d0
       gamma  = 1.0d0 / gammai
       gamm1  = gamma - 1.0d0
       y0(1)  = upd
       if (dp0 .ne. 0.0d0) then
         dptot = dp0
         y0(2) = dp0 + bx0**2 * gammai
       else
         dptot = upp - bx0**2 * gammai
         if (abs(dptot) .le. zero) dptot = 0.0d0
         y0(2) = upp
       endif
       y0(3) = upvx
       y0(4) = upvy
       y0(5) = upvz
       y0(6) = upby
       y0(7) = upbz
       if (slim .le. 0.0d0) slim = huge
c
c      Close input file.
c
       close ( ioin )
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                N M L S T S                \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                P L O T I T                \\\\\\\\\\
c
c=======================================================================
c
       subroutine plotit ( x, prvar, alfp, amag, ntot, phdr )
c
c    dac:fans.plotit <--------------------------- plots solution profile
c    from ASTR5700/programs/hdriemann/hdrsx.plotit            june, 2018
c
c    written by: David Clarke
c    modified 1: November, 2018 by DAC; adapted to FANS
c
c  PURPOSE:  This subroutine plots profiles across an MHD rarefaction
c  fan for the primitive variables and a few derived variables each in a
c  separate frame, all arranged on the same page, using the GRFX03
c  library. Nine variables are plotted on the same page on a 3 x 3 grid:
c
c          rho  p    vx
c          vy   vz   By
c          Bz   am   ap
c
c  where am is the fast or slow magnetosonic speed, and ap is
c  alpha_perp.
c
c  INPUT VARIABLES:
c    x           abscissa against which all variables are plotted
c    prvar       seven primitive variables: d, p, vx, vy, vz, by, bz
c    alfp        ap**2/cs**2 (alpha_perp)
c    amag        magnetosonic speed
c    ntot        total number of points in each plot
c    phdr        title for graph (blank => no title)
c
c  OUTPUT VARIABLES:
c
c  EXTERNALS:
c    GRFXOPN   |
c    HDRFTR    |
c    GRFX1D    |---from library lib1dplot.a
c    GRFXCLS   |
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       character*(*) phdr
       character*7   filename
       character*64  xlab    , ylab
       integer       ntot    , n
c
       character*1   pftr    (   4)
       real          x       (   *), alfp    (   *), amag    (   *)
c
       real          prvar   (imax,pvar)
c
       external      grfxopn , hdrftr  , grfx1d  , grfxcls
c
c-----------------------------------------------------------------------
c
c      Open graphics file.
c
       filename = 'fans.ps'
       call grfxopn ( filename )
c
c      Write header and footer.
c
       call hdrftr ( phdr, 1, filename )
c
c      Set labels.
c
       xlab    = '|F10|x/t'
c
c      Plot density.
c
       ylab = '|F5|r'
       call grfx1d ( x, prvar(1,1), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 1, 3, 1, xlab, ylab )
c
c      Plot pressure.
c
       ylab = '|F10|p'
       call grfx1d ( x, prvar(1,2), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 2, 3, 1, xlab, ylab )
c
c      Plot x-velocity.
c
       ylab = '|F10|v|B|x'
       call grfx1d ( x, prvar(1,3), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 3, 3, 1, xlab, ylab )
c
c      Plot y-velocity.
c
       ylab = '|F10|v|B|y'
       call grfx1d ( x, prvar(1,4), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 1, 3, 2, xlab, ylab )
c
c      Plot z-velocity.
c
       ylab = '|F10|v|B|z'
       call grfx1d ( x, prvar(1,5), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 2, 3, 2, xlab, ylab )
c
c      Plot y-magnetic field.
c
       ylab = '|F10|B|B|y'
       call grfx1d ( x, prvar(1,6), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 3, 3, 2, xlab, ylab )
c
c      Plot z-magnetic field.
c
       ylab = '|F10|B|B|z'
       call grfx1d ( x, prvar(1,7), 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 1, 3, 3, xlab, ylab )
c
c      Plot magnetosonic speed.
c
       ylab = '|F10|a|B|s'
       if ( (family .eq. 1) .or. (family .eq. 7) ) ylab = '|F10|a|B|f'
       call grfx1d ( x, amag, 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 2, 3, 3, xlab, ylab )
c
c      Plot alphap.
c
       ylab = '|F5|a|BF18|A'
       call grfx1d ( x, alfp, 0, 1, ntot, 1, 1, 0.0, 0.0, 0.0, 0.0
     1             , 2, 0.9, 1, 2, 3, 3, 3, 3, xlab, ylab )
c
c      Close graphics file.
c
       call grfxcls
       write ( iotty, 2010 ) filename, ntot
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-------------0----+----1----+----2----+----3----+----4----+----5----+--
c
2010   format('PLOTIT  : PSPLOT output file ',a,' written to disc with '
     1       ,i4,' points per plot.')
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                P L O T I T                \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                    R K                    \\\\\\\\\\
c
c=======================================================================
c
       subroutine rk ( yl, yr, node, h, dpl, delp )
c
c    dac:fans.rk <----------------- performs the Runge-kutta integration
c                                                         november, 2018
c
c    written by: David Clarke
c    modified 1: February 2019, by DAC; added tracking of accumulated
c                deviation from critical pressure
c
c  PURPOSE:  This routine performs a single step of the 6th order Runge-
c  Kutta algorithm.  For this application, the independent variable is
c  not needed.
c
c  INPUT VARIABLES:
c    yl         dependent variables at the left side of the interval
c    node       dimension of "yl"
c    h          step size
c    dpl        accumulated pressure difference from the critical
c               pressure, "pcr", at left side of interval
c
c  OUTPUT VARIABLES:
c    yr         dependent variables at the right side of the interval
c    delp       amount by which pressure changed across interval which
c               calling routine uses to update accumulated difference
c               between pressure and "pcr".  Used by SPEEDS to calculate
c               "deltax" should "alphax" gets too close to 1.
c
c  LOCAL VARIABLES:
c    fl     |
c    fhtwid |__ temporary 1-D arrays dimensioned to "node" for storing
c    fh     |   the "node" derivatives.
c    f1twid |
c
c  EXTERNALS:
c    DERIVS
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       integer       node    , n
       real*8        h       , dpl     , dpr     , delp    , halfh
 
       real*8        yl      (node), yr      (node), fl      (node)
     1             , fhtwid  (node), fh      (node), f1twid  (node)
c
       external      derivs
c
c-----------------------------------------------------------------------
c
       halfh = 0.5d0 * h
c
c      Evaluate derivatives and calculate anew "yr" at each quarter
c  step.  "dpr" is the accumulated pressure difference from the critical
c  pressure, "pcr", up to and including the right side of the quarter
c  step, and is used by the next call to DERIVS-->SPEEDS to calculate
c  "deltax" should "alphax" gets too close to 1.
c
       call derivs ( yl, dpl, fl )
       do 10 n=1,node
         yr(n) = yl(n) + halfh * fl(n)
10     continue
       dpr = dpl + halfh * fl(2)
c
       call derivs ( yr, dpr, fhtwid )
       do 20 n=1,node
         yr(n) = yl(n) + halfh * fhtwid(n)
20     continue
       dpr = dpl + halfh * fhtwid(2)
c
       call derivs ( yr, dpr, fh )
       do 30 n=1,node
         yr(n) = yl(n) + h * fh(n)
30     continue
       dpr = dpl + h * fh(2)
c
       call derivs ( yr, dpr, f1twid )
       do 40 n=1,node
         yr(n) = yl(n) + h * ( fl(n) + 2.0d0 * ( fhtwid(n) + fh(n) )
     1                       + f1twid(n) ) / 6.0d0
40     continue
       delp = h * ( fl(2) + 2.0d0 * ( fhtwid(2) + fh(2) ) + f1twid(2) )
     1      / 6.0d0
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                    R K                    \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                S P E E D S                \\\\\\\\\\
c
c=======================================================================
c
       subroutine speeds ( var, dpacc )
c
c    dac:fans.speeds <-------------- computes speeds and alphas in state
c                                                         november, 2018
c
c    written by: David Clarke
c    modified 1: January, 2019 by DAC; reworked to include MHD alphas.
c
c  PURPOSE: This routine computes the speed-related quantities needed
c  for DERIVS, with full machine precision by avoiding differences of
c  nearly equal numbers.
c
c  1.  To avoid the difference in the slow alpha, "alphas", it is
c      computed as alphax/alphaf.  This requires there to be reliable
c      values for both "alphax" and "alphaf", which there are.
c
c  2.  Since deltax = alphax - 1, it is dominated by round-off error
c      when alphax -> 1.  Thus, if alphax ~ 1, an alternative expression
c      for "deltax" is used based on a binomial expansion on dpacc/pcr,
c      where "dpacc" is the accumulated difference between the current
c      pressure and the critical pressure, pcr = bx0**2/gamma.  See
c      notes "deltax without machine round-off error", January 2019.
c
c  INPUT VARIABLES:
c    var         array dimensioned "node" with seven primitive variable
c                values (rho, p, vx, vy, vz, by, bz); bx0 is in common.
c    dpacc       accumulated difference between pressure and "pcr".
c
c  OUTPUT VARIABLES: (all in common block comvarr)
c    alphap      apsq / cssq
c    deltax      alphax - 1 = axsq / cssq - 1
c    af          fast speed
c    as          slow speed
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [none]
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c-----------------------------------------------------------------------
c----------------------------- PARAMETERS ------------------------------
c-----------------------------------------------------------------------
c
c  pvar    number of primitive variables (7 for regular 1-D MHD)
c  imax    maximum number of points at which RF is evaluated.
c  emin    minimum value of "eps" before binomial expansions are used.
c  zero    what, to round-off error, should be interpreted as 0.
c  tiny    a small number
c  huge    a large number
c
       integer       pvar,     imax
       real*8        emin    , zero    , tiny    , huge
c
       parameter     ( pvar = 7, imax = 10000 )
       parameter     ( emin = 1.0d-5, zero = 1.0d-15, tiny = 1.0d-99
     1               , huge = 1.0d+99 )
c-----------------------------------------------------------------------
c--------------------------- ROOT VARIABLES ----------------------------
c-----------------------------------------------------------------------
c
       character*1   blank
       integer       iotty   , ioin    , ioplt   , family
       real*8        bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
c
       real*8        y0      (pvar)
c
       common      / comvarc  /
     1               blank
c
       common      / comvari  /
     1               iotty   , ioin    , ioplt   , family
c
       common      / comvarr  /
     1               bx0     , sgnbx0  , gammai  , gamma   , gamm1
     1             , slim    , alphap  , deltax  , af      , as
     1             , coschi  , sinchi  , dp0     , dptot
     1             , y0
c
       real*8        dpacc   , alphax  , eps     , disc    , alphaf
     1             , alphas  , cssq
c
       real*8        var     (   *)
c
c-----------------------------------------------------------------------
c
c      Compute "alphap" and "alphax", then calculate deltax = alphax - 1
c  directly, or by a three-term binomial expansion of eps = dpacc/pcr
c  should eps < emin.
c
       alphap = ( var(6)**2 + var(7)**2 ) * gammai / var(2)
       alphax = bx0**2 * gammai / var(2)
       if (abs(alphax-1.0d0) .lt. zero) alphax = 1.0d0
       deltax = alphax - 1.0d0
       if (bx0 .ne. 0.0d0) then
         eps = dpacc * gamma / bx0**2
         if (abs(eps) .lt. emin) then
           deltax =-eps + eps**2 - eps**3
           alphax = 1.0d0 + deltax
         endif
       endif
c
c      Compute the fast and slow speeds, "af" and "as".
c
       disc   = sqrt ( ( deltax + alphap )**2 + 4.0d0 * alphap )
       alphaf = 1.0d0 + 0.5d0 * ( deltax + alphap + disc )
       alphas = alphax / alphaf
       cssq   = gamma * var(2) / var(1)
       af     = sqrt ( alphaf * cssq )
       as     = sqrt ( alphas * cssq )
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                S P E E D S                \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 1             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac01 ( nmlst, nmax )
c
c    dac:namelist.nlsdac01 <----------------------- reset namelist array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine initialises all the elements of the array
c  "nmlst" to 32 character blanks words.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*32  blank32
       integer       nmax    , i
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank32 /'                                '/
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         nmlst(1,i) = blank32
         nmlst(2,i) = blank32
10     continue
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 1             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 2             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac02 ( nlunit, cname, nmlst, nmax, ival )
c
c    dac:namelist.nlsdac02 <----------------- reads input character data
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February, 1991 by David Clarke; Fixed various bugs
c                  discovered in the previous 20 months of use.
c                  Character strings of indefinite length can now be
c                  read.
c      Modified 2: January, 1993 by David Clarke; Generalised array
c                  reader so that ar(1,1) is understood as ar(1:1,1:1).
c
c  PURPOSE:  This subroutine reads the input decks originally designed
c  for CFT "namelist" (under CTSS), and stores the character
c  representations of the input data in "nmlst(n,2)".  Meanwhile,
c  "nmlst(n,1)" contains a list of the corresponding variable names,
c  which were set by the calling program.  Other subroutines (called by
c  the calling program) convert the character strings to the correct
c  attribute.
c
c  EXTERNALS:  NLSDAC15 - 23
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1   , char2   , char3
       character*8   cname   , word8
       character*32  blank32 , word32  , temp32
       character*72  line    , oldline
       integer       nlunit  , nmax    , ival    , nlist   , jbeg
     1             , jend    , jran    , jbegold , jendold , i
     2             , i1      , i2      , ilist   , ilisto  , match
     3             , iqte
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data blank32 /'                                '/
c
c      External statements
c
       external      nlsdac15, nlsdac16, nlsdac17, nlsdac18, nlsdac19
     1             , nlsdac20, nlsdac21, nlsdac22, nlsdac23
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.  Note that all the second elements of the
c  array "nmlst" (nmlst(2,*)) and all but the first "ival" of the first
c  elements (nmlst(1,*)) are reset to 32 character blanks words.  This
c  is to allow for the possibility that the READ statement appears in a
c  loop, in which case, most of "nmlst" must be reset.
c
       do 10 i=1,nmax
         nmlst(2,i) = blank32
10     continue
       if (ival .ge. nmax) return
       do 20 i=ival+1,nmax
         nmlst(1,i) = blank32
20     continue
       nlist = ival
       iqte  = 0
30     continue
c
c      Read line from input deck.
c
       read (nlunit, 1010, end=150, err=160) line
c
c      Error message if column 1 contains anything but a 'c', 'C', or a
c  blank.
c
       char2 = line(1:1)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char2 .ne. ' ') ) call nlsdac15 ( line, 1, 1 )
c
c      Error message if column 2 contains anything but a '$' or a blank
c  (unless column 1 contains a 'c' or a 'C').
c
       char3 = line(2:2)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char3 .ne. '$') .and. (char3 .ne. ' ') )
     2   call nlsdac15 ( line, 2, 2 )
c
c      If column 1 contains a 'c' or a 'C', the line is "commented out",
c  and is echoed on the terminal.
c
       if ( (char2 .eq. 'c') .or. (char2 .eq. 'C') ) then
         write (6, 2010) line
         go to 30
       endif
c
c      If column 2 contains a '$' sentinal, then a new namelist begins.
c  Determine if name of namelist matches "cname", and if it does, read
c  the contents of the namelist.  The next '$' sentinal will close the
c  namelist.
c
       if (char3 .eq. '$') then
         jbeg = 3
         call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
         if (char1 .eq. '$') go to 30
         call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte, word32 )
         word8 = word32(1:8)
         if (word8 .eq. cname) then
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .eq. '$') return
40         continue
           i1 = 0
           i2 = 0
c
c      Read the next specified variable.
c
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           do 50 i=1,nmax
             ilist  = i
             temp32 = blank32
             temp32(1:8) = nmlst(1,ilist)(1:8)
             if (temp32 .eq. blank32) go to 60
             if (temp32 .eq.  word32) then
               if (nmlst(2,ilist) .ne. blank32) then
                 temp32 = nmlst(1,ilist)
                 nlist  = nlist + 1
                 ilist  = nlist
                 nmlst(1,ilist)( 1: 8) = temp32( 1: 8)
                 nmlst(1,ilist)(25:32) = temp32(25:32)
               endif
               go to 70
             endif
50         continue
c
c      Error:  variable specified in input deck was not included in
c  "nmlst".
c
60         continue
           call nlsdac15 ( line, jbeg, 3 )
c
c      Specified variable found in "nmlst".  Read the next character.
c  If "char1" .ne. '(' or '=', issue error message.  If "char1" = '(', a
c  portion of an array will be set.  If "char1" = '=', the variable
c  value will be read next.
c
70         continue
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. '(') match = 2
           if (char1 .eq. '=') match = 3
           go to (80, 90, 130) match
c
c      Error: unexpected character.
c
80         continue
           call nlsdac15 ( line, jbeg, 4 )
c
c      Specify portion of vector or array to be set.  The limits are
c  stored in columns 10 through 28  of "nmlst(1,ilist)", and are used by
c  other subroutines to initiate the variable.
c
90         continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           call nlsdac19 ( word32, i1 )
           i2 = i1
           nmlst(1,ilist)(9:12) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. ')') match = 2
           if (char1 .eq. ':') match = 3
           if (char1 .eq. ',') match = 4
           go to (80, 100, 110, 111) match
100        continue
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .ne. '=') go to 80
           go to 130
110        continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           nmlst(1,ilist)(13:16) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. ')') match = 2
           if (char1 .eq. ',') match = 3
           go to (80, 100, 120) match
111        continue
           nmlst(1,ilist)(13:16) = word32(1:4)
120        continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           nmlst(1,ilist)(17:20) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. ':') match = 2
           if (char1 .eq. ')') match = 3
           go to (80, 121, 122) match
121        continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           nmlst(1,ilist)(21:24) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. ')') match = 2
           go to (80, 123) match
122        continue
           nmlst(1,ilist)(21:24) = word32(1:4)
123        continue
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .ne. '=') go to 80
c
c      Read the character representation of the variable value.
c
130        continue
           jbeg    = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           jbegold = jbeg
           jendold = jend
           oldline = line
           jbeg    = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .eq. '*') then
             char1 = nmlst(1,ilist)(32:32)
             if (char1 .ne. 'v') call nlsdac15 ( line, jbeg, 8 )
             if (i1 .eq. 0) i1 = 1
             call nlsdac18 ( line, jbeg, word32 )
             call nlsdac19 ( word32, i2 )
             i2 = i1 + i2 - 1
             call nlsdac20 ( i2, word32 )
             nmlst(1,ilist)(13:16) = word32(1:4)
             call nlsdac20 ( i1, word32 )
             nmlst(1,ilist)( 9:12) = word32(1:4)
             jbeg = jend + 1
             call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                     , word32 )
           else
             jbeg = jbegold
             jend = jendold
             if (line .ne. oldline) then
               backspace (nlunit)
               line = oldline
             endif
           endif
           ilisto = ilist
           char1  = nmlst(1,ilist)(30:30)
           if (char1 .eq. 'l') call nlsdac22 ( line, jbeg, word32 )
           if (char1 .eq. 'i') call nlsdac18 ( line, jbeg, word32 )
           if (char1 .eq. 'r') call nlsdac23 ( line, jbeg, word32 )
           if (char1 .eq. 'c') then
c
c      Single or double quotes must surround each character string.  If
c  a second quote is not found, it will be assumed that the character
c  variable is being assigned a string longer than 30 characters.  In
c  this case, multiple entries are made in "nmlst(1,ilist)" for the
c  character variable, with successive pieces of the assigned value
c  stored in "nmlst(2,ilist)".  Continuation characters (&) are
c  appended to each piece of the input character value to tell the
c  subroutines which set the values of character variables that each
c  entry is to be concatenated, rather than overwriting the previous
c  entry.
c
             char1 = word32(1:1)
             if ( (char1 .ne. '"') .and. (char1 .ne. '''') )
     1         call nlsdac15 ( line, jbeg, 10 )
             temp32           = blank32
             temp32(1:jran-1) = word32(2:jran)
             word32           = temp32
             jran             = jran - 1
140          continue
             char1 = word32(jran:jran)
             if ( (char1 .ne. '"') .and. (char1 .ne. '''') ) then
               char1 = nmlst(1,ilist)(32:32)
               if ( (char1 .eq. 'v') .and. (i1 .eq. 0) ) then
                 nmlst(1,ilist)(9:9) = '1'
                 i1 = 1
               endif
               if (jran .le. 30) then
                 jbeg = - jend - 1
               endif
               if (jran .eq. 31) then
                 jran = 30
                 jbeg = - jend
               endif
               if (jran .eq. 32) then
                 jran = 30
                 jbeg = - jend + 1
               endif
               word32 (31:31) = char(jran)
               word32 (32:32) = '&'
               nmlst(2,ilist) = word32
               word32         = nmlst(1,ilist)
               nlist          = nlist + 1
               ilist          = nlist
               nmlst(1,ilist) = word32
               call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                       , word32 )
               go to 140
             endif
             temp32           = blank32
             temp32(1:jran-1) = word32(1:jran-1)
             word32           = temp32
           endif
           nmlst(2,ilist) = word32
           ilist          = ilisto
c
c      Make sure next character is a comma or a dollar sign.
c
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if ( (char1 .ne. ',') .and. (char1 .ne. '$') ) go to 80
           jend = jbeg
c
c      If the variable is a vector, check to see if there are more data
c  (a propos for the syntax: iplot=1,1,1,2,0).
c
           if (nmlst(1,ilist)(32:32) .eq. 'v') then
             if (i1 .eq. 0) then
               nmlst(1,ilist)(9:9) = '1'
               i1 = 1
             endif
             if (i2 .eq. 0) i2 = i1
             if (char1 .eq. '$') return
             jbegold = jbeg
             jendold = jend
             oldline = line
             jbeg    = jend + 1
             call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                     , word32 )
             char2 = word32(1:1)
             jbeg  = jend + 1
             call nlsdac17 ( nlunit, line, jbeg, jend, char3 )
             if ( (char3 .eq. ',') .or. (char3 .eq. '$') .or.
     1            (char3 .eq. '*') .or. (char2 .eq. '"') .or.
     2            (char2 .eq. '''') ) then
               word32 = nmlst(1,ilist)
               nlist  = nlist + 1
               ilist  = nlist
               nmlst(1,ilist)( 1: 8) = word32( 1: 8)
               nmlst(1,ilist)(25:32) = word32(25:32)
               i1 = i2 + 1
               i2 = i1
               call nlsdac20 ( i1, word32 )
               nmlst(1,ilist)( 9:12) = word32(1:4)
             endif
             jbeg = jbegold
             jend = jendold
             if (line .ne. oldline) then
               backspace (nlunit)
               line = oldline
             endif
             if ( (char3 .eq. ',') .or. (char3 .eq. '$') .or.
     1            (char3 .eq. '*') .or. (char2 .eq. '"') .or.
     2            (char2 .eq. '''') ) go to 130
           endif
c
c     Go on to next variable in "line".
c
           if (char1 .eq. '$') return
           go to 40
         endif
       endif
c
c      No match for "cname" in current "line".  Read next "line".
c
       go to 30
c
c      Error:  "cname" does not exist, or was out of order and thus
c  skipped over.
c
150    continue
       call nlsdac15 ( cname, 0, 6 )
c
c      Error:  Problem reading input deck.
c
160    continue
       call nlsdac15 ( blank32, 0, 7 )
c
c-----------------------------------------------------------------------
c----------------------- Read format statements ------------------------
c-----------------------------------------------------------------------
c
1010   format(a72)
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(a72)
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 2             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 3             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac03 ( cscalar, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac03 <--------------------- fills character scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February 1991, by David Clarke; rewritten to allow
c                  character strings to be specified in pieces.
c
c  PURPOSE:  This subroutine assigns the input value to a character
c  scalar.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  blank32 , word32
       character*(*) cscalar
       integer       nmax    , i       , ibeg    , mult    , lbeg
     1             , lend    , lran
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c-----------------------------------------------------------------------
c
       mult = 0
       ibeg = 1
       lbeg = 1
10     continue
c
c      Find next entry in "nmlst(1,*)" which matches the name of the
c  variable to be set ("cstring").
c
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. cstring) go to 30
         if (word8 .eq. blank8 ) return
20     continue
       return
30     continue
       word32 = nmlst(2,i)
c
c      If character string is longer than 30 characters (final character
c  is '&'), it has been stored in "nmlst" in pieces.  Thus, set the next
c  piece of "cscalar" to the contents of "nmlst(2,i)", and then go back
c  to read the next piece.
c
       if (word32(32:32) .eq. '&') then
         mult = 1
         lran = ichar ( word32(31:31) )
         lend = lbeg + lran - 1
         cscalar(lbeg:lend) = word32(1:lran)
         lbeg = lend + 1
         ibeg = i + 1
         go to 10
       endif
c
c      If the character string was stored in pieces, "mult" will have
c  been set to 1.  If final character is not a '&', then this is the
c  last piece to read in.  Do not include trailing blanks in "cscalar".
c
c      If "mult" is 0, then the character string is stored in its
c  entirety in "nmlst(2,i)".  Thus, set cscalar trivially.
c
       if (mult .eq. 1) then
         do 60 lran=31,1,-1
           if (word32(lran:lran) .ne. ' ') go to 70
60       continue
70       continue
         lend = lbeg + lran - 1
         cscalar(lbeg:lend) = word32(1:lran)
       else
c
c      See comments from DAC, Jan/08 and Apr/08 in NLSDAC07.
c
         if (word32 .ne. blank32) cscalar = word32
c        cscalar = word32
       endif
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 3             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 4             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac04 ( iscalar, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac04 <----------------------- fills integer scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input value to an integer
c  scalar.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  blank32 , word32
       integer       iscalar , nmax    , i
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. cstring) go to 20
         if (word8 .eq. blank8 ) return
10     continue
       return
20     continue
       word32 = nmlst(2,i)
       if (word32 .ne. blank32) call nlsdac19 ( word32, iscalar )
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 4             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 5             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac05 ( rscalar, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac05 <-------------------------- fills real scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input value to a real scalar.
c
c  EXTERNALS:  NLSDAC21
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  blank32 , word32
       integer       nmax    , i
       real*8        rscalar
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac21
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. cstring) go to 20
         if (word8 .eq. blank8 ) return
10     continue
       return
20     continue
       word32 = nmlst(2,i)
       if (word32 .ne. blank32) call nlsdac21 ( word32, rscalar )
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 5             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 6             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac06 ( lscalar, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac06 <----------------------- fills logical scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input value to a logical
c  scalar.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lscalar
       character*8   cstring , word8   , blank8
       integer       nmax    , i
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. cstring) go to 20
         if (word8 .eq. blank8 ) return
10     continue
       return
20     continue
       word8 = nmlst(2,i)(1:8)
       if (word8 .eq. '.true.  ') lscalar = .true.
       if (word8 .eq. '.t.     ') lscalar = .true.
       if (word8 .eq. '.false. ') lscalar = .false.
       if (word8 .eq. '.f.     ') lscalar = .false.
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 6             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 7             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac07 ( cvector, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac07 <--------------------- fills character vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February 1991, by David Clarke; rewritten to allow
c                  character strings to be specified in pieces.
c
c  PURPOSE:  This subroutine assigns the input values to a character
c  vector.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*12  cstr12  , word12
       character*32  word32  , blank32
       integer       nmax    , mult    , i       , ibeg    , j
     1             , jbeg    , jend    , lbeg    , lend    , lran
c
       character*(*) cvector(*)
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       lbeg = 1
       mult = 0
c
c      Find next entry in "nmlst(1,*)" which matches the name of the
c  variable to be set ("cstring").
c
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. cstring) go to 30
         if (word8 .eq. blank8 ) go to 999
20     continue
30     continue
       cstr12 = nmlst(1,i)(1:12)
c
c      Set range of variable index (jbeg through jend) to be set to
c  what will be stored in "cvector(jbeg)".
c
       word32      = blank32
       word32(1:4) = nmlst(1,i)(9:12)
c
c      Note from DAC, Apr/08: Added conditional go to 120 when jbeg = 0,
c  which will trip the `array bound checking' (-C) in loop 110 below.
c  If jbeg=0, the array isn't being set anyway, so skipping on to line
c  120 should be innocuous.
c
       if (word32 .eq. blank32) go to 120
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
c
c      If character string is longer than 30 characters (final character
c  is '&'), it has been stored in "nmlst" in pieces.  Thus, set the next
c  piece of "cvector(jbeg)" to the contents of "nmlst(2,i)", and then
c  read the next piece.
c
40     continue
       word32 = nmlst(2,i)
       if (word32(32:32) .eq. '&') then
         mult = 1
         lran = ichar ( word32(31:31) )
         lend = lbeg + lran - 1
         cvector(jbeg)(lbeg:lend) = word32(1:lran)
         lbeg = lend + 1
         ibeg = i + 1
         do 70 i=ibeg,nmax
           word12 = nmlst(1,i)(1:12)
           if (word12 .eq. cstr12 ) go to 80
70       continue
80       continue
         go to 40
       endif
c
c      If the character string was stored in pieces, "mult" will have
c  been set to 1.  If final character is not a '&', then this is the
c  last piece to read in.  Do not include trailing blanks in
c  "cvector(jbeg)".
c
c      If "mult" is 0, then the character string is stored in its
c  entirety in "nmlst(2,i)".  Thus, set cvector(jbeg) trivially.
c
       if (mult .eq. 1) then
         do 90 lran=31,1,-1
           if (word32(lran:lran) .ne. ' ') go to 100
90       continue
100      continue
         lend = lbeg + lran - 1
         cvector(jbeg)(lbeg:lend) = word32(1:lran)
       else
c
c      Comment from DAC, Jan/08: the line commented out below prevented
c  character strings from being set to blanks in namelists, such as
c  p1ylab=' ' in namelist plt1con so that no y-label is put on the plot
c  (but annotations remain).  I cannot recall at all why "cvector" was
c  protected from being set to blank.
c
c      Answer Apr/08: It's to prevent defaults from being clobbered by
c  blanks if the namelist variable is not set.  The original intent of a
c  blank in "nmlst(2,i)" was to indicate that the variable given in
c  "nmlst(1,i)" is not being set, and therefore the default shouldn't be
c  overwritten with a blank.  This means character variables cannot be
c  set to a blank, which is a problem.
c
c      Comment from DAC, Apr/08: If the variable is not being set,
c  jbeg=0 and this portion of the routine is skipped.  Thus, if
c  execution gets here, the variable is to be set, and there is no
c  default value to protect.  Thus, the original first line is commented
c  out, and the second "unprotected" line is revealed.  This tact should
c  work in NLSDAC11 (character arrays) but won't for NLSDAC03 (character
c  scalars) since the latter have no "jbeg" to set.
c
c        if (word32 .ne. blank32) cvector(jbeg) = word32
         cvector(jbeg) = word32
       endif
c
c      Set desired range of "cvector" to the desired constant.
c
       do 110 j=jbeg,jend
         cvector(j) = cvector(jbeg)
110    continue
c
c      Search for next range of elements in "cvector" to be set.
c
120    continue
       ibeg = i + 1
       if (ibeg .gt. nmax) go to 999
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 7             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 8             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac08 ( ivector, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac08 <----------------------- fills integer vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to an integer
c  vector.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , i       , ibeg    , j       , jbeg
     1             , jend    , ival
c
       integer       ivector (   *)
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. blank8) go to 999
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = nmlst(2,i)
       if (word32 .eq. blank32) go to 999
       ibeg = i + 1
       if (ibeg .gt. nmax+1) go to 999
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
       call nlsdac19 ( nmlst(2,i), ival )
       do 40 j=jbeg,jend
         ivector(j) = ival
40     continue
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 8             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 9             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac09 ( rvector, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac09 <-------------------------- fills real vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a real vector.
c
c  EXTERNALS:  NLSDAC19, NLSDAC21
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , i       , ibeg    , j       , jbeg
     1             , jend
       real*8        rval
c
       real*8        rvector (   *)
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19, nlsdac21
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. blank8) go to 999
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = nmlst(2,i)
       if (word32 .eq. blank32) go to 999
       ibeg = i + 1
       if (ibeg .gt. nmax+1) go to 999
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
       call nlsdac21 ( nmlst(2,i), rval )
       do 40 j=jbeg,jend
         rvector(j) = rval
40     continue
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 9             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 0             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac10 ( lvector, cstring, nmlst, nmax )
c
c    dac:namelist.nlsdac10 <----------------------- fills logical vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a logical
c  vector.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lval
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , i       , ibeg    , j       , jbeg
     1             , jend
c
       logical       lvector (   *)
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. blank8) go to 999
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = nmlst(2,i)
       if (word32 .eq. blank32) go to 999
       ibeg = i + 1
       if (ibeg .gt. nmax+1) go to 999
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
       word8 = nmlst(2,i)(1:8)
       if (word8 .eq. '.true.  ') lval = .true.
       if (word8 .eq. '.t.     ') lval = .true.
       if (word8 .eq. '.false. ') lval = .false.
       if (word8 .eq. '.f.     ') lval = .false.
       do 40 j=jbeg,jend
         lvector(j) = lval
40     continue
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 0             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 1             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac11 ( carray, cstring, nmlst, nmax, nx )
c
c    dac:namelist.nlsdac11 <------ fills two-dimensional character array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February 1991, by David Clarke; rewritten to allow
c                  character strings to be specified in pieces.
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional character array.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*24  cstr24  , word24
       character*32  word32  , blank32
       integer       nmax    , nx      , mult    , i       , ibeg
     1             , j       , jbeg    , jend    , k       , kbeg
     2             , kend    , lbeg    , lend    , lran
c
       character*32  nmlst   (   2,   *)
       character*(*) carray  (  nx,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg  = 1
10     continue
       lbeg  = 1
       mult  = 0
c
c      Find next entry in "nmlst(1,*)" which matches the name of the
c  variable to be set ("cstring").
c
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. cstring) go to 30
         if (word8 .eq. blank8 ) go to 999
20     continue
30     continue
       cstr24 = nmlst(1,i)(1:24)
c
c      Set range of variable index (jbeg through jend; kbeg through
c  kend) to be set to what will be stored in "carray(jbeg,kbeg)".
c
c      See comments from DAC, Apr/08 in NLSDAC07.
c
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       if (word32 .eq. blank32) go to 130
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       call nlsdac19 ( word32, jend )
       word32(1:4) = nmlst(1,i)(17:20)
       if (word32 .eq. blank32) go to 130
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = nmlst(1,i)(21:24)
       call nlsdac19 ( word32, kend )
c
c      If character string is longer than 30 characters (final character
c  is '&'), it has been stored in "nmlst" in pieces.  Thus, set the next
c  piece of "carray(jbeg,kbeg)" to the contents of "nmlst(2,i)", and
c  then read the next piece.
c
40     continue
       word32 = nmlst(2,i)
       if (word32(32:32) .eq. '&') then
         mult = 1
         lran = ichar ( word32(31:31) )
         lend = lbeg + lran - 1
         carray(jbeg,kbeg)(lbeg:lend) = word32(1:lran)
         lbeg = lend + 1
         ibeg = i + 1
         do 70 i=ibeg,nmax
           word24 = nmlst(1,i)(1:24)
           if (word24 .eq. cstr24 ) go to 80
70       continue
80       continue
         go to 40
       endif
c
c      If the character string was stored in pieces, "mult" will have
c  been set to 1.  If final character is not a '&', then this is the
c  last piece to read in.  Do not include trailing blanks in
c  "carray(jbeg,kbeg)".
c
c      If "mult" is 0, then the character string is stored in its
c  entirety in "nmlst(2,i)".  Thus, set carray(jbeg,kbeg) trivially.
c
       if (mult .eq. 1) then
         do 90 lran=31,1,-1
           if (word32(lran:lran) .ne. ' ') go to 100
90       continue
100      continue
         lend = lbeg + lran - 1
         carray(jbeg,kbeg)(lbeg:lend) = word32(1:lran)
       else
c
c      See comments from DAC, Jan/08 and Apr/08 in NLSDAC07.
c
c        if (word32 .ne. blank32) carray(jbeg,kbeg) = word32
         carray(jbeg,kbeg) = word32
       endif
c
c      Set desired range of "carray" to the desired constant.
c
       do 120 k=kbeg,kend
         do 110 j=jbeg,jend
           carray(j,k) = carray(jbeg,kbeg)
110      continue
120    continue
c
c      Search for next range of elements in "carray" to be set.
c
130    continue
       ibeg = i + 1
       if (ibeg .gt. nmax) go to 999
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 1             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 2             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac12 ( iarray, cstring, nmlst, nmax, nx )
c
c    dac:namelist.nlsdac12 <-------- fills two-dimensional integer array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional integer array.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , nx      , i       , ibeg    , j
     1             , jbeg    , jend    , k       , kbeg    , kend
     2             , ival
c
       character*32  nmlst   (   2,   *)
       integer       iarray  (  nx,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. blank8) go to 999
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = nmlst(2,i)
       if (word32 .eq. blank32) go to 999
       ibeg = i + 1
       if (ibeg .gt. nmax+1) go to 999
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       call nlsdac19 (word32, jend )
       word32(1:4) = nmlst(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = nmlst(1,i)(21:24)
       call nlsdac19 ( word32, kend )
       call nlsdac19 ( nmlst(2,i), ival )
       do 50 k=kbeg,kend
         do 40 j=jbeg,jend
           iarray(j,k) = ival
40       continue
50     continue
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 2             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 3             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac13 ( rarray, cstring, nmlst, nmax, nx )
c
c    dac:namelist.nlsdac13 <----------- fills two-dimensional real array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional real array.
c
c  EXTERNALS:  NLSDAC19, NLSDAC21
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , nx      , i       , ibeg    , j
     1             , jbeg    , jend    , k       , kbeg    , kend
       real*8        rval
c
       character*32  nmlst   (   2,   *)
       real*8        rarray  (  nx,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19, nlsdac21
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. blank8) go to 999
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = nmlst(2,i)
       if (word32 .eq. blank32) go to 999
       ibeg = i + 1
       if (ibeg .gt. nmax+1) go to 999
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       call nlsdac19 ( word32, jend )
       word32(1:4) = nmlst(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = nmlst(1,i)(21:24)
       call nlsdac19 ( word32, kend )
       call nlsdac21 ( nmlst(2,i), rval )
       do 50 k=kbeg,kend
         do 40 j=jbeg,jend
           rarray(j,k) = rval
40       continue
50     continue
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 3             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 4             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac14 ( larray, cstring, nmlst, nmax, nx )
c
c    dac:namelist.nlsdac14 <-------- fills two-dimensional logical array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional logical array.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lval
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , nx      , i       , ibeg    , j
     1             , jbeg    , jend    , k       , kbeg    , kend
c
       logical       larray  (  nx,   *)
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = nmlst(1,i)(1:8)
         if (word8 .eq. blank8) go to 999
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = nmlst(2,i)
       if (word32 .eq. blank32) go to 999
       ibeg = i + 1
       if (ibeg .gt. nmax+1) go to 999
       word32 = blank32
       word32(1:4) = nmlst(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = nmlst(1,i)(13:16)
       call nlsdac19 ( word32, jend )
       word32(1:4) = nmlst(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = nmlst(1,i)(21:24)
       call nlsdac19 ( word32, kend )
       word8 = nmlst(2,i)(1:8)
       if (word8 .eq. '.true.  ') lval = .true.
       if (word8 .eq. '.t.     ') lval = .true.
       if (word8 .eq. '.false. ') lval = .false.
       if (word8 .eq. '.f.     ') lval = .false.
       do 50 k=kbeg,kend
         do 40 j=jbeg,jend
           larray(j,k) = lval
40       continue
50     continue
       go to 10
c
999    continue
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 4             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 5             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac15 ( phrase, ipoint, ierror )
c
c    dac:namelist.nlsdac15 <---------------------- issues error messages
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: May, 2013 by DAC; corrected a long-standing format
c                  error (0x is not a legal descriptor, at least for
c                  gfortran; who knew!)
c
c  PURPOSE:  This subroutine is called when an error is detected in the
c  namelist by one of the subroutines.  The appropriate error message
c  is issued to the CRT, and execution is terminated.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) phrase
       character*9   frmt
       integer       ipoint  , ierror
c
c-----------------------------------------------------------------------
c
       write (   6, 2010)
       write (   6, 2020) phrase
       if (ipoint .eq. 1) write ( 6, 2021 )
       if (ipoint .gt. 1) then
         write (frmt, 2030) ipoint - 1
         write (   6, frmt)
       endif
c
       if (ierror .eq.  1) write (6,2040)
       if (ierror .eq.  2) write (6,2050)
       if (ierror .eq.  3) write (6,2060)
       if (ierror .eq.  4) write (6,2070)
       if (ierror .eq.  5) write (6,2080)
       if (ierror .eq.  6) write (6,2090)
       if (ierror .eq.  7) write (6,2100)
       if (ierror .eq.  8) write (6,2110)
       if (ierror .eq.  9) write (6,2120)
       if (ierror .eq. 10) write (6,2130)
       if (ierror .eq. 11) write (6,2140)
       if (ierror .eq. 12) write (6,2150)
       if (ierror .eq. 13) write (6,2160)
       if (ierror .eq. 14) write (6,2170)
       if (ierror .eq. 15) write (6,2180)
       if (ierror .eq. 16) write (6,2190)
       if (ierror .eq. 17) write (6,2200)
       if (ierror .eq. 18) write (6,2210)
       if (ierror .eq. 19) write (6,2220)
       if (ierror .eq. 20) write (6,2230)
c
       write (6,2010)
       stop
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format('************************************'
     1       ,'************************************')
2020   format( (a) )
2021   format('^')
2030   format("(",i2.2,"x,'^')")
2040   format('NAMELIST ERROR  1 ---> column 1 reserved for comment '
     1       ,'sentinal: c')
2050   format('NAMELIST ERROR  2 ---> column 2 reserved for $ '
     1       ,'delimeter')
2060   format('NAMELIST ERROR  3 ---> variable not found in namelist')
2070   format('NAMELIST ERROR  4 ---> unexpected character - check '
     1       ,'syntax')
2080   format('NAMELIST ERROR  5 ---> invalid logical expression')
2090   format('NAMELIST ERROR  6 ---> namelist does not exist, or is '
     1       ,'out of sequence')
2100   format('NAMELIST ERROR  7 ---> error reading input deck')
2110   format('NAMELIST ERROR  8 ---> variable not declared as a '
     1       ,'vector')
2120   format('NAMELIST ERROR  9 ---> next namelist begun before '
     1       ,'closing quote found')
2130   format('NAMELIST ERROR 10 ---> missing opening quote')
2140   format('NAMELIST ERROR 11 ---> blank data')
2150   format('NAMELIST ERROR 12 ---> premature end of file')
2160   format('NAMELIST ERROR 13 ---> missing $ sentinal to close '
     1       ,'namelist')
2170   format('NAMELIST ERROR 14 ---> exponent must not exceed 999')
2180   format('NAMELIST ERROR 15 ---> no more than 15 digits in single '
     1       ,'precision')
2190   format('NAMELIST ERROR 16 ---> ')
2200   format('NAMELIST ERROR 17 ---> ')
2210   format('NAMELIST ERROR 18 ---> ')
2220   format('NAMELIST ERROR 19 ---> ')
2230   format('NAMELIST ERROR 20 ---> ')
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 5             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 6              \\\\\\\\\\
c
c=======================================================================
c
c
       subroutine nlsdac16 ( nlunit, line, jbeg, jend, jran, iquote
     1                     , word32 )
c
c    dac:namelist.nlsdac16 <-------------- returns next "word" in "line"
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine returns the next 32 character "word" found
c  after the input position of "jbeg" in "line".  The beginning and
c  ending positions of "word" are returned as well ("jbeg" and "jend").
c
c  There are two types of delimiters which define "word".  The first
c  type include the double and single quotation marks [" and '].  The
c  quotes are returned with "word".  Both the beginning and ending of
c  "word" must be delimited with the same type of quotation mark.
c
c  The second type include the equals sign [=], the asterisk [*], the
c  comma [,], the full colon [:], the opening parenthesis [(], the
c  closing parenthseis [)], the dollar sign [$], the blank, and the end
c  of "line".  These delimiters are not returned with "word", and so
c  "word" may be as long as 32 characters.  "word" may begin and end
c  with different delimiters of the second type.  (Note that $ and the
c  end of "line" can only delimit the end of "word".)  Delimiters of the
c  first type may be used to end "word" beginning with a delimiter of
c  the first type, and will not be returned with "word".  The reverse is
c  not true.  Delimiters of the second type found between like quotation
c  marks are returned as characters in "word".  Note that the search for
c  the next word will start directly at abs(jbeg) if "jbeg" < 0.
c
c  If the end of "line" is encountered before the next "word" is found,
c  the next "line" is read.  Both "line" and "word" are returned to the
c  calling subroutine.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1   , char2   , char3
       character*32  word32  , blank32
       character*72  line    , oldline
       integer       nlunit  , jbeg    , jend    , jran    , iquote
     1             , j       , j1      , j2      , iflag
c
c      Data statements
c
       data          blank32 /'                                '/
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Scan "line" for beginning of next "word".
c
       iflag = 1
       if (jbeg .lt. 0) then
         iflag = - 1
         jbeg  = - jbeg
         if (jbeg .gt. 72) go to 30
         go to 70
       endif
       j1 = jbeg
       if (jbeg .eq. 0) j1 =  3
10     continue
       do 20 j=j1,72
         char1 = line(j:j)
         if ( (char1 .eq. ' ') .or. (char1 .eq. ',') .or.
     1        (char1 .eq. '=') .or. (char1 .eq. ')') .or.
     2        (char1 .eq. '(') .or. (char1 .eq. '*') .or.
     3        (char1 .eq. ':') ) go to 20
         jbeg = j
         go to 70
20     continue
30     continue
c
c      The only way for execution to have reached this point is for the
c  end of the line to have been reached without finding another "word".
c  Read next "line", and resume search.
c
       oldline = line
40     continue
       read  (nlunit, 1010, end=50, err=60) line
c
c      Error message if column 1 contains anything but a 'c', 'C', or a
c  blank.
c
       char2 = line(1:1)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char2 .ne. ' ') ) call nlsdac15 ( line, 1, 1 )
c
c      Error message if column 2 contains anything but a '$' or a blank
c  (unless column 1 contains a 'c' or a 'C').
c
       char3 = line(2:2)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char3 .ne. '$') .and. (char3 .ne. ' ') )
     2   call nlsdac15 ( line, 2, 2 )
c
c      If column 1 contains a 'c' or a 'C', the line is "commented out",
c  and is echoed on the terminal.
c
       if ( (char2 .eq. 'c') .or. (char2 .eq. 'C') ) then
         write (6, 2010) line
         go to 40
       endif
c
c      If column 2 contains a '$' sentinal, then a new namelist has
c  begun without the previous one being closed.  Issue error message.
c
       if (char3 .eq. '$') then
         if (iquote .eq. 0) then
           call nlsdac15 ( oldline, 72, 13 )
         else
           call nlsdac15 ( line, 2, 9 )
         endif
       endif
c
c      New "line" is OK, resume scanning.
c
       if (iflag .eq. -1) then
         jbeg = 3
         go to 70
       else
         j1 = 3
         go to 10
       endif
c
c      Error: premature end of file.
c
50     continue
       call nlsdac15 ( line, 72, 12 )
c
c      Error: problem reading input deck
c
60     continue
       call nlsdac15 ( blank32, 0, 7 )
c
c      "jbeg" has been determined.  Now find "jend", which cannot be
c  greater than "jbeg" + 31.
c
70     continue
       j1   = jbeg + 1
       j2   = jbeg + 31
       if (j2 .gt. 72) j2 = 72
       jend = jbeg
c
c      Case:  first delimiter is a single quote.
c
       if ( (iquote .eq. 1) .or. (char1 .eq. '''') ) then
         do 80 j=j1,j2
           jend  = j
           char1 = line(j:j)
           if (char1 .eq. '''') go to 110
80       continue
c
c      Closing single quote not found.  Set "iquote" to 1.
c
         iquote = 1
         go to 120
       endif
c
c      Case:  first delimiter is a double quote.
c
       if ( (iquote .eq. 2) .or. (char1 .eq. '"') ) then
         do 90 j=j1,j2
           jend  = j
           char1 = line(j:j)
           if (char1 .eq. '"') go to 110
90       continue
c
c      Closing double quote not found.  Set "iquote" to 2.
c
         iquote = 2
         go to 120
       endif
c
c      Case:  first delimiter is not a quote.
c
       do 100 j=j1,j2
         char1 = line(j:j)
         if ( (char1 .eq. ' ') .or. (char1 .eq. ',') .or.
     1        (char1 .eq. '=') .or. (char1 .eq. ')') .or.
     2        (char1 .eq. '(') .or. (char1 .eq. '*') .or.
     3        (char1 .eq. ':') .or. (char1 .eq. '"') .or.
     4        (char1 .eq. '''') .or. (char1 .eq. '$') ) go to 110
         jend = j
100    continue
110    continue
       iquote = 0
120    continue
       jran   = jend - jbeg + 1
       word32 = blank32
       word32(1:jran) = line(jbeg:jend)
c
       return
c
c-----------------------------------------------------------------------
c----------------------- Read format statements ------------------------
c-----------------------------------------------------------------------
c
1010   format(a72)
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(a72)
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 6              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 7              \\\\\\\\\\
c
c=======================================================================
c
c
       subroutine nlsdac17 ( nlunit, line, jbeg, jend, char1 )
c
c    dac:namelist.nlsdac17 <----------- returns next non-blank character
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine returns the first non-blank character found
c  at or after the input position "jbeg" in "line".  If no non-blank
c  character is found, the next line is read and scanned for the first
c  non-blank character.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1   , char2   , char3
       character*8   blank8
       character*72  line    , oldline
       integer       nlunit  , jbeg    , jend    , j       , j1
c
c      Data statements
c
       data          blank8 / '        ' /
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Scan "line" for next non-blank character.
c
       j1 = jbeg
       if (j1 .eq. 0) j1 = 3
10     continue
       do 20 j=j1,72
         jbeg  = j
         jend  = j
         char1 = line(j:j)
         if (char1 .ne. ' ') return
20     continue
c
c      End of line reached before non-blank character found.  Read
c  next line.
c
       oldline = line
30     continue
       read  (nlunit, 1010, end=40, err=50) line
c
c      Error message if column 1 contains anything but a 'c', 'C', or a
c  blank.
c
       char2 = line(1:1)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char2 .ne. ' ') ) call nlsdac15 ( line, 1, 1 )
c
c      Error message if column 2 contains anything but a '$' or a blank
c  (unless column 1 contains a 'c' or a 'C').
c
       char3 = line(2:2)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char3 .ne. '$') .and. (char3 .ne. ' ') )
     2   call nlsdac15 ( line, 2, 2 )
c
c      If column 1 contains a 'c' or a 'C', the line is "commented out",
c  and is echoed on the terminal.
c
       if ( (char2 .eq. 'c') .or. (char2 .eq. 'C') ) then
         write (6, 2010) line
         go to 30
       endif
c
c      If column 2 contains a '$' sentinal, then a new namelist has
c  begun without the previous one being closed.  Issue error message.
c
       if (char3 .eq. '$') call nlsdac15 ( oldline, 72, 13 )
c
c      New "line" is OK, resume scanning.
c
       j1 = 3
       go to 10
c
c      Error: premature end of file.
c
40     continue
       call nlsdac15 ( line, 72, 12 )
c
c      Error: problem reading input deck
c
50     continue
       call nlsdac15 ( blank8, 0, 7 )
c
c-----------------------------------------------------------------------
c----------------------- Read format statements ------------------------
c-----------------------------------------------------------------------
c
1010   format(a72)
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(a72)
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 7              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 8              \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac18 ( line, jbeg, word32 )
c
c    dac:namelist.nlsdac18 <---- checks for valid integer representation
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine calls nlsdac15 (which issues fatal error
c  messages) if the input character string ("word32") does not represent
c  an integer.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       character*72  line
       integer       jbeg    , ineg    , ipos    , idig    , i
     1             , ibeg    , iend    , j       , jpos
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.
c
       ineg  = 0
       ipos  = 0
       idig  = 0
c
c      Find first non-blank character.
c
       do 10 i=1,32
         ibeg  = i
         char1 = word32(i:i)
         if (char1 .ne. ' ') go to 20
10     continue
c
c      "word32" is blank.  Issue an error message.
c
       call nlsdac15 ( line, jbeg, 11 )
c
c      Scan word32 up to the next blank.  An error message is issued if:
c
c   1. more than one sign (+ or -) is encountered, or
c   2. a sign is embedded in the integer, or
c   3. if the integer is found to have more than 15 digits, or
c   4. if a character other than +, -, or a digit is found.
c
20     continue
       do 40 i=ibeg,32
         jpos  = jbeg + i
         iend  = i
         char1 = word32(i:i)
         if (char1 .eq. ' ') go to 50
         if (char1 .eq. '-') then
           ineg = ineg + 1
           if ( (ineg .gt. 1) .or. (ipos .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (idig .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if (char1 .eq. '+') then
           ipos = ipos + 1
           if ( (ipos .gt. 1) .or. (ineg .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (idig .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         do 30 j=1,10
           if (char1 .eq. ch(j)) then
             idig = idig + 1
             if (idig .gt. 15) call nlsdac15 ( line, jpos, 15 )
             go to 40
           endif
30       continue
         call nlsdac15 ( line, jpos, 4 )
40     continue
c
c      First blank after data entry is found.
c
50     continue
c
c      If there are no digits after sign, issue error message.
c
       if (idig .eq. 0) call nlsdac15 ( line, jpos, 4 )
c
c      Scan rest of "word32" and issue error message if a non-blank
c  character is found.
c
       if (iend .lt. 32) then
         do 60 i=iend,32
           char1 = word32(i:i)
           jpos  = jbeg + i
           if (char1 .ne. ' ') call nlsdac15 ( line, jpos, 4 )
60       continue
       endif
       return
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 8              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 9              \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac19 ( word32, ival )
c
c    dac:namelist.nlsdac19 <----- translates character string to integer
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine translates a character string to an
c  integer.  It is assumed that the character string is a valid
c  representation of an integer, and is error-free (i.e., NLSDAC18 has
c  been called).
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       integer       ival    , ichar1  , ifact   , i
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c-----------------------------------------------------------------------
c
       ichar1 = 32
       ifact  = 1
       ival   = 0
10     continue
       char1 = word32(ichar1:ichar1)
       if (char1 .ne. ' ') then
         do 20 i=1,10
           if (char1 .eq. ch(i)) go to 30
20       continue
         if (char1 .eq. '-') then
           ival = - ival
           return
         endif
30       continue
         ival  = ival + ifact * (i - 1)
         ifact = ifact * 10
       endif
       ichar1 = ichar1 - 1
       if (ichar1 .gt. 0) go to 10
       return
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 9              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 2 0              \\\\\\\\\\
c
c=======================================================================
c
c
       subroutine nlsdac20 ( ival, word32 )
c
c    dac:namelist.nlsdac20 <----- translates integer to character string
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine translates an integer into a left-justified
c  character string.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*32  word32  , blank32
       integer       ival    , j       , jj      , jbeg    , jend
     1             , ilog
       real*8        x       , x1      , xlog
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
       data          blank32  /'                                '/
c
c-----------------------------------------------------------------------
c
       word32 = blank32
       if (ival .lt. 0) then
         jbeg = 2
         ival = - ival
         word32(1:1) = '-'
       else
         jbeg = 1
       endif
       x = dfloat(ival)
       if (ival .eq. 0) then
         ilog = 0
       else
         xlog = dlog10(x)
         ilog = idint (xlog + .000001d0)
       endif
       jend = jbeg + ilog
       if (jend .ge. 32) jend = 32
       do 10 j=jbeg,jend
         x1 = x / 10.0d0**ilog
         jj = idint ( x1 + .000001d0 )
         word32(j:j) = ch(jj+1)
         x  = x - dfloat(jj) * 10**ilog
         ilog = ilog - 1
10     continue
       return
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 2 0              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 1             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac21 ( word32, rval )
c
c    dac:namelist.nlsdac21 <-------- translates character string to real
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine translates a character string to a real
c  constant.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       integer       i       , esign   , ipnt    , npnt    , more
     1             , iblnk   , iman    , iexp    , j
       real*8        rval    , msign   , term    , fact
c
       character*1   ch      (  10)
       integer       exponent(   4), mantissa(  16)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.
c
       do 10 i=1,4
         exponent(i) = 0
10     continue
       do 20 i=1,16
         mantissa(i) = 0
20     continue
       msign = 1.0d0
       esign = 1
       ipnt  = 0
       npnt  = 0
       more  = 1
       iblnk = 0
       iman  = 0
       iexp  = 0
c
c      Scan "word32".  It is assumed that "word32" has already been
c  determined to be error-free, and that it is a valid character
c  representation of a real value (i.e., NLSDAC23 has been called).
c
       do 40 i=1,32
         char1 = word32(i:i)
         if (char1 .eq. ' ') then
           iblnk = iblnk + 1
           go to 40
         endif
         if (char1 .eq. '-') then
           if (more .eq. 1) msign = -1.0d0
           if (more .eq. 2) esign = -1
           go to 40
         endif
         if (char1 .eq. '+') go to 40
         if (char1 .eq. '.') then
           npnt = 1
           ipnt = i - 2 - iblnk
           if (msign .eq. -1.0d0) ipnt = ipnt - 1
           go to 40
         endif
         if ( (char1 .eq. 'e') .or. (char1 .eq. 'd') ) then
           more = 2
           if (npnt .eq. 0) then
             npnt = 1
             ipnt = i - 2 - iblnk
             if (msign .eq. -1.0d0) ipnt = ipnt - 1
             if (ipnt .lt. 0) ipnt = 0
           endif
           go to 40
         endif
         do 30 j=1,10
           if (char1 .eq. ch(j)) then
             if (more .eq. 1) then
               iman = iman + 1
               mantissa(iman) = j - 1
             endif
             if (more .eq. 2) then
               iexp = iexp + 1
               exponent(iexp) = j - 1
             endif
             go to 40
           endif
30       continue
40     continue
       if (npnt .eq. 0) ipnt = iman - 1
c
c      Evaluate the mantissa.
c
       rval = 0.0d0
       if (iman .gt. 0) then
         do 50 i=1,iman
           term = dfloat(mantissa(i)) * 10.0d0**ipnt
           rval = rval + term
           ipnt = ipnt - 1
50       continue
       else
         rval = 1.0d0
       endif
       rval = rval * msign
c
c      Evaluate the exponential factor.
c
       fact = 1.0d0
       if (iexp .gt. 0) then
         do 60 i=1,iexp
           ipnt = iexp - i
           fact = fact * 10.0d0 ** (exponent(i) * 10.0d0**ipnt)
60       continue
       endif
       if (esign .eq. -1) fact = 1.0d0 / fact
c
c      Return real representation of character string.
c
       rval = rval * fact
       return
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 1             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 2             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac22 ( line, jbeg, word32 )
c
c    dac:namelist.nlsdac22 <---- checks for valid logical representation
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine calls nlsdac15 (which issues fatal error
c  messages) if the input character string ("word32") does not represent
c  a valid logical value.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*32  word32
       character*72  line
       integer       jbeg
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
       if ( (word32(1:8) .ne. '.true.  ') .and.
     1      (word32(1:8) .ne. '.t.     ') .and.
     2      (word32(1:8) .ne. '.false. ') .and.
     3      (word32(1:8) .ne. '.f.     ') )
     4   call nlsdac15 ( line, jbeg, 5 )
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 2             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 3              \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac23 ( line, jbeg, word32 )
c
c    dac:namelist.nlsdac18 <-- determines if character string is a valid
c                                         representation of a real value
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine calls nlsdac15 (which issues various fatal
c  error messages) if the input character string ("word32") does not
c  represent a real value.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       character*72  line
       integer       jbeg    , mneg    , eneg    , mpos    , epos
     1             , iexp    , iman    , ipnt    , more    , i
     2             , ibeg    , iend    , j       , jpos
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.
c
       mneg  = 0
       eneg  = 0
       mpos  = 0
       epos  = 0
       iexp  = 0
       iman  = 0
       ipnt  = 0
       more  = 1
c
c      Find first non-blank character.
c
       do 10 i=1,32
         ibeg  = i
         char1 = word32(i:i)
         if (char1 .ne. ' ') go to 20
10     continue
c
c      "word32" is blank.  Issue an error message.
c
       call nlsdac15 ( line, jbeg, 11 )
c
c      Scan word32 up to the next blank.  An error message is issued if:
c
c   1. more than one sign (+ or -) is encountered in the mantissa, or
c   2. a sign is embedded in the mantissa, or
c   3. more than one sign (+ or -) is encountered in the exponent, or
c   4. a sign is embedded in the exponent, or
c   5. if more than one decimal point is encountered in the mantissa, or
c   6. if a decimal point is encountered in the exponent, or
c   7. if more than one exponential sentinal ("e" or "d") is found, or
c   8. if the mantissa is found to have more than 16 digits, or
c   9. if the mantissa is found to have more than  4 digits, or
c  10. if a character other than +, -, ., e, d, or a digit is found.
c
20     continue
       do 40 i=ibeg,32
         jpos  = jbeg + i
         iend  = i
         char1 = word32(i:i)
         if (char1 .eq. ' ') go to 50
         if ( (char1 .eq. '-') .and. (more .eq. 1) ) then
           mneg = mneg + 1
           if ( (mneg .gt. 1) .or. (mpos .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iman .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '-') .and. (more .eq. 2) ) then
           eneg = eneg + 1
           if ( (eneg .gt. 1) .or. (epos .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iexp .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '+') .and. (more .eq. 1) ) then
           mpos = mpos + 1
           if ( (mpos .gt. 1) .or. (mneg .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iman .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '+') .and. (more .eq. 2) ) then
           epos = epos + 1
           if ( (epos .gt. 1) .or. (eneg .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iexp .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '.') .and. (more .eq. 1) ) then
           ipnt = ipnt + 1
           iman = iman + 1
           if (ipnt .gt. 1) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '.') .and. (more .eq. 2) ) then
           call nlsdac15 ( line, jpos, 4 )
         endif
         if ( (char1 .eq. 'e') .or. (char1 .eq. 'd') ) then
           if (more .eq. 2) call nlsdac15 ( line, jpos, 4 )
           more = 2
           go to 40
         endif
         do 30 j=1,10
           if (char1 .eq. ch(j)) then
             if (more .eq.  1) iman = iman + 1
             if (iman .gt. 16) call nlsdac15 ( line, jpos, 15 )
             if (more .eq.  2) iexp = iexp + 1
             if (iexp .gt.  3) call nlsdac15 ( line, jpos, 14 )
             go to 40
           endif
30       continue
         call nlsdac15 ( line, jpos, 4 )
40     continue
c
c      First blank after data entry is found.
c
50     continue
c
c      If there are no digits after sign and/or decimal point, issue
c  error message.
c
       iman = iman - ipnt
       if (iman .eq. 0) call nlsdac15 ( line, jpos, 4 )
c
c      Scan rest of "word32" and issue error message if a non-blank
c  character is found.
c
       if (iend .lt. 32) then
         do 60 i=iend,32
           char1 = word32(i:i)
           jpos  = jbeg + i
           if (char1 .ne. ' ') call nlsdac15 ( line, jpos, 4 )
60       continue
       endif
       return
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 3             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 4             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac24 ( lunit, cname, nmlst, nmax )
c
c    dac:namelist.nlsdac24 <---- dumps namelist values to specified unit
c                                                         february, 1991
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine dumps a formatted summary of all user-
c  specified namelist parameters in the namelist "cname" to the logical
c  unit "lunit".
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lval
       character*1   char1   , attr    , type
       character*4   jbeg    , jend    , kbeg    , kend
       character*8   cname   , word8   , temp8   , nword8
       character*25  var
       character*32  blank32
       character*512 cval
       integer       lunit   , nmax    , i       , ilist   , ival
     1             , lbeg    , lend    , lran    , j       , j1
     2             , j2      , mbeg    , mend
       real*8        rval
c
       character*32  nmlst   (   2,   *)
c
c      Data statements
c
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19, nlsdac21
c
c-----------------------------------------------------------------------
c
       lbeg = 1
       do 10 j=1,16
         j2 = 32 * j
         j1 = j2 - 31
         cval(j1:j2) = blank32
10     continue
c
c      First line to output unit.
c
       write (lunit, 2010) cname
c
c      Read namelist entries, and write contents to output unit.
c
       do 100 i=1,nmax
         ilist = i
         if (nmlst(2,ilist) .eq. blank32) go to 100
         char1 = nmlst(1,ilist)(31:31)
         if (char1 .eq. '*') go to 100
         word8 = nmlst(1,ilist)( 1: 8)
         if (word8 .eq. '        ') go to 110
         nmlst(1,ilist)(31:31) = '*'
20       continue
         var   = blank32(1:25)
         attr  = nmlst(1,ilist)(30:30)
         type  = nmlst(1,ilist)(32:32)
c
c      Representation of variable name (contents of "nmlst(1,*)")
c  depends on type of variable (ie, scalar, vector, or 2-D array).
c
         if (type .ne. 's') then
           jbeg = nmlst(1,ilist)( 9:12)
           jend = nmlst(1,ilist)(13:16)
           if (jend .eq. '    ') jend = jbeg
           kbeg = nmlst(1,ilist)(17:20)
           kend = nmlst(1,ilist)(21:24)
           if (kend .eq. '    ') kend = kbeg
           if (type .eq. 'v') write (var, 2020) word8, jbeg, jend
           if (type .eq. 'a') write (var, 2030) word8, jbeg, jend
     1                                               , kbeg, kend
         else
           write (var, 2040) word8
         endif
c
c      Representation of variable value (contents of "nmlst(2,*)")
c  depends on attribute (ie, integer, real, logical, or character).
c
c  Case:  integer
c
         if (attr .eq. 'i') then
           call nlsdac19 ( nmlst(2,ilist), ival )
           write (lunit, 2050) var, ival
         endif
c
c  Case:  real
c
         if (attr .eq. 'r') then
           call nlsdac21 ( nmlst(2,ilist), rval )
           write (lunit, 2060) var, rval
         endif
c
c  Case:  logical
c
         if (attr .eq. 'l') then
           temp8 = nmlst(2,ilist)(1:8)
           if (temp8 .eq. '.true.  ') lval = .true.
           if (temp8 .eq. '.t.     ') lval = .true.
           if (temp8 .eq. '.false. ') lval = .false.
           if (temp8 .eq. '.f.     ') lval = .false.
           if (      lval) write (lunit, 2070) var
           if (.not. lval) write (lunit, 2080) var
         endif
c
c  Case:  character
c
c  For character strings, there is the possibility that the string is
c  stored in pieces throughout "nmlst(2,*)".  Thus, delay writing the
c  entry to the output unit until all the character string can be
c  gathered.
c
         if (attr .eq. 'c') then
           if (nmlst(2,ilist)(32:32) .eq. '&') then
             lran = ichar ( nmlst(2,ilist)(31:31) )
             lend = lbeg + lran - 1
             cval(lbeg:lend) = nmlst(2,ilist)(1:lran)
             lbeg = lend + 1
           else
             do 50 lran=31,1,-1
               if (nmlst(2,ilist)(lran:lran) .ne. ' ') go to 60
50           continue
60           continue
             lend = lbeg + lran - 1
             cval(lbeg:lend) = nmlst(2,ilist)(1:lran)
             if (lend .le. 50) then
               write (lunit, 2090) var, cval(1:lend)
             else
               mbeg = 1
               mend = 50
               write (lunit, 2100) var, cval(mbeg:mend)
70             continue
               mbeg = mend + 1
               mend = mend + 50
               if (mend .ge. lend) then
                 write (lunit, 2110) cval(mbeg:lend)
               else
                 write (lunit, 2120) cval(mbeg:mend)
                 go to 70
               endif
             endif
             lbeg = 1
             do 80 j=1,16
               j2 = 32 * j
               j1 = j2 - 31
               cval(j1:j2) = blank32
80           continue
           endif
         endif
c
c      Check the rest of "nmlst(1,*)" for any other variable entries
c  with the same variable name.  Such entries may exist for vectors and
c  arrays of all attributes, and scalars for character variables.  An
c  asterisk is put (temporarily) in column 31 of "nmlst(1,ilist)" so
c  that this entry will not be read again.
c
         if (ilist .ne. nmax) then
           do 90 j=ilist+1,nmax
             nword8  = nmlst(1,j)( 1: 8)
             if (nword8 .eq. '        ') go to 100
             if (nword8 .eq. word8) then
               nmlst(1,j)(31:31) = '*'
               ilist = j
               go to 20
             endif
90         continue
         endif
100    continue
110    continue
c
c      Last line to output unit.
c
       write (lunit, 2130) cname
c
c      Replace asterisks put in column 31 of "nmlst(1,*)" with
c  blanks.
c
       do 120 i=1,nmax
         if (nmlst(1,i)(1:8) .eq. '        ') return
         nmlst(1,i)(31:31) = ' '
120    continue
c
       return
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(/,'======================================================'
     1       ,'> BEGIN NAMELIST: ',a8)
2020   format(a8,'(',a3,':',a3,')')
2030   format(a8,'(',a3,':',a3,',',a3,':',a3,')')
2040   format(a8)
2050   format(a25,' =',i10)
2060   format(a25,' =',1pg20.12)
2070   format(a25,' = .true.')
2080   format(a25,' = .false.')
2090   format(a25,' = "',a,'"')
2100   format(a25,' = "',a)
2110   format(29x,a,'"')
2120   format(29x,a)
2130   format('========================================================'
     1       ,'> END NAMELIST: ',a8)
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 4             \\\\\\\\\\
c
c=======================================================================
c
c
c---+----1----+----2----+----3----+----4----+----5----+----6----+----7--
c
c>>>>>>>>>>>>>>>>>>>>> EXTERNALLY CALLED ROUTINES <<<<<<<<<<<<<<<<<<<<<<
c
c  To use the capabilities of this library, the user would only call the 
c  following three routines in their program.  All subsequent routines 
c  are included to satisfy all the calls that the following three 
c  routines make.
c
c      GRFXOPN    opens the plot file
c      GRFX1D     makes a single page, multiple-frame, single variable 
c                 per frame plot
c      GRFXCLS    closes the plot file.
c
c  Consult the headers of each of these routines to determine the 
c  required syntax and purpose of each variable in the argument list.
c
c  As an example of a call to GRFX1D:
c
c      call grfx1d ( psi, den, 0, 1, npnt, 1, 1, 0.0, 0.0, 0.0, 0.0
c    1             , 2, 1.0, 1, 2, 2, 1, 2, 1, xlab, ylab )
c
c  creates a square linear-linear graph of all "npnt" points of the 
c  dependent variable "den" (density) plotted against the independent 
c  variable "psi", allowing the plotter to determine the optimal 
c  plotting window, joining all points with lines.  The last six 
c  integers the plotter to arrange the graphs in a 2 x 2 fashion on a 
c  single page (rather than one plot per page).
c
c  The last two parameters, "xlab" and "ylab" use something called "NCAR 
c  mark-up language" to label the plots.  You're perfectly safe to just 
c  set xlab='x' and ylab='rho' here, though if you wanted to be a little 
c  fancy, try: xlab = '|F10|x/t', ylab = '|F5|r'.
c
c=======================================================================
c
       subroutine grfx1d ( xvar, yvar, yexp, ibeg, iend, linlog, imm
     1                   , xmn, xmx, ymn, ymx, corl, aspect, allxl
     2                   , allyl, np1h, ip1h, np1v, ip1v, xlab, ylab )
c
c    dac:grfx03.grfx1d <-------------------------------- plots 1-d slice
c                                                             june, 1990
c
c    written by: David Clarke
c    modified 1: June, 1996 by David Clarke; rewritten to generate
c                publication quality plots.
c    modified 2: December, 1996 by David Clarke; added multiple frames
c                per page capability.
c    modified 3: October, 2003 by Alexander Men'shchikov; generalized to
c                include the PSPLOT graphics library.
c    modified 4: February, 2005 by David Clarke; streamlined Sasha's
c                modifications
c    modified 5: April, 2006 by DAC; introduced "yexp", in analogy to 
c                "qexp" and "vexp" in GRFX2D.
c    modified 6: December, 2016 by DAC; simplified from July, 2015 
c                version for this "cheap and cheerful" standalone 
c                library to do single-variable, multi-framed plots on a 
c                single page.
c
c  PURPOSE: This is a pared down version of the subroutine by the same 
c  name in GRFX03 (as of last changes made in July, 2015) that isolates 
c  all the calls to the PSPLOT library necessary for making a single 
c  page of possibly multiple frames, each with a single variable 
c  plotted.  There is no legend capability, and the NCAR option has been 
c  deleted.  While this version does not directly support writing a 
c  header (title) or footer to the graph, there is a standalone routine 
c  in this package called HDRFTR that can put a header and/or footer to 
c  the graph is desired.  For the full capabilities of GRFX03, that 
c  library should be used.
c
c  For clarity, the terms "page", "frame", and "plot" are used 
c  consistently.  Thus, on a single page, there may be numerous frames 
c  (np1h x np1v), and on each frame, there may be numerous plots (though 
c  in this version, only one plot per frame).  The word "graph" is used 
c  generically and refers to any graphical unit, be it a single plot, a 
c  frame, or the entire page.  Its meaning is gleaned from context.
c
c  If a plot is to start from within the frame rather than from the 
c  border (e.g., there may be no solution to what is being plotted for 
c  some values of the abscissa), the function value should be set to 
c  -1.1e-32 or more negative at the points to be ignored.  Such points 
c  will not have line segments drawn to or from them, even to intersect 
c  the border.
c
c  INPUT VARIABLES:
c    xvar        vector    of independent variable (x-axis)
c    yvar        vector(s) of   dependent variable (y-axis)
c    yexp        log10 of scaling factor generated when "yvar" was
c                converted to single precision (integer)
c    ibeg        index of first value in "xvar" and "yvar" to plot
c    iend        index of last  value in "xvar" and "yvar" to plot
c    linlog      = 1 => linear x and y;    = 2 => linear x, log y
c                = 3 => log x, linear y;   = 4 => log x and y
c    imm         = 0 => use input "ymn" and "ymx" to limit range of data
c                       plotted
c                =-1 => same as 0, and extend y-axis by one minor tick
c                = 1 => plot entire range of data; extend y-axis
c    xmn         minimum value on the x-axis
c    xmx         maximum value on the x-axis
c    ymn         minimum value on the y-axis
c    ymx         maximum value on the y-axis
c    corl        =-6 => open diamonds; one symbol per datum
c                =-5 => open pentagrams
c                =-4 => asterisk (*)
c                =-3 => open squares
c                =-2 => open triangles
c                =-1 => plus signs (+)
c                = 0 => crosses (x)
c                = 1 => open circles (o)
c                = 2 => smooth curve drawn through data
c    aspect      desired aspect ratio of axes.
c                < 1.0 => short, wide frame
c                = 1.0 => square frame
c                > 1.0 => tall, narrow frame
c    allxl       =2 => all x labels/annotations are drawn (default)
c                =1 => only the x-label/annotations on the bottom row
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the bottom row.
c                if allxl < 0, same as abs(allxl) except last annotation
c                is not drawn if it falls at xxmax.
c    allyl       =2 => all y labels/annotations are drawn (default)
c                =1 => only the y-label/annotations on the left column
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the left column.
c                if allyl < 0, same as abs(allyl) except last annotation
c                is not drawn if it falls at yymax.
c    np1h        number of frames to be arranged horizontally per page
c    ip1h        in the current row   , this is the ip1h'th frame
c    np1v        number of frames to be arranged vertically   per page
c    ip1v        in the current column, this is the ip1v'th frame
c    xlab        label for x-axis
c    ylab        label for y-axis
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    GRFXAXES
c    PSCHARS
c    SETLW               )
c    SETCOLR             )
c    OCIRCLE , TIMES     )
c    PLUS    , TRIANGLE  )---- PSPLOT routines
c    SQUARE  , ASTERISK  )
c    STAR    , DIAMOND   )
c    DSHCRV  , PLOT      )
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       ndim
       real          tiny    , huge    
       parameter     ( ndim = 9999
     1               , tiny = 1.0e-32, huge = 1.0e32 )
c
       character*(*) xlab    , ylab    
       logical       lmax0   , lastpt  
       integer       yexp    , ibeg    , iend    , linlog  , imm      
     1             , corl    , allxl   , allyl   , np1h    , ip1h    
     2             , np1v    , ip1v    , lnlg    , nzi     , i       
     4             , i1      , ip1     , ip2     , ip3     , eye     
     6             , ndash   , n       , ista    , ifin    
       real          xmn     , xmx     , ymn     , ymx     , penwt   
     1             , aspect  , xmin    , xmax    , ymin    , ymax    
     2             , yming   , ymaxg   , yminl   , ymaxl   , aymaxl  
     2             , q1      , q2      , ymingt0 , p1x0    , p1y0       
     4             , size    , deks    , eksip1  , eksmin  , eksmax  
     7             , delwhy  , dwhyll  , dwhyl   , dwhyr   , dwhy    
     8             , dwhyrr  , whymin  , whymax  , whyip1  , ffx     
     9             , ffy     
c
       real          xvar    (   *), yvar    (   *)
     1             , eks     (ndim), ex      (ndim), why     (ndim)
c
       real          wxmin   , wxmax   , wymin   , wymax   , xxmin
     1             , xxmax   , yymin   , yymax   , scale
       common      / win1d /
     1               wxmin   , wxmax   , wymin   , wymax   , xxmin
     2             , xxmax   , yymin   , yymax   , scale
c
c      External statements
c
       external      grfxaxes, pschars , setlw   , setcolr , ocircle 
     1             , times   , plus    , triangle, square  , asterisk
     3             , star    , diamond , dshcrv  , plot     
c
c-----------------------------------------------------------------------
c
       lnlg  = max ( 1, min ( 4, linlog ) )
       penwt = 1.0
       call setlw ( 0.03 * penwt )
c
c      Set origin and symbol sizes.
c
       p1x0 = 4.5
       p1y0 = 10.8
       if (np1h*np1v .eq. 1) p1y0 = 13.4
       size = 0.085
c
c-----------------------------------------------------------------------
c---------------------------- E X T R E M A ----------------------------
c-----------------------------------------------------------------------
c
c      Set the domain, "xmin" and "xmax".  
c
       xmin = xmn
       xmax = xmx
       if (xmn .ge. xmx) then
         xmin = xvar(ibeg)
         xmax = xvar(iend)
       endif
c
c      If a log x-axis is desired (lnlg=3,4), make sure xvar > 0.  If
c  not, reset lnlg to give a linear x-axis and issue a warning.
c
       if (lnlg .ge. 3) then
         if (xmin .le. 0.0) then
           lnlg = lnlg - 2
           write (6, 2010)
         endif
       endif
c
c      Determine the actual x-limits of the plot ("ista" and "ifin").
c
       ista = ibeg
       ifin = iend
       do 20 i=ibeg,iend
         if ( xvar(i) .lt. xmin ) ista = i + 1
         if ( xvar(i) .le. xmax ) ifin = i
20     continue
c
c      Set the range of the frame.  Four measures of the extrema are 
c  considered:
c
c  1.  ymn, ymx    user-input extrema to be used for "yvar", and take
c                  precedent over all other evaluations.
c  2.  ymaxg,      global (ibeg:iend) extrema for "yvar", used below for
c        yming     detecting odd poles.
c  3.  ymaxl,      local (ista:iend) extrema for "yvar", used to compute
c        yminl     ymin and ymax.
c  4.  ymin, ymax  set to ymn, ymx if imm .le. 0, or the extrema of 
c                  ymaxl and yminl if imm = 1; used by GRFXAXES to 
c                  determine the scale for the ordinate.
c
       ymaxg =-huge
       yming = huge
       do 30 i=ibeg,iend
         ymaxg = max ( ymaxg, yvar(i) )
         yming = min ( yming, yvar(i) )
30     continue
       ymaxl =-huge
       yminl = huge
       do 40 i=ista,ifin
         if (yvar(i) .lt. huge) ymaxl = max ( ymaxl, yvar(i) )
         if (yvar(i) .gt.-huge) yminl = min ( yminl, yvar(i) )
40     continue
       aymaxl = max ( abs(yminl), abs(ymaxl) )
c
c      Set the range, "ymin" and "ymax".  
c
       if ( (imm .le. 0) .and. (ymx .gt. ymn) ) then
         ymin = ymn
         ymax = ymx
       else
         ymax = max (-huge, ymaxl )
         ymin = min ( huge, yminl )
       endif
c
c      Adjust extrema of range if both are zero, or if they differ by 
c  less than one part in 10**5.  NB: SIGN function is used (instead of 
c  testing ymin*ymax directly) in case ymin*ymax is larger than 1.0e38, 
c  thus triggering an overflow NaN.
c
       if ( (ymin .eq. 0.0) .and. (ymax .eq. 0.0) ) then
         ymin =-1.0
         ymax = 1.0
       else
         q1 = sign ( 1.0, ymin ) * sign ( 1.0, ymax )
         if (q1 .ge. 0.0) then
           q2 = 2.0 * ( ymax - ymin ) / ( ymax + ymin )
           if (abs(q2) .le. 1.0e-5) then
             if (ymax .gt. 0.0) then
               ymax = 1.005 * ymax
               ymin = 0.995 * ymin
             else
               ymax = 0.995 * ymax
               ymin = 1.005 * ymin
             endif
           endif
         endif
       endif
c
c      If a log y-axis is desired (lnlg=2,4), make sure yvar > 0.  If
c  ymin < 0 or if ymin = ymax = 0, reset "lnlg" to give a linear y-axis 
c  and issue a warning.  If ymin = 0 and ymax > 0, reset "ymin" and all 
c  zero-values of "yvar" (if any) to the smallest non-zero value in 
c  "yvar" ("ymingt0"), leave "lnlg" alone, and issue a warning.
c
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) ) then
c
         if (ymin .lt. 0.0) then
           lnlg = lnlg - 1
           write (6, 2020)
         endif
c
         if (ymin .eq. 0.0) then
           lmax0 = .false.
           if (aymaxl .eq. 0.0) lmax0 = .true.
           if (lmax0) then
             lnlg = lnlg - 1
             write (6, 2021)
           else
             ymingt0 = huge
             do 90 i=ista,ifin
               if (yvar(i) .gt. 0.0) ymingt0 = min ( ymingt0, yvar(i) )
90           continue
             do 110 i=ista,ifin
               yvar(i) = max ( ymingt0, yvar(i) )
110          continue
             ymin = ymingt0
             write (6, 2030)
           endif
         endif
c
       endif
c
c-----------------------------------------------------------------------
c------------------------------- A X E S -------------------------------
c-----------------------------------------------------------------------
c
       call grfxaxes ( yexp, xlab, ylab, imm, xmin, xmax, ymin, ymax
     1               , aspect, p1x0, p1y0, allxl, allyl, np1h, ip1h
     2               , np1v, ip1v, lnlg )
c
c      Apply "scale" computed in GRFXAXES to font and symbol sizes.
c
       size = size * scale
c
c-----------------------------------------------------------------------
c-------------------------- P L O T   D A T A --------------------------
c-----------------------------------------------------------------------
c
       nzi = ifin - ista + 1
c
       call plot ( wxmin, wymin, 3 )
c
c      Set scaling factors "ffx" and "ffy".
c
       ffx = ( wxmax - wxmin ) / ( xxmax - xxmin )
       ffy = ( wymax - wymin ) / ( yymax - yymin )
       if (lnlg .ge. 3)
     1   ffx = ( wxmax - wxmin ) / log10 ( xxmax / xxmin )
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) )
     1   ffy = ( wymax - wymin ) / log10 ( yymax / yymin )
c
c      Bounds to keep plots inside frame windows.
c
       eksmin = wxmin
       eksmax = wxmin + ffx * ( xxmax - xxmin )
       whymin = wymin
       whymax = wymin + ffy * ( yymax - yymin )
       if (lnlg .ge. 3)
     1   eksmax = wxmin + ffx * log10 ( xxmax / xxmin )
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) )
     1   whymax = wymin + ffy * log10 ( yymax / yymin )
c
c       Fill array "eks" with scaled independent variable.  "ex" is a 
c  permanent copy of the abscissa should "eks" have to be adjusted to 
c  avoid a pole, start the graph on a border, etc.
c 
       do 220 i=1,nzi
         eye    = i + ista - 1
         eks(i) = wxmin + ffx * ( xvar(eye) - xxmin )
         if ( (lnlg .eq. 3) .or. (lnlg .eq. 4) )
     1     eks(i) = wxmin + ffx * log10 ( xvar(eye) / xxmin )
         ex (i) = eks(i)
220    continue
c
c       Fill array "why" with scaled dependent variable(s), and reset 
c  the abscissa.
c 
       do 250 i=1,nzi
         eye    = i + ista - 1
         why(i) = yvar(eye)
         if (why(i) .gt.-huge) then
           why(i) = wymin + ffy * ( yvar(eye) - yymin )
           if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) )
     1       why(i) = wymin + ffy * log10 ( yvar(eye) / yymin )
         endif
         eks(i) = ex(i)
250    continue
c
c      Set colour.
c
       call setcolr ( 0.2, 0.2, 0.2 )
c
c      Draw plots with one symbol per datum, wikth the symbol type set 
c  by "corl".  Only points between "xmin" and "xmax" (by virtue of 
c  running "i" from "ista" to "ifin") and between "ymin" and "ymax" are 
c  plotted.
c
       if ( (corl .ge.-6) .and. (corl .le. 1) ) then
         do 260 i=1,nzi
           if ( ( eks(i) .le. eksmax ) .and. 
     1          ( eks(i) .ge. eksmin ) .and. 
     2          ( why(i) .le. whymax ) .and. 
     3          ( why(i) .ge. whymin ) ) then
             if (corl .eq. 1) call ocircle  ( eks(i), why(i), size )
             if (corl .eq. 0) call times    ( eks(i), why(i), size )
             if (corl .eq.-1) call plus     ( eks(i), why(i), size )
             if (corl .eq.-2) call triangle ( eks(i), why(i), size )
             if (corl .eq.-3) call square   ( eks(i), why(i), size )
             if (corl .eq.-4) call asterisk ( eks(i), why(i), size )
             if (corl .eq.-5) call star     ( eks(i), why(i), size )
             if (corl .eq.-6) call diamond  ( eks(i), why(i), size )
           endif
260      continue
       endif
c
c      Variables are plotted with lines.  Set line weight and dash 
c  pattern.
c
       if (corl .eq. 2) then
         call setlw ( 0.03 * penwt )
         ndash = 0
c
c      Scan the independent variable to make sure it stays within the
c  limits "whymax" and "whymin".  An effort is made to avoid drawing 
c  curves through "odd poles" (where the functional value goes from +/- 
c  infinity to -/+ infinity).
c
c  "i1"     is the beginning index of the next line segment to be drawn.
c  "delwhy" is a measure of the greatest jump in why(i) that might be 
c           expected without it being an odd pole.  There is no need to 
c           prepare "delwhy" for a possible log plot as an odd pole 
c           would already have reset the log-y request to a linear-y.
c  "lastpt" keeps track of whether why(ip1) is within the frame and is
c           used to determine if a line segment should be drawn.
c  
         i1     = 1
         delwhy = ( min ( ymax, ymaxg )
     1            - max ( ymin, yming ) ) * ffy * 0.8
         dwhyll = 0.0
         dwhyl  = 0.0
         dwhy   = why(i1+1) - why(i1  )
         dwhyr  = why(i1+2) - why(i1+1)
         lastpt = .false.
         if ( (why(i1) .ge. whymin) .and. 
     1        (why(i1) .le. whymax) ) lastpt = .true.
         do 270 i=1,nzi-1
           ip1    = i + 1
           ip2    = min ( i+2, nzi )
           ip3    = min ( i+3, nzi )
           dwhyrr = why(ip3) - why(ip2)
           deks   = eks(ip1) - eks(i  )
c
c      Note from DAC, 7/15: last two criteria added to help prevent 
c  points flagged not to be plotted (by setting why=-huge in calling 
c  routine) from being mistaken for an odd pole.  Presumably, "why" 
c  never gets to -huge, even for an odd pole.
c
           if ( ( dwhyll*dwhyl    .gt. 0.0    ) .and.
     1          ( dwhyl *dwhy     .lt. 0.0    ) .and. 
     2          ( dwhy  *dwhyr    .lt. 0.0    ) .and.
     3          ( dwhyr *dwhyrr   .gt. 0.0    ) .and.
     3          ( why(i)*why(ip1) .lt. 0.0    ) .and.
     4          ( abs(dwhy)       .gt. delwhy ) .and.
     3          ( why(i)          .gt.-huge   ) .and.
     1          ( why(ip1)        .gt.-huge   ) ) then
c
c      An odd pole has been detected.  If a line segment is pending
c  (e.g., lastpt .eq. .true.), draw the current segment up to the left
c  of the pole, and set the starting point for the next line segment.
c
             if ( lastpt ) then
               q1 = deks / ( dwhy + tiny )
               if ( why(i) .gt. why(ip1) ) then
                 eks(i  ) = eks(i) + ( whymax - why(i) ) * q1 
                 why(i  ) = whymax
                 eks(ip1) = eks(i) + ( whymin - why(i) ) * q1
                 why(ip1) = whymin
               else
                 eks(i  ) = eks(i) + ( whymin - why(i) ) * q1 
                 why(i  ) = whymin
                 eks(ip1) = eks(i) + ( whymax - why(i) ) * q1 
                 why(ip1) = whymax
               endif
               call dshcrv ( eks(i1), why(i1), ip1-i1, ndash, 0.0 )
               i1 = ip1
             endif
c
           else
c
c      If why(i) and why(ip1) do not straddle an odd pole, perform the 
c  tests necessary to keep the plot inside the frame window.
c
c  1.  If "why" enters the frame from below, interpolate eks(i) to the 
c      entry point, and reset the initial point of the current segment.
c
c  Note from DAC, 7/15: the if-test on -huge here and in case 3 below is
c  the mechanism introduced to allow some points not to be plotted at 
c  all.  Such points are set to -1.0e32 in the calling routine, and are 
c  then skipped altogether (as opposed to being interpolated to the plot 
c  border for points outside the frame).
c
             if ( (why(i  ) .lt. whymin) .and. 
     1            (why(ip1) .gt. whymin) ) then
               eks(i) = eks(i) + ( whymin - why(i) ) * deks / dwhy 
               if (why(i) .le.-huge) then
                 why(i) = why(ip1)
               else
                 why(i) = whymin
               endif
               i1 = i
             endif
c
c  2.  If "why" enters the frame from above, interpolate eks(i) to the 
c      entry point, and reset the initial point of the current segment.
c
             if ( (why(i  ) .gt. whymax) .and. 
     1            (why(ip1) .lt. whymax) ) then
               eks(i) = eks(i) + ( whymax - why(i) ) * deks / dwhy 
               why(i) = whymax
               i1     = i
             endif
c
c  3.  If "why" falls below the frame, interpolate eks(ip1) to the exit 
c      point, draw the current segment to the exit point, and set the 
c      starting point for the next segment.
c
             if ( (why(i  ) .gt. whymin) .and. 
     1            (why(ip1) .lt. whymin) ) then
               eksip1   = eks(ip1)
               whyip1   = why(ip1)
               eks(ip1) = eks(i) + ( whymin - why(i) ) * deks / dwhy 
               if (why(ip1) .le.-huge) then
                 why(ip1) = why(i)
               else
                 why(ip1) = whymin
               endif
               call dshcrv ( eks(i1), why(i1), i-i1+2, ndash, 0.0 )
               eks(ip1) = eksip1
               why(ip1) = whyip1
               i1       = ip1
             endif
c
c  4.  If "why" rises above the frame, interpolate eks(ip1) to the exit 
c      point, draw the current segment to the exit point, and set the 
c      starting point for the next segment.
c
             if ( (why(i  ) .lt. whymax) .and. 
     1            (why(ip1) .gt. whymax) ) then
               eksip1   = eks(ip1)
               whyip1   = why(ip1)
               eks(ip1) = eks(i) + ( whymax - why(i) ) * deks / dwhy 
               why(ip1) = whymax
               call dshcrv ( eks(i1), why(i1), i-i1+2, ndash, 0.0 )
               eks(ip1) = eksip1
               why(ip1) = whyip1
               i1       = ip1
             endif
c
           endif
c
c      Update "lastpt".
c
           lastpt = .false.
           if ( (why(ip1) .ge. whymin) .and. 
     1          (why(ip1) .le. whymax) ) lastpt = .true.
c
c      Advance "dwhyll" to "dwhyl", etc., before next iteration.
c
           dwhyll = dwhyl
           dwhyl  = dwhy
           dwhy   = dwhyr
           dwhyr  = dwhyrr
270      continue
c
c      Draw last segment, if necessary.
c
         if ( lastpt ) then
           call dshcrv ( eks(i1), why(i1), nzi-i1+1, ndash, 0.0 )
         endif
c
       endif
c
c      Restore pen weight and colour before completing graph.
c
       call setlw   ( 0.03 * penwt )
       call setcolr ( 0.0, 0.0, 0.0 )
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format('GRFX1D  : |---WARNING---> The x-axis includes the '
     1       ,'origin.  Log option switched',/
     2       ,'GRFX1D  : to linear.')
2020   format('GRFX1D  : |---WARNING---> At least one dependent '
     1       ,'variable includes negative',/
     2       ,'GRFX1D  : values.  Log option switched to linear.')
2021   format('GRFX1D  : |---WARNING---> At least one dependent '
     1       ,'variable includes no positive',/
     2       ,'GRFX1D  : values.  Log option switched to linear.')
2030   format('GRFX1D  : |---WARNING---> The y-axis includes the '
     1       ,'origin.  Zeroes reset to ',/
     2       ,'GRFX1D  : minimum non-zero value; log option retained.')
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine grfxcls
c
c    dac:grfx03.grfxcls <---------------------------- closes a plot file
c                                                             june, 1990
c
c    written by: David Clarke
c    modified 1: 
c
c  PURPOSE: This subroutine closes a PSPLOT plot file and should be 
c  called after each call to GRFX1D and before the next plot is begun.
c
c  INPUT VARIABLES:
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    PLOT     )---- PSPLOT routine
c
c-----------------------------------------------------------------------
c
       implicit      none
c
c      External statements
c
       external      plot
c
c-----------------------------------------------------------------------
c
       call plot ( 0.0, 0.0, 999 )
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine grfxopn ( fname )
c
c    dac:grfx03.grfxopn <----------------------------- opens a plot file
c                                                             june, 1990
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This subroutine opens a PSPLOT plot file with the specified 
c  file name.  It should be called before GRFX1D is called.
c
c  INPUT VARIABLES:
c    fname       file name of plot file as set by calling routine
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    NEWDEV  )---- PSPLOT routines
c    PSINIT  )
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) fname   
c
c      External statements
c
       external      newdev  , psinit
c
c-----------------------------------------------------------------------
c
       call newdev ( fname )
       call psinit ( .true. )
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine hdrftr ( phdr, iftr, filename )
c
c    dac:1dplot.hdrftr <------------------ writes header/footer to graph
c                                                             june, 2018
c
c    written by: David Clarke
c    modified 1: 
c
c  PURPOSE: This subroutine writes the header (title) "phdr" to the top-
c  centre of the graph.  If desired, a footer with the filename and the 
c  date and time of creation is created and written at the bottom left 
c  of the graph.
c
c  INPUT VARIABLES:
c    phdr        header (title) to be written to graph.
c                blank => no header
c    iftr        =1 => footer created and written to graph
c                =0 => not
c    filename    used to create footer
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    SETLW            )  PSPLOT routine
c    STRLEN
c    PSCHARS
c    TIMEDATE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) phdr    , filename
       character*8   chtime
       character*10  chdate
       character*80  pftr
       integer       iftr    , nch   
       real          pt12    , pt20    , x1      , y1      
c
c      External statements
c
       integer       strlen
       external      setlw   , strlen  , pschars , timedate
c
c-----------------------------------------------------------------------
c
c      Set line weight and font sizes.
c
       call setlw ( 0.03 )
       pt12 = 0.264
       pt20 = 0.440
c
c      Write user-specified title, if desired.
c
       nch  = strlen ( phdr )
       if (nch .gt. 0) then
         x1 = 9.5
         y1 = 24.3
         call pschars ( x1, y1, pt20, phdr(1:nch), 0.0, 1 )
       endif
c
c      Create and write footer, if desired.
c
       if (iftr .eq. 1) then
         call timedate ( chtime, chdate )
         nch = strlen ( filename )
         write ( pftr, 2010 ) filename(1:nch), chdate, chtime
         nch = strlen ( pftr )
         x1  = 2.5 
         y1  = 7.0
         call pschars ( x1, y1, pt12, pftr(1:nch), 0.0, 0 )
       endif
c
c-----------------------------------------------------------------------
c
2010   format('Plot file `',a,''' created on ',a10,' at ',a8,'.')
c
       return
       end
c
c=======================================================================
c
c>>>>>>>>>>>>>>>>>>>>> INTERNALLY CALLED ROUTINES <<<<<<<<<<<<<<<<<<<<<<
c
c++++++++++++++++++++++++ GRFX03 (DAC) ROUTINES ++++++++++++++++++++++++
c
c=======================================================================
c
       subroutine axislab ( xory, wxmin, wxmax, wymin, wymax, scale
     1                    , wconv, nchmax, clab, fact, iexp )
c
c    dac:grfx03.axislab <----------------------------------- labels axis
c                                                             june, 1996
c
c    written by: David Clarke
c    modified 1: December 1996, by David Clarke; added multiple plot per
c                frame capability.
c    modified 2: June, 2003, by David Clarke; generalised for use by 2-D
c                plotter as well.
c    modified 3: October 2003 by Alexander Men'shchikov; generalized to
c                include the PSPLOT graphics library.
c    modified 5: June, 2014 by DAC; introduced "fact" to scale down 
c                annotation sizes for long, short plots.
c
c  PURPOSE: This subroutine labels the axis.
c
c  INPUT VARIABLES:
c    xory        =1 => label the x-axis
c                =2 => label the y-axis
c    wxmin       minimum x-window coordinate
c    wxmax       maximum x-window coordinate
c    wymin       minimum y-window coordinate
c    wymax       maximum y-window coordinate
c    scale       scaling factor for multiple plots per frame
c    wconv       window conversion factor.  1-D plots are done in window
c                coordinates, and thus wconv = 1.0.  2-D plots are done
c                in grid coordinates, and thus wconv = width of window
c                in grid units.
c    nchmax      maximum number of characters in y-annotation (needed
c                so the y-axis label won't overwrite the annotations).
c    clab        character string containing the label.  Should be
c                written in `NCAR markup language'.
c    fact        scale factor of annotation size for long, short plots
c    iexp        .eq. 0 => ignored
c                .ne. 0 => "iexp" is the exponent of the annotations
c                          expressed in scientific notation and is
c                          incorporated into the axis label.
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    STRLEN
c    PSCHARS
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) clab
       character*34  clab2
       integer       xory    , iexp    , nchmax  , nch     , nch2
     1             , aexp
       real          xpos    , ypos    , angle   , wxmin   , wxmax
     1             , wymin   , wymax   , scale   , wlab    , wlab2
     2             , fnt1    , fnt2    , wconv   , q1      , q2
     3             , fact    , scfa    , scfa2
c
c      External statements
c
       integer       strlen
       external      strlen  , pschars
c
c-----------------------------------------------------------------------
c
       scfa  = scale * fact
       scfa2 = scale * fact**0.8
       fnt1  = 0.5 * scfa
       fnt2  = 0.4 * scfa
       q1    = wconv * scfa
       q2    = 0.02  * q1
c
c      Determine the number of characters in "clab".
c
       nch  = strlen ( clab )
       if (nch .eq. 0) return
c
c      If necessary, create a label for the axis scaling factor.
c
       wlab2 = 0.0
       if (iexp .ne. 0) then
         iexp = min ( 999, max ( -999, iexp ) )
         nch2 = 17
         clab2(1:nch2) = ' (|F18|U|F0|10|S|'
         if (iexp .lt. 0) then
           clab2(nch2+1:nch2+10) = '|F18|-|F0|'
           nch2 = nch2 + 10
         endif
         aexp = abs ( iexp )
         if (aexp .lt. 10) then
           write ( clab2(nch2+1:nch2+1), 2010 ) aexp
           nch2 = nch2 + 1
         endif
         if ( ( (aexp .ge. 10) .and. (aexp .lt. 99) ) ) then
           write ( clab2(nch2+1:nch2+2), 2020 ) aexp
           nch2 = nch2 + 2
         endif
         if (aexp .ge. 100) then
           write ( clab2(nch2+1:nch2+3), 2030 ) aexp
           nch2 = nch2 + 3
         endif
         clab2(nch2+1:nch2+4) = '|N|)'
         nch2 = nch2 + 4
       endif
c
c      Set coordinates and orientation of label, depending on whether
c  this is the x- or y-axis, and depending on whether the exponent of
c  the annotations has to be incorporated into the axis label.
c
       if (iexp .ne. 0.0) wlab2 = 4.5 * fnt2
       if (xory .eq. 1) then
         angle = 0.0
         xpos  = 0.5 * ( wxmax + wxmin - wlab2 ) + 0.1 * scfa
         ypos  = wymin - 1.6 * scfa2
       else
         angle = 90.0
         xpos  = wxmin - 0.75 * fnt1 * real ( nchmax ) - 0.15 * scale
         ypos  = 0.5 * ( wymax + wymin - wlab2 ) + 0.1 * scfa
       endif
c
c      Write the axis label.
c
       call pschars ( xpos, ypos, fnt1, clab(1:nch), angle, 1 )
       if (iexp .ne. 0) then
         call pschars ( 999.0, 999.0, fnt2, clab2(1:nch2), angle, 1)
       endif
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format ( i1 )
2020   format ( i2 )
2030   format ( i3 )
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine axisset1 ( xory, qmin, qmax, imm, aspect, ndec, iexp
     1                     , nchq, qqmin, qqmax, nqdiv, nqann, qann
     2                     , cqann)
c
c    dac:grfx03.axisset1 <------------------- parameters for linear axis
c                                                             june, 1996
c
c    written by: David Clarke
c    modified 2: July, 2010 by DAC; modified to give proper minus signs 
c                for PSPLOT, and upgraded over-crowding algorithm to 
c                take into account number of characters per annotation.
c
c  PURPOSE: This subroutine computes the parameters needed to draw a
c  linear axis.  Parameters for logarithmic axes are computed by
c  AXISSET2.  Constraints include:
c
c      1. annotation and tick spacings chosen so they are not crowded.
c      2. annotations will always be multiples of 1, 2, or 5.
c      3. annotations will be expressed in scientific notation if the
c         absolute value of the annotation is .le. 0.001 or .ge. 1000.
c      4. new extrema are chosen to be multiples of the tick spacing.
c      5. Axes are not extended to fall exactly on an annotated tick.
c      6. y-axes are extended by one minor tick for imm=1,-1.
c
c  INPUT VARIABLES:
c    xory        = 1 => x-axis
c                = 2 => y-axis
c                = 3 => polar axis
c    qmin        minimum value on axis.
c    qmax        maximum value on axis.
c    imm         = 1,-1 =>     extend grid by one minor tick mark
c                = 0 => do not extend grid by one minor tick mark
c    aspect      desired aspect ratio of axes.
c                < 1 => short, wide plot
c                = 1 => square plot
c                > 1 => tall, narrow plot
c    ndec        the number of significant figures to appear to the
c                right of the decimal in the annotations.
c                =-1 => minimum number of decimal places required to
c                       make each annotation unique.
c
c  OUTPUT VARIABLES:
c    iexp        If the absolute annotation values are .le. 0.001 or
c                .ge. 1000., "iexp" is the exponent of the annotations
c                expressed in scientific notation.  Otherwise, iexp = 0.
c    nchq        maximum number of characters in any annotation.
c    qqmin       greatest multiple of the major tick spacing < "qmin".
c    qqmax       smallest multiple of the major tick spacing > "qmax".
c    nqdiv       total number of divisions along axis (always > 0).
c    nqann       total number of annotations along axis.
c    qann        1-D real array containing the annotation locations
c                along the axis.
c    cqann       1-D character array (at least character*21) containing 
c                the axis annotations in NCAR markup language.
c
c  LOCAL VARIABLES:
c    ccqann      temporary 1-D character array.
c    mant        1-D integer array containing int(qann).
c    idec        1-D integer array containing the decimal portion of
c                "qann".
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       mann
       parameter     ( mann=20 )
c
       integer       xory    , ndec    , iexp    , nchq    , nqdiv
     1             , nqann   , i1      , i       , j       , jj
     2             , nmax    , imm     , ndeco   , nchars
       real          qmin    , qmax    , aspect  , qqmin   , qqmax
     1             , fact    , dq1     , dq2     , dq3     , dq4
     2             , dq5     , dq6     , qamin   , qamax   , sqmin
     3             , sqmax   , sqqmin  , sqqmax  , aqamax  , q1
     4             , q2      , qdec    , tick
c
       character*(*) cqann   (   *)
       character*16  ccqann  (mann)
       integer       mant    (mann), idec    (mann)
       real          qann    (   *)
c
c-----------------------------------------------------------------------
c
       ndeco = ndec
       if (xory .eq. 1) fact = 0.1 * max ( 1.0, aspect )
       if (xory .eq. 2) fact = 0.1 / min ( 1.0, aspect )
       if (xory .eq. 3) fact = 0.1 * max ( 1.0, aspect )
c
c      dq1    is the tentative spacing between annotations.
c      i1     is chosen so that 0.1 .lt. dq2 = dq1 * 10**(-i1) .le. 1.0
c      dq3    is the spacing between annotations (modulo a factor
c             10**i1)
c
       dq1   = ( qmax - qmin ) * fact
       i1    = int ( alog10 ( dq1 ) )
       if (dq1 .gt. 1.0) i1 = i1 + 1
       dq2   = dq1 * 10.0**(-i1)
       if (dq2 .le. 0.1) then
         i1  = i1 - 1
         dq2 = dq2 * 10.0
       endif
       dq3   = 1.0
       if ( (dq2 .gt. 0.2) .and. (dq2 .le. 0.5) ) dq3 = 0.5
       if ( (dq2 .gt. 0.1) .and. (dq2 .le. 0.2) ) dq3 = 0.2
c
c      dq4    is the final spacing between annotations (dq3 * 10**i1)
c      dq5    is the final spacing between ticks
c      qqmin  is the greatest multiple of dq5 less    than  qmin
c      qamin  is the smallest multiple of dq4 greater than qqmin
c      qqmax  is the smallest multiple of dq5 greater than  qmax
c      qamax  is the greatest multiple of dq4 less    than qqmax
c
10     continue
       dq4    = dq3 * 10.0**i1
       tick   = 5.0
       if ( dq3 .eq. 0.2 ) tick = 4.0
       dq5    = dq4 / tick
c
       sqmin  = sign ( 0.5 , qmin )
       sqmax  = sign ( 0.5 , qmax )
       qqmin  = real ( int ( qmin / dq5 - 0.499 + sqmin ) ) * dq5
       qqmax  = real ( int ( qmax / dq5 + 0.499 + sqmax ) ) * dq5
       if ( (xory .eq. 2) .and. (imm .ne. 0) ) then
         if ( qmin-qqmin .lt. 0.5*dq5) qqmin = qqmin - dq5
         if (qqmax- qmax .lt. 0.5*dq5) qqmax = qqmax + dq5
       endif
       if (xory .eq. 3) then
         if ( (qqmin .gt.  0.0) .and. (qqmin .lt.  5.0) ) qqmin =  0.0
         if ( (qqmax .gt. 85.0) .and. (qqmax .lt. 90.0) ) qqmax = 90.0
       endif
       sqqmin = sign ( 0.5, qqmin )
       sqqmax = sign ( 0.5, qqmax )
       qamin  = real ( int ( qqmin / dq4 + 0.499 + sqqmin ) ) * dq4
       qamax  = real ( int ( qqmax / dq4 - 0.499 + sqqmax ) ) * dq4
c
c      nqann  is the number of annotations along the axis
c      nqdiv  is the number of divisions along the axis (always > 0)
c
c      Note: if nqann > nmax, the value of dq3 is increased and
c  execution is returned to statement 10 above so that nqann is .le.
c  nmax.
c
       nqann = int ( ( qamax - qamin ) / dq4 + 0.01 ) + 1
       nmax  = int ( 8.0 / max ( 1.0, aspect ) + 0.5 )
       if (xory .eq. 2)
     1   nmax = int ( 8.0 * min ( 1.0, aspect ) + 0.5 )
       if (nqann .gt. nmax) then
         if (dq3 .eq. 0.2) then
           dq3 = 0.5
           go to 10
         endif
         if (dq3 .eq. 0.5) then
           dq3 = 1.0
           go to 10
         endif
         if (dq3 .eq. 1.0) then
           dq3 = 0.2
           i1 = i1 + 1
           go to 10
         endif
       endif
       nqdiv = int ( ( qqmax - qqmin ) / dq5 + 0.01 )
c
c      If the largest absolute value at a major tick mark (aqmax) is
c  greater than 1000 or less than 0.01, the axis values are printed in
c  scientific notation where the mantissa is written next to the major
c  tick mark and the exponent (q1 = 10**iexp) is given as part of the
c  axis label.
c      q2     is the reciprocal of q1.
c      dq6    is the spacing between annotations, modulo q2
c
       aqamax = max ( abs(qamax), abs(qamin) )
       iexp  = 0
       if (aqamax .ge. 1000.) iexp = int ( alog10 ( aqamax ) )
       if (aqamax .lt. 0.01 ) iexp = int ( alog10 ( 0.1001 * aqamax ) )
       q1    = 10.0**iexp
       q2    = 10.0**(-iexp)
       dq6   = dq4 * q2
c
c      ndec   is the number of decimal places to appear in the mantissa.
c
c      If ndec=-1, then the routine automatically selects ndec to the
c  minimum number required for all annotations along the axis to be
c  different.  e.g., for the values 1.210, 1.220, 1.230, 1.240,
c  1.250, ndec should be 2.  If it were 1, then all values would appear
c  as 1.2.
c      If the automatic routine doesn't work for some pathelogical case,
c  or if the user requires additional zeroes after the last decimal
c  place chosen by the routine, then specifying ndec will override the
c  automatic routine.  Note that ndec can be no larger than 6.
c
       if (ndeco .eq. -1) ndec =-int ( alog10 ( dq6 ) - 0.7 )
       ndec = max ( 0, min ( 6, ndec ) )
       if ( (aqamax .lt. 20.0) .and. (ndec .eq. 0) ) ndec = 1
c
c      qann   is a real 1-D array with the annotations for the axis.
c
       qann(1) = qamin * q2
       do 20 i=2,nqann
         qann(i) = qann(i-1) + dq6
20     continue
c
c      The annotations are converted to character strings with all the
c  machine round-off noise cleaned out.  This is done by separating the
c  portions to the left (mant) and right (idec) of the decimal point,
c  then formatting the character string according to the value of
c  "ndec".  Loop 40 squeezes out extraneous blanks.
c
       do 30 i=1,nqann
         mant(i) = int ( abs ( qann(i) ) + 0.000005 )
         qdec    = abs ( qann(i) ) - real ( mant(i) )
         idec(i) = int ( 0.1 + 10.0**ndec * qdec )
         qann(i) = qann(i) * q1
30     continue
       nchq = 1
       do 50 i=1,nqann
         if (ndec .eq. 0) write (ccqann(i),2010) mant(i)
         if (ndec .eq. 1) write (ccqann(i),2020) mant(i), idec(i)
         if (ndec .eq. 2) write (ccqann(i),2030) mant(i), idec(i)
         if (ndec .eq. 3) write (ccqann(i),2040) mant(i), idec(i)
         if (ndec .eq. 4) write (ccqann(i),2050) mant(i), idec(i)
         if (ndec .eq. 5) write (ccqann(i),2060) mant(i), idec(i)
         if (ndec .eq. 6) write (ccqann(i),2070) mant(i), idec(i)
         cqann(i) = ' '
         jj = 0
         if (qann(i) .lt. 0.0) then
           cqann(i)(1:10) = '|F18|-|F0|'
           jj = 10
         endif
         do 40 j=1,16
           if (ccqann(i)(j:j) .ne. ' ') then
             jj = jj + 1
             cqann(i)(jj:jj) = ccqann(i)(j:j)
           endif
40       continue
         if (qann(i) .lt. 0.0) jj = jj - 9
         nchq = max ( nchq, jj )
50     continue
c
c      To avoid x-axis crowding, count the total number of characters 
c  (excluding decimal points) to be printed along the x-axis.  If this
c  is greater than 30, reset "dq3" and return to statement 10 to try
c  again.
c
       nchars = nqann * ( nchq - 1 )
       if ( (xory .eq. 1) .and. (nchars .gt. 30) ) then
         if (dq3 .eq. 0.2) then
           dq3 = 0.5
           go to 10
         endif
         if (dq3 .eq. 0.5) then
           dq3 = 1.0
           go to 10
         endif
         if (dq3 .eq. 1.0) then
           dq3 = 0.2
           i1  = i1 + 1
           go to 10
         endif
       endif
c
c      "nchq" is returned so that the y-axis label may be positioned 
c  just left of the left-most character in the annotations.
c
       nchq = nchq + 1
       if (ndec .eq. 0) nchq = nchq + 1
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format(i4,12x)
2020   format(i4,'.',i1.1,10x)
2030   format(i4,'.',i2.2, 9x)
2040   format(i4,'.',i3.3, 8x)
2050   format(i4,'.',i4.4, 7x)
2060   format(i4,'.',i5.5, 6x)
2070   format(i4,'.',i6.6, 5x)
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine axisset2 ( xory, qmin, qmax, yexp, aspect, nchq, qqmin
     1                     , qqmax, nqdiv, nqann, qann, cqann )
c
c    dac:grfx03.axisset2 <-------------- parameters for logarithmic axis
c                                                             july, 1996
c
c    written by: David Clarke
c    modified 2: July, 2010 by DAC; modified to give proper minus signs 
c                for PSPLOT.
c
c  PURPOSE: This subroutine computes the parameters needed to draw a
c  logarithmic axis.  Parameters for linear axes are computed by
c  AXISSET1.  Constraints include:
c
c      1. annotation and tick spacings chosen so they are not crowded.
c      2. annotations will always be powers of 10.
c      3. annotations will be expressed in scientific notation if the
c         absolute value of the annotation is .le. 0.001 or .ge. 1000.
c      4. new extrema are chosen to be multiples of the tick spacing.
c      5. Axes are not extended to fall exactly on an annotated tick.
c
c  INPUT VARIABLES:
c    xory        = 1 => x-axis
c                = 2 => y-axis
c    qmin        minimum value on axis.
c    qmax        maximum value on axis.
c    yexp        log10 of scaling factor generated when y was converted
c                to single precision
c    aspect      desired aspect ratio of axes.
c                < 1 => short, wide plot
c                = 1 => square plot
c                > 1 => tall, narrow plot
c
c  OUTPUT VARIABLES:
c    nchq        maximum number of characters in any annotation.
c    qqmin       location of first tick mark.
c    qqmax       location of last  tick mark.
c    nqdiv       >0 => one less than the total number of decades along
c                      axis.  These will mark location of minor ticks.
c                =0 => minor ticks will mark units within each decade.
c    nqann       total number of annotations along axis.
c    qann        1-D real array containing the annotation locations
c                along the axis.
c    cqann       1-D character array (at least character*21) containing 
c                the axis annotations in NCAR markup language.
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       mann
       parameter     ( mann=20 )
c
       integer       xory    , yexp    , nchq    , nqdiv   , nqann   
     1             , i1      , i2      , i3      , i4      , i5      
     2             , i6      , i7      , i8      , dnq     , i
     3             , n
       real          qmin    , qmax    , aspect  , qqmin   , qqmax
     1             , fact    , ri5     , ri6     , rdnqi
c
       character*(*) cqann   (   *)
       real          qann    (   *)
c
c-----------------------------------------------------------------------
c
c      i1     is the highest power of ten less than "qmin".
c      i2     is a multiple of 10**i1 less than "qmin".
c      qqmin  is the log of the location of the first tick mark
c
       i1    = int ( alog10 ( qmin * 1.01 ) )
       if (qmin .lt. 1.0) i1 = i1 - 1
       i1    = max ( -32, min ( 32, i1 ) )
       i2    = int ( qmin * 10.0**(-i1) + 0.01 )
       if (i2 .ge. 8) i2 = i2 - 1
       if (i2 .ge. 6) i2 = i2 - 1
       if (i2 .ge. 4) i2 = i2 - 1
       qqmin = alog10 ( real ( i2 ) ) + real ( i1 )
c
c      i3     is the highest power of ten less than "qmax".
c      i4     is a multiple of 10**i3 greater than "qmax".
c      qqmax  is the log of the location of the last tick mark
c
       i3    = int ( alog10 ( qmax * 0.99 ) )
       if (qmax .lt. 1.0) i3 = i3 - 1
       i3    = max ( -32, min ( 32, i3 ) )
       i4    = int ( qmax * 10.0**(-i3) + 1.01 )
       if (i4 .ge. 8) i4 =            i4 + 1
       if (i4 .ge. 6) i4 = min ( 10, i4 + 1 )
       if (i4 .ge. 4) i4 = min ( 10, i4 + 1 )
       qqmax = alog10 ( real ( i4 ) ) + real ( i3 )
c
c      If "qqmin" and "qqmax" fall within the same decade, reset them to
c  the extremes of that decade.
c
       if (i3 .eq. i1) then
         qqmin = real ( i1     )
         qqmax = real ( i1 + 1 )
       endif
c
c      i5     is the smallest integer greater than "qqmin"
c      i6     is the largest  integer less    than "qqmax"
c
       i5    = int ( qqmin - 0.001 )
       if (qqmin .gt. 0.0) i5 = i5 + 1
       i6    = int ( qqmax + 0.001 )
       if (qqmax .lt. 0.0) i6 = i6 - 1
c
c      To avoid over-crowded annotations, only every "dnq"'th decade is
c  annotated.  If there are fewer than 8 or 9 decades between "qqmin"
c  and "qqmax", "dnq" is 1 and minor ticks mark off units within each
c  decade.  If there are more than 8 or 9 decades, "dnq" is greater than
c  1 and minor ticks mark off non-annotated decades.
c
c      i5     is the log of the location of the first annotation.
c      i6     is the log of the location of the last  annotation.
c
       if (xory .eq. 1) fact = max ( 1.0, aspect ) / 8.0
       if (xory .eq. 2) fact = 0.1 / min ( 1.0, aspect )
       dnq = int ( real ( i6 - i5 ) * fact ) + 1
       if (dnq .gt. 1) then
         ri5   = real ( i5 )
         ri6   = real ( i6 )
         rdnqi = 1.0 / real ( dnq )
         i5    = int ( ri5 * rdnqi + sign ( 0.001, ri5 ) ) * dnq
         i6    = int ( ri6 * rdnqi + sign ( 0.999, ri6 ) ) * dnq
         qqmin = real ( i5 )
         qqmax = real ( i6 )
         nqann = ( i6 - i5 ) / dnq + 1
c        nqdiv = i2 - i1
         nqdiv = i6 - i5
       else
         nqann = i6 - i5 + 1
         nqdiv = 0
       endif
c
c      Create real and character annotations for the axis.
c
       i5    = i5 + yexp
       i6    = i6 + yexp
       i7    = i5
       nchq  = 4
       do 10 i=1,nqann
         qann (i) = real ( i7 - yexp )
         cqann(i) = ' '
         i8 = abs ( i7 )
         if (i8 .le. 2) then
           if (i7 .eq. -2) then
             cqann(i)(1:4) = '0.01'
             nchq = max ( nchq, 5 )
           endif
           if (i7 .eq. -1) cqann(i)(1:3) = '0.1'
           if (i7 .eq.  0) cqann(i)(1:3) = '1.0'
           if (i7 .eq.  1) then
             cqann(i)(1:4) = '10.0'
             nchq = max ( nchq, 5 )
           endif
           if (i7 .eq.  2) then
             cqann(i)(1:5) = '100.0'
             nchq = max ( nchq, 6 )
           endif
         else
           cqann(i)(1:5) = '10|S|'
           n = 5
           if (i7 .lt. 0) then
             cqann(i)(6:15) = '|F18|-|F0|'
             n = 15
           endif
           if (i8 .le. 9) write ( cqann(i)(n+1:n+1), '(i1)' ) i8
           if ( (i8 .ge. 10) .and. (i8 .le. 99) )
     1       write ( cqann(i)(n+1:n+2), '(i2)' ) i8
           if (i8 .ge. 100) write ( cqann(i)(n+1:n+3), '(i3)' ) i8
         endif
         i7 = i7 + dnq
10     continue
       if ( (i5 .lt.    0) .or. (i6 .ge.  10) ) nchq = max ( nchq, 5 )
       if ( (i5 .le.  -10) .or. (i6 .ge. 100) ) nchq = 6
       if   (i5 .le. -100)                      nchq = 7
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine grfxaxes ( yexp, xlab, ylab, imm, xmin, xmax, ymin
     1                     , ymax, aspect, p1x0, p1y0, allxl, allyl
     2                     , np1h, ip1h, np1v, ip1v, lnlg )
c
c    dac:grfx03.grfxaxes <---------------------- draws axes for graphics
c                                                             june, 1996
c
c    written by: David Clarke
c    modified 1: December 1996, by David Clarke; added multiple frame 
c                per page capability.
c    modified 2: October 2003 by Alexander Men'shchikov; generalized to
c                include the PSPLOT graphics library.
c    modified 3: June, 2010 by DAC; moved determination of extrema and 
c                log-plot checks to GRFX1D.
c
c  PURPOSE: This subroutine draws and labels the axes (perimeter style)
c  for 1-D PSPLOT graphics.
c
c  INPUT VARIABLES:
c    yexp        log10 of scaling factor generated when y was converted
c                to single precision
c    xlab        character string for x-axis label (NCAR markup 
c                language).
c    ylab        character string for y-axis label (ditto).
c    imm         = 1,-1 =>     extend grid by one minor tick mark
c                = 0 => do not extend grid by one minor tick mark
c    xmin        minimum value on the x-axis
c    xmax        maximum value on the x-axis
c    ymin        minimum value on the y-axis
c    ymax        maximum value on the y-axis
c    aspect      desired aspect ratio of axes.
c                < 1 => short, wide frame
c                = 1 => square frame
c                > 1 => tall, narrow frame
c    p1x0        x-coordinate of the lower left corner of the frame
c    p1y0        y-coordinate of the lower left corner of the frame
c    allxl       =2 => all x labels/annotations are drawn (default)
c                =1 => only the x-label/annotations on the bottom row
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the bottom row.
c                if allxl < 0, same as abs(allxl) except last annotation
c                is not drawn if it falls at xxmax.
c    allyl       =2 => all y labels/annotations are drawn (default)
c                =1 => only the y-label/annotations on the left column
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the left column.
c                if allyl < 0, same as abs(allyl) except last annotation
c                is not drawn if it falls at yymax.
c    np1h        number of frames to be arranged horizontally per page
c    ip1h        in the current row   , this is the ip1h'th frame
c    np1v        number of frames to be arranged vertically   per page
c    ip1v        in the current column, this is the ip1v'th frame
c    lnlg        = 1 => linear x and y;    = 2 => linear x, log y
c                = 3 => log x, linear y;   = 4 => log x and y
c
c  OUTPUT VARIABLES (via common block win1d):
c    scale       scaling factor for multiple frames per page
c    wxmin, etc. coordinates of plotting window in graphical units
c    xxmin, etc. coordinates of plotting window in "real" units after 
c                adjustments to "xmin", etc. have been made to allow x- 
c                and y-axes to start and end on major/minor tick marks
c
c  LOCAL VARIABLES:
c    cxann       1-D character array containing all x-axis annotations
c                in NCAR markup language.
c    cyann       1-D character array containing all y-axis annotations
c                in NCAR markup language.
c
c  EXTERNALS:
c    AXISSET1, AXISSET2
c    STRLEN
c    PSCHARS
c    AXISLAB
c    SLDLIN     )---- PSPLOT routine
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       mann
       real          small
c       parameter     ( mann=20, small=0.001 )
       parameter     ( mann=20, small=0.05 )
c
       character*(*) xlab    , ylab
       integer       yexp    , imm     , allxl   , allyl   , np1h    
     1             , ip1h    , np1v    , ip1v    , lnlg    
     2             , aallxl  , aallyl  , ixsen   , iysen   
     3             , ndec    , iend    , i       , nch     , iexp    
     3             , nchmax  , ndiv    , nann    , i1      , ibeg
       real          xmin    , xmax    , ymin    , ymax    , aspect 
     1             , p1x0    , p1y0    , phw     , wx      , dwx     
     2             , dwx1    , dwx2    , wy      , dwy     , dwy1    
     3             , dwy2    , fact    , q1      , q2      , q3      
     4             , xx      , dxx     , yy      , dyy
       real          wxmin   , wxmax   , wymin   , wymax   , xxmin
     1             , xxmax   , yymin   , yymax   , scale
c
       character*32  cxann   (mann), cyann   (mann)
       real          xann    (mann), yann    (mann)
c
       common      / win1d /
     1               wxmin   , wxmax   , wymin   , wymax   , xxmin
     2             , xxmax   , yymin   , yymax   , scale
c
c      External statements
c
       integer       strlen
       external      axisset1, axisset2, strlen  , pschars , sldlin  
     1             , axislab
c
c-----------------------------------------------------------------------
c
c      Set axes sentinels.  If i*sen=1, *=x,y, *-label and annotations
c  are drawn.
c
       aallxl = abs ( allxl )
       ixsen  = 0
       if   (aallxl .eq. 2)                           ixsen = 1
       if ( (aallxl .eq. 1) .and. (ip1v  .eq. np1v) ) ixsen = 1
       aallyl = abs ( allyl )
       iysen  = 0
       if   (aallyl .eq. 2)                           iysen = 1
       if ( (aallyl .eq. 1) .and. (ip1h  .eq. 1   ) ) iysen = 1
c
c-----------------------------------------------------------------------
c----------------  W I N D O W   C O O R D I N A T E S  ----------------
c-----------------------------------------------------------------------
c
c      Compute window coordinates ("wxmin", etc.) and the scaling factor 
c  ("scale"), all in common block "win1d" declared also by GRFX1D.  
c
       scale = 1.0
       wxmin = p1x0
       wxmax = p1x0 + 10.0
       wymin = p1y0
       wymax = p1y0 + 10.0
       phw   = 5.0
c
       if (aspect .gt. 1.0) then
         wxmin = wxmin + phw * ( 1.0 - 1.0 / aspect )
         wxmax = wxmax - phw * ( 1.0 - 1.0 / aspect )
       endif
c
       if (aspect .lt. 1.0) then
         wymin = wymin + phw * ( 1.0 - aspect )
         wymax = wymax - phw * ( 1.0 - aspect )
       endif
c
c      Adjust for multiple frames per page.  The factor of 17.0 in 
c  "scale" allows room for the footer to be put at the bottom of the 
c  page in GRFX1D.
c
       dwx = 0.0
       dwy = 0.0
       if (np1h*np1v .gt. 1) then
         dwx   = 3.5
         dwy   = 2.2
         dwx1  = ( wxmax - wxmin + dwx ) * real ( np1h )
         dwy1  = ( wymax - wymin + dwx ) * real ( np1v )
         scale = 17.0 / max ( dwx1, dwy1 )
         if (aallyl .le. 1) then
           dwx  = 0.3
           dwx1 = ( wxmax - wxmin + dwx ) * real ( np1h ) + 3.2
         endif
         if (aallxl .le. 1) then
           dwy  = 0.3
           dwy1 = ( wymax - wymin + dwx ) * real ( np1v ) + 1.9
         endif
         wx    = 12.8
         wy    = 12.5
         dwx1  = ( wxmax - wxmin + dwx ) * scale
         dwx2  = ( wxmax - wxmin       ) * scale
         wxmax = wx - dwx1 * real ( np1h - ip1h ) + p1x0
         dwy1  = ( wymax - wymin + dwy ) * scale
         dwy2  = ( wymax - wymin       ) * scale
         wymax = wy - dwy1 * real ( ip1v - 1    ) + p1y0
         wxmin = wxmax - dwx2
         wymin = wymax - dwy2
c
       endif
c
c-----------------------------------------------------------------------
c------------------------  T H E   X - A X I S  ------------------------
c-----------------------------------------------------------------------
c
c      Set parameters for the x-axis.
c
       ndec =-1
       if (lnlg .le. 2) then
         call axisset1 ( 1, xmin, xmax, imm, aspect, ndec, iexp, nchmax
     1                 , xxmin, xxmax, ndiv, nann, xann, cxann )
       else
         call axisset2 ( 1, xmin, xmax, 0, aspect, nchmax
     1                 , xxmin, xxmax, ndiv, nann, xann, cxann )
         iexp = 0
       endif
c
c      Write the x-axis annotations and draw the annotated ticks.
c
       fact = ( wxmax - wxmin ) / ( xxmax - xxmin )
       wy   = wymin - 0.75  * scale
       q1   = 0.06 * scale
       q2   = 0.45 * scale
       q3   = 0.38 * scale
c
       iend = nann
       if (allxl .lt. 0) then
         q1 =  abs ( xxmax - xann(nann) ) / ( xxmax - xxmin )
         if (q1 .lt. small) iend = nann - 1
       endif
       do 20 i=1,iend
         wx = wxmin + ( xann(i) - xxmin ) * fact - q1
         if (ixsen .eq. 1) then
           nch = strlen ( cxann(i) )
           call pschars ( wx, wy, q2, cxann(i)(1:nch), 0.0, 1 )
         endif
         wx  = wx + q1
         call sldlin ( wx, wymin, wx, wymin+q3, 0.0 )
         call sldlin ( wx, wymax, wx, wymax-q3, 0.0 )
20     continue
c
c      Draw the unannotated ticks.
c
       q1 = 0.25  * scale
       if (ndiv .gt. 0) then
         dxx = ( xxmax - xxmin ) / real ( ndiv )
         do 30 i=2,ndiv
           xx = real ( i - 1 ) * dxx
           wx = wxmin + xx * fact
           call sldlin ( wx, wymin, wx, wymin+q1, 0. )
           call sldlin ( wx, wymax, wx, wymax-q1, 0. )
30       continue
       else
         xx = xxmin
40       continue
         i1 = int ( xx - 0.499 + sign ( 0.5, xx ) )
         xx = alog10 ( 10.0**xx + 10.0**i1 )
         if (xx .lt. xxmax) then
           wx = wxmin + ( xx - xxmin ) * fact
           call sldlin ( wx, wymin, wx, wymin+q1, 0. )
           call sldlin ( wx, wymax, wx, wymax-q1, 0. )
           go to 40
         endif
       endif
c
c      Draw and label the axis.
c
       call sldlin ( wxmin, wymin, wxmax, wymin, 0. )
       call sldlin ( wxmin, wymax, wxmax, wymax, 0. )
       if (ixsen .eq. 1)
     1   call axislab ( 1, wxmin, wxmax, wymin, wymax, scale, 1.0
     2                , nchmax, xlab, 1.0, iexp )
c
c-----------------------------------------------------------------------
c------------------------  T H E   Y - A X I S  ------------------------
c-----------------------------------------------------------------------
c
c      Set the parameters for the y-axis.
c
       ndec =-1
       if ( (lnlg .eq. 1) .or. (lnlg .eq. 3) ) then
         call axisset1 ( 2, ymin, ymax, imm, aspect, ndec, iexp, nchmax
     1                 , yymin, yymax, ndiv, nann, yann, cyann )
         iexp = iexp + yexp
       else
         call axisset2 ( 2, ymin, ymax, yexp, aspect, nchmax
     1                 , yymin, yymax, ndiv, nann, yann, cyann )
         iexp = 0
       endif
c
c      Write the y-axis annotations and draw the annotated ticks.  The
c  first y-axis annotation is skipped if both it and the first x-axis
c  annotation are at the origin.
c
       wx   = wxmin - 0.20  * scale
       fact = ( wymax - wymin ) / ( yymax - yymin )
       q1   = abs ( xann(1) - xxmin ) / ( xxmax - xxmin )
       q2   = abs ( yann(1) - yymin ) / ( yymax - yymin )
       ibeg = 1
       if ( (q1 .le. small) .and. (q2 .le. small) ) ibeg = 2
       q2   = 0.45 * scale
       q3   = 0.38 * scale
c
       iend = nann
       if (allyl .lt. 0) then
         q1 =  abs ( yymax - yann(nann) ) / ( yymax - yymin )
         if (q1 .lt. small) iend = nann - 1
       endif
       do 90 i=ibeg,iend
         wy = wymin + ( yann(i) - yymin ) * fact
         if (iysen .eq. 1) then
           nch = strlen ( cyann(i) )
           call pschars ( wx, wy-q2/2.0, q2, cyann(i)(1:nch), 0.0, 2 )
         endif
         call sldlin ( wxmin, wy, wxmin+q3, wy, 0.0 )
         call sldlin ( wxmax, wy, wxmax-q3, wy, 0.0 )
90     continue
c
c      Draw the unannotated ticks.
c
       q1 = 0.25  * scale
       if (ndiv .gt. 0) then
         dyy = ( yymax - yymin ) / real ( ndiv )
         do 100 i=2,ndiv
           yy = real ( i - 1 ) * dyy
           wy = wymin + yy * fact
           call sldlin ( wxmin, wy, wxmin+q1, wy, 0. )
           call sldlin ( wxmax, wy, wxmax-q1, wy, 0. )
100      continue
       else
         yy = yymin
110      continue
         i1 = int ( yy - 0.499 + sign ( 0.5, yy ) )
         yy = alog10 ( 10.0**yy + 10.0**i1 )
         if (yy .lt. yymax) then
           wy = wymin + ( yy - yymin ) * fact
           call sldlin ( wxmin, wy, wxmin+q1, wy, 0. )
           call sldlin ( wxmax, wy, wxmax-q1, wy, 0. )
           go to 110
         endif
       endif
c
c      Draw and label the axis.
c
       call sldlin ( wxmin, wymin, wxmin, wymax, 0. )
       call sldlin ( wxmax, wymin, wxmax, wymax, 0. )
       if (iysen .eq. 1)
     1   call axislab ( 2, wxmin, wxmax, wymin, wymax, scale, 1.0
     2                , nchmax, ylab, 1.0, iexp )
c
c-----------------------------------------------------------------------
c
c      Final adjustments for log axes.
c
       if (lnlg .ge. 3) then
         xxmin = 10.0**xxmin
         xxmax = 10.0**xxmax
       endif
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) ) then
         yymin = 10.0**yymin
         yymax = 10.0**yymax
       endif
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine pschars ( x, y, fsize, chars, ang, just )
c
c    abm:grfx03.pschars <------------- emulate NCAR typesetting language
c                                                          october, 2003
c
c    written by: Alexander Men'shchikov
c    modified 1: February, 2005 by David Clarke, generalised for format 
c                statements 2050 and 2060 in GRFXCNTR.
c    modified 2: July, 2010 by DAC; completely rewritten to make it more 
c                readable and robust, and to enable sub-sub, sub-super, 
c                super-sub, and super-super scripting.
c
c  PURPOSE: This subroutine emulates the NCAR library's typesetting
c  language used to draw labels and legends in plots, and is for use 
c  with the PSPLOT graphics library.
c
c  NCAR markup language:
c
c    Fonts: |F0|  = times-roman (default)
c           |F10| = times-roman italic
c           |F14| = times-roman italic bold
c           |F5|  = greek italic
c           |F18| = math characters
c           |F34| = more math characters
c
c    Position: |B| => subscript; |S| => superscript; |N| => normal
c
c    One may combine one position designator with one font change in
c    each "field" (bounded by ||), and then string as many fields
c    together as one needs
c
c  Note: this routine is not general, and works based on several 
c  formatting assumptions.  
c
c  1.  Position control characters (B, S, N) may appear in the same 
c      control sequence as the font specifier (e.g., F0) but must appear 
c      before the font specifier.  Thus, |BF0| is OK, |F0B| is not.  
c
c  2.  There may be one S, one B, or one or two Ns before a font 
c      specifier.  Thus, |SF0| and |NNF0| are OK, |NSF0| is not.
c
c  3.  Some key characters are sensitive to extraneous blanks.  Thus, 
c      and for example, |F18|U gives a multiplication symbol, |F18| U 
c      gives a 'Y'.
c
c  INPUT VARIABLES:
c    x,y        position coordinates (in grid units) of the first 
c               character sequence printed.  (999.,999.) => current pen 
c               position.
c    fsize      font size (in cm?)
c    chars      character string (in NCAR markup language) to be parsed 
c               and interpreted
c    ang        angle of text.  Currently, allowed angles are 0 (x-axis) 
c               and 90 (y-axis).  With a little work, any angle could be 
c               interpreted as PSPLOT doesn't restrict angles to 0 & 90.
c    just       =0 => (x,y) at left side of "text"
c               =1 => (x,y) at centre of "text"
c               =2 => (x,y) at right side of "text"
c               where "text" is the entire character string for (x,y)
c              .ne. (999.,999.), only the first character "token" 
c               otherwise.
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    STRLEN
c    KEKSYM   )
c    SUPER    )---- PSPLOT routines
c    SUBBER   )
c    SETFNT   )
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       nlim
       parameter     ( nlim=100 )
c
       character*(*) chars
       character*128 word
       integer       i       , j       , k       , n       , nch
     1             , nch2    , i1      , i2      , nfont   , nfnt    
     2             , just    , k0      , kmax    , nmax    , lev
       real          x       , y       , xi      , yi      , fsize
     1             , fs      , ang     , width   , fact
c
       character*128 control (nlim), token   (nlim)
       integer       tlen    (nlim), idc     (nlim), level   (nlim)
c
c      External statements
c
       integer       strlen
       external      strlen  , keksym  , super   , subber  , setfnt
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nlim
         control(i) = ''
         token  (i) = ''
         tlen   (i) = 0
10     continue
c
c      Find the location of all delineation characters (|) in the 
c  string.
c
       nch = strlen ( chars )
       n   = 0
       do 20 i=1,nch
         if (chars(i:i) .eq. '|') then
           n      = n + 1
           idc(n) = i
         endif
20     continue
       nmax = n
       if (nmax .eq. 0) then
c
c      If no delineation characters were found, assume the entire string 
c  is to be printed as is at the default font (F0 => nfnt=24).
c
         control(1) = 'F0'
         tlen   (1) = nch
         token  (1) = chars(1:nch)
         kmax       = 1
c
       else
c
c      If the first character (including blanks!) is not a |, assume 
c  this string is the first token to be printed at the default font, F0.
c
         k0 = 0
         if (idc(1) .gt. 1) then
           control(1) = 'F0'
           tlen   (1) = idc(1)-1
           token  (1) = chars(1:tlen(1))
           k0         = 1
         endif
c
c      For syntactically correct NCAR markup language, control codes 
c  follow an odd number of delineation characters, while character 
c  substrings to be printed (tokens) follow an even number of 
c  delineation characters.
c
         do 30 n=1,nmax
           k  = n/2
           i1 = idc(n) + 1
           i2 = nch
           if (n .lt. nmax) i2 = idc(n+1) - 1
           if (i2 .ge. i1) then
             if (n .eq. 2*k) then
               token  (k+k0  ) = chars(i1:i2)
               tlen   (k+k0  ) = i2 - i1 + 1
             else
               control(k+k0+1) = chars(i1:i2)
             endif
           endif
30       continue
         kmax = nmax/2 + k0
c
c      Squeeze out any blanks that may be in the control sequences.
c
         do 50 k=1,kmax
           write ( word, '(64x)' )
           i   = 0
           nch2 = strlen ( control(k) )
           do 40 n=1,nch2
             if (control(k)(n:n) .ne. ' ') then
               i = i + 1
               word(i:i) = control(k)(n:n)
             endif
40         continue
           control(k) = word
           if ( (word(1:1) .ne. 'F') .and. (word(1:1) .ne. 'N') .and. 
     1          (word(1:1) .ne. 'S') .and. (word(1:1) .ne. 'B') ) then
             write (6, 2010) word(1:i), chars(1:nch)
             control(k) = 'NF0'
           endif
50       continue
c
       endif
c
c      Determine the text level (sub-superscripting) applicable for each 
c  token.  Text can be at any one of seven positions:
c
c  lev = -3 => sub-subscript                     1 => super-subscript
c        -2 => subscript         0 => normal     2 => superscript
c        -1 => sub-superscrpt                    3 => super-superscript
c
       lev = 0
       do 60 k=1,kmax
         if (control(k)(1:1) .eq. 'N') then
           if (abs(lev) .eq. 2) lev = 0
           if ( (lev .eq. 1) .or. (lev .eq. 3) ) lev = 2
           if ( (lev .eq.-1) .or. (lev .eq.-3) ) lev =-2
         endif
         if (control(k)(2:2) .eq. 'N') then
           if (abs(lev) .eq. 2) lev = 0
           if ( (lev .eq. 1) .or. (lev .eq. 3) ) lev = 2
           if ( (lev .eq.-1) .or. (lev .eq.-3) ) lev =-2
         endif
         if (control(k)(1:1) .eq. 'B') then
           if (abs(lev) .eq. 2) lev = lev - 1
           if (lev .eq. 0) lev =-2
         endif
         if (control(k)(1:1) .eq. 'S') then
           if (abs(lev) .eq. 2) lev = lev + 1
           if (lev .eq. 0) lev = 2
         endif
         level(k) = lev
60     continue
c
c      The first token is centred according to "just" after which the 
c  remaining tokens are appended.  This is fine for left-justification
c  but not for centring or right-justification.  For the latter two, it 
c  is necessary to adjust the initial coordinates (xi,yi), which is done 
c  by estimating the length of text to be printed, including an account 
c  of the smaller characters that may be sub-superscripted.
c
       xi = x
       yi = y
       if (kmax .gt. 1) then
         width = 0.0
         do 70 k=2,kmax
           fact = 0.4
           if (    level(k)  .eq. 0) fact = 0.9
           if (abs(level(k)) .eq. 2) fact = 0.6
           width = width + fact * fsize * real ( tlen(k) )
           do 61 i=1,tlen(k)
             if (token(k)(i:i) .eq. '.') 
     1         width = width - 0.5 * fact * fsize
61         continue
70       continue
         if ( (ang .eq. 0.0) .and. (x .ne. 999.0) ) then
           if (just .eq. 1) xi = x - width * 0.5
           if (just .eq. 2) xi = x - width
         endif
         if ( (ang .eq. 90.0) .and. (y .ne. 999.0) ) then
           if (just .eq. 1) yi = y - width * 0.5
           if (just .eq. 2) yi = y - width
         endif
       endif
c
c      Print each token with the specified font and at the specified 
c  sub-superscript level.
c
       do 80 k=1,kmax
c
c      Set font.
c
         if ( (control(k)(1:1) .eq. 'F') .or.
     1        (control(k)(2:2) .eq. 'F').or.
     1        (control(k)(3:3) .eq. 'F') ) then
c
           if (control(k)(1:1) .eq. 'F') then
             read (control(k)(2:),'(i2)') nfont
           else
             if (control(k)(2:2) .eq. 'F') then
               read (control(k)(3:),'(i2)') nfont
             else
               read (control(k)(4:),'(i2)') nfont
             endif
           endif
c
c      Substitution of characters that are different in NCAR and PSPLOT
c
           if ( (nfont .eq. 18) .and. (token(k)(1:1) .eq. 'R') ) then
             token(k) = '||'
             tlen (k) = 2
           endif
           if ( (nfont .eq. 18) .and. (token(k)(1:1) .eq. 'U') ) then
             token(k) = '264'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 18) .and. (token(k)(1:1) .eq. 'A') ) then
             token(k) = '136'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 34) .and. (token(k)(1:1) .eq. 'Q') ) then
             token(k) = '321'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 34) .and. (token(k)(1:1) .eq. 'W') ) then
             token(k) = '327'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 34) .and. (token(k)(1:1) .eq. '0') ) then
             token(k) = '260'
             tlen (k) = -999
           endif
c
           nfnt =-1
c
c      default is NewCenturySchlbk-Roman
c
           if (nfont .eq.  0) nfnt = 24
c
c      Italic --> NewCenturySchlbk-Italic
c
           if (nfont .eq. 10) nfnt = 23
c
c      Bold --> NewCenturySchlbk-Bold
c
           if (nfont .eq. 13) nfnt = 21
c
c      Bold italic --> NewCenturySchlbk-BoldItalic
c
           if (nfont .eq. 14) nfnt = 22
c
c      Greek --> Symbol
c
           if (nfont .eq.  5) nfnt = 29
c
c      Symbol Set 1 --> Symbol
c
           if (nfont .eq. 18) nfnt = 29
c
c      Math Symbols --> Symbol
c
           if (nfont .eq. 34) nfnt = 29
c
           if (nfnt .eq. -1) then
             write (6, 2020) nfont, chars(1:nch)
             nfnt = 24
           endif
           call setfnt ( nfnt )
c
         endif
c
c      Font size.
c
         fs = fsize
         if (abs(level(k)) .eq. 1) fs = 2.0 * fsize / 3.0
         if (abs(level(k)) .eq. 3) fs = 2.0 * fsize / 3.0
c
c      Additional positional shift for secondary sub-superscripting.
c
         fact = 1.0
         if (level(k) .eq.-3) fact = 2.5
         if (level(k) .eq.-1) fact = 0.08
         if (level(k) .eq. 1) fact = 0.8
         if (level(k) .eq. 3) fact = 2.5
c
c      Write text.
c
         if (level(k) .ge. 1) then
           call super  ( token(k), tlen(k), fs, ang, fact )
         endif
         if (level(k) .eq. 0) then
           call keksym ( xi, yi, fs, token(k), ang, tlen(k), just )
         endif
         if (level(k) .le.-1) then
           call subber ( token(k), tlen(k), fs, ang, fact )
         endif
c
         xi = 999.0
         yi = 999.0
80     continue
c
c      Restore default font.
c
       call setfnt ( 24 )
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format('PSCHARS : |---WARNING---> Unrecognised control '
     1       ,'sequence "',a,'" in input string:',/
     2       ,'PSCHARS : "',a,'"',/
     3       ,'PSCHARS : Recognised control sequences begin with "F", '
     4       ,'"N", "S", or "B".',/
     5       ,'PSCHARS : Resetting control sequence to "NF0"; '
     6       ,'execution continuing...')
2020   format('PSCHARS : |---WARNING---> Unknown font F',i2,' in input '
     1       ,'character string:',/
     2       ,'PSCHARS : "',a,'"',/
     3       ,'PSCHARS : Recognised fonts include F0, F5, F10, F13, '
     4       ,'F14, F18, and F34.',/
     5       ,'PSCHARS : Resetting to default font, F0; execution '
     6       ,'continuing...')
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       function strlen ( word )
c
c    dac:grfx03.strlen <--------------------- length of character string
c    from dac:editor.strlen                               february, 1992
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This function returns the number of characters in the
c  character string "word".  Note that PSPLOT has a function LENSTR with 
c  essentially the same purpose.
c
c  INPUT VARIABLES:
c    word        input character*(*) string
c
c  OUTPUT VARIABLES:
c    strlen      number of characters up to and including the last
c                non-blank character in "word"
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) word
       integer       strlen  , i       , imax
c
c-----------------------------------------------------------------------
c
       imax   = len ( word )
       strlen = 0
       do 10 i=1,imax
         if (word(i:i) .ne. ' ') strlen = i
10     continue
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine timedate ( chtime, chdate )
c
c    dac:plotit.timedate <------------------------ returns time and date 
c    from dac:zeus3d.wclock                                   june, 2018
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This routine returns the wall-clock time (hh:mm:ss) and date 
c  (dd/mm/yyyy) in character variables; a pared-down version of WCLOCK 
c  in dzeus36 meant only for gfortran, ifort, and pgfortran (not pgf77 
c  or pgf95).
c
c  INPUT VARIABLES:
c
c  OUTPUT VARIABLES:
c    chtime      time returned as hh:mm:ss
c    chdate      date returned as dd/mm/yyyy
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [none]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) chtime  , chdate
       character*10  chstr
       integer       year    , month   , day     , hour    , minute  
     1             , second  , ncht    , nchd    , nch     
c
       integer       values  (   8)
c
c-----------------------------------------------------------------------
c
c  DATE_AND_TIME returns eight integers in "values", where:
c
c  values(1)   the year (e.g., 2011)
c  values(2)   the month of the year (1-12)
c  values(3)   the day of the month (1-31)
c  values(4)   the number of minutes from GMT (e.g., -240 for Halifax)
c  values(5)   the hour of the day (0-23)
c  values(6)   the minute of the hour (0-59)
c  values(7)   the second of the minute (0-59)
c  values(8)   the number of milliseconds of the second (0-999)
c
       call date_and_time ( values=values )
       year   = values(1)
       month  = values(2)
       day    = values(3)
       hour   = values(5)
       minute = values(6)
       second = values(7)
c
c      Set "chtime" and "chdate"
c
       ncht   = len ( chtime )
       chtime = ' '
       write ( chstr, 2010 ) hour, minute, second
       nch           = min ( 8, ncht )
       chtime(1:nch) = chstr(1:nch)
c
       nchd   = len ( chdate )
       chdate = ' '
       write ( chstr, 2020 ) day, month, year
       nch           = min ( 10, nchd )
       chdate(1:nch) = chstr(1:nch)
c
c-----------------------------------------------------------------------
c
2010   format(i2.2,':',i2.2,':',i2.2)
2020   format(i2,'/',i2.2,'/',i4.4)
c
       return
       end
c
c=======================================================================
c
c+++++++++++++++++++++++++++ PSPLOT ROUTINES +++++++++++++++++++++++++++
c
c  The following modules represent a subset of the full PSPLOT library 
c  (/Users/dclarke/grfx/psplot) that support simple line graphs.  No 2-D 
c  colour or gray-scale contouring routines are included.
c
c  All PSPLOT modules were kindly provided by the late Kevin Kohler,
c  formerly of Nova Southeastern University, Fort Lauderdale, FL.
c
c  Modifications to individual routines made by Alexander Men'shchikov 
c  and David Clarke.
c
c=======================================================================
       subroutine asterisk ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, June 2010 using dk times as a template:  This routine 
c  draws an asterisk symbol on a 1 - D plot.
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius
c
       real          xa      (   2), ya      (   2)
c
       external      drwcrv  
c----------------------------------------------------------------------- 
       xa(1) = xc
       ya(1) = yc - radius
       xa(2) = xc
       ya(2) = yc + radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       xa(1) = xc - 0.87 * radius
       ya(1) = yc - 0.50 * radius
       xa(2) = xc + 0.87 * radius
       ya(2) = yc + 0.50 * radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       ya(1) = yc + 0.50 * radius
       ya(2) = yc - 0.50 * radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine blkstp ( ch, ndim, a, leng )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine strips out blanks
c
c  idim        leading dimension of arr in calling program
c  ch          input character*1 array
c  ndim        dimension of "ch" and "a"
c  leng        number of non-blank characters
c----------------------------------------------------------------------- 
       implicit      none
c
       integer       ndim    , leng    , i       , l
c
       character*1   ch      (ndim), a       (ndim)
c----------------------------------------------------------------------- 
c
c      Strip out blanks only (leave in esc, etc.)
c
       i    = 1
       leng = 0
10     continue
       if ( ichar(ch(i)) .ne. 32 )then
         leng    = leng + 1
         a(leng) = ch(i)
       endif
c
       if ( i .eq. ndim ) then
c
c      Blankfill remainder of output array
c
         do 20 l=leng+1,ndim
           a(l) = ' '
20       continue
         go to 30
       endif
c
       i = i + 1
       go to 10
30     continue
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine circle ( xc, yc, radius, fill )
c----------------------------------------------------------------------- 
c  written by KEK: writes a (filled) circle as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of circle
c  radius      radius of circle (inches)
c  fill        logical toggle indicating if circle is to be filled
c----------------------------------------------------------------------- 
       implicit      none
c
       logical        fill
       character*132  cmdstr  , scrc
       real           xc      , yc      , radius  , conver  , xci     
     1              , yci     , rad
c
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       integer       lenstr
       external      lenstr  , filler
c----------------------------------------------------------------------- 
       xci  = xc     * conver
       yci  = yc     * conver
       rad  = radius * conver
       scrc = ' '
       write ( scrc, '(f8.2,'' '',f8.2,'' '',f8.2,'' C'')' )
     1   xci, yci, rad
       if ( fill ) then
         cmdstr = 'Np '//scrc(1:lenstr(scrc,132))//' fill'
       else
         cmdstr = 'Np '//scrc(1:lenstr(scrc,132))//' stroke'
       endif
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
c----------------------------------------------------------------------- 
       subroutine dshcrv ( x, y, npts, idshpn, thk )
c----------------------------------------------------------------------- 
c  Written by KEK: this routine draws a dashed curve connecting points 
c  (x(n),y(n)), n=1,npts.
c
c  x,y         coordinate arrays in inches
c  npts        dimension of x and y; number of points to connect
c              starting at (x(1),y(1))
c  idshpn      number of solid/blank patterns per inch.  0 => solid 
c              curve is drawn.
c  thk         thickness of the dashes (inches).  0.0 => a single line
c              is drawn for the dashes
c----------------------------------------------------------------------- 
       implicit      none
c
       integer       npts    , idshpn  , iset    , ipen    , n       
     1             , nn      
       real          thk     , curwid  , curwids , pi      , absx    
     1             , ang     , dist    , xe      , xnew    , ye      
     2             , ynew
       real*8        dlen    , dleft   , xsd     , ysd     , xed
     1             , yed
c
       real          x       (npts), y       (npts)
 
       common      / lcom     / curwid
c
       external      drwcrv  , plot
c
       integer       icall
       data          icall    / 1 /
       save          icall
c----------------------------------------------------------------------- 
       if (icall .eq. 0) then
         write(6,3000)
3000     format('PSPLOT: EXTERNAL: DSHCRV')
         icall = 1
       endif
c
       if (idshpn .le. 0) then
         call drwcrv ( x, y, npts, thk, .false. )
         return
       endif
c
       pi = acos (-1.0)
c
       iset = 0
       if ( (thk .ne. 0.0) .and. (thk.ne.curwid) ) then 
         iset    = 1
         curwids = curwid
         call setlw ( thk )
       endif
c
       dlen = 0.5d0 / dble(idshpn)
       ipen = 3
       call plot ( x(1), y(1), ipen )
       xsd  = dble(x(1))
       ysd  = dble(y(1))
       n    = 2
c
10     continue
       if ( n .gt. npts ) go to 30         ! we're done
       ipen = 5 - ipen                     !(3-->2, 2-->3)
c
c      Find angle to next data point.
c
       absx = abs ( x(n) - real(xsd) )
       if ( absx .lt. 1.0e-5 ) then
         ang = pi/2.0 * sign ( 1.0, y(n) - real(ysd) )
       else
         ang = atan2 ( y(n) - real(ysd), x(n) - real(xsd) )
       endif
c
c      Calculate end point of dash or space.
c
       xed = xsd + dlen * dble ( cos(ang) )
       yed = ysd + dlen * dble ( sin(ang) )
       xe  = real ( xed )
       ye  = real ( yed )
c
c      See if we can fit this in before next data point.
c
       dist = sqrt ( (x(n) - real(xsd) )**2 + ( y(n) - real(ysd) )**2 )
       if ( dist .le. dlen ) then
c
c      Can't fit
c
         dleft = dlen
         do 20 nn=n,npts
c
c      Find angle to next data point.
c
           absx = abs ( x(nn) - real(xsd) )
           if ( absx .lt. 1.0e-5 ) then
             ang = pi/2.0 * sign ( 1.0, y(nn) - real(ysd) )
           else
             ang = atan2 ( ( y(nn) - real(ysd) )
     1                   , ( x(nn) - real(xsd) ) )
           endif
           dist = sqrt ( ( x(nn) - real(xsd) )**2 
     1                 + ( y(nn) - real(ysd) )**2 )
           if ( dist .le. dleft ) then
c
c      Draw line to next data point and calculate rest.
c
             call plot ( x(nn), y(nn), ipen )
             xsd   = dble ( x(nn) )
             ysd   = dble ( y(nn) )
             dleft = dleft - dble ( dist )     !Remaining portion
             if ( dleft .eq. 0.0d0 ) then 
c
c      Exact fit, so just do next point
c
               n = nn + 1
               go to 10
             endif
           else
c
c      Draw remaining portion and move toward.
c
             xnew = real ( xsd + dleft * dcos ( dble(ang) ) )
             ynew = real ( ysd + dleft * dsin ( dble(ang) ) )
             call plot ( xnew, ynew, ipen )
             xsd  = dble ( xnew )
             ysd  = dble ( ynew )
             n    = nn
             go to 10
           endif
20       continue
       else
c
c      Can fit, so just draw normally.  Check to see if we should just 
c  move to data point itself since roundoff could cause weird angles.
c
         dist = sqrt ( ( x(n) - real(xed) )**2
     1               + ( y(n) - real(yed) )**2 )
         if ( dist .le. 1.0e-5 ) then
           xed = dble(x(n))
           yed = dble(y(n))
           xe  = real(xed)
           ye  = real(yed)
           n   = n + 1
         endif
         call plot ( xe, ye, ipen )
         xsd = dble ( xed )
         ysd = dble ( yed )
         go to 10
       endif
c
30     continue
c
c      Reset linewidth to original
c
       if ( iset .eq. 1 ) call setlw ( curwids )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine diamond ( xc, yc, height )
c----------------------------------------------------------------------- 
c  written by DAC, June 2010 using dk times as a template:  This routine 
c  draws a diamond symbol on a 1-D plot.
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , height  , width
c
       real          xa      (   4), ya      (   4)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       width = 0.8 * height
       xa(1) = xc - width
       ya(1) = yc
       xa(2) = xc
       ya(2) = yc - height
       xa(3) = xc + width
       ya(3) = yc
       xa(4) = xc
       ya(4) = yc + height
       call drwcrv ( xa, ya, 4, 0.0, .true. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine drwcrv ( xarr, yarr, n, thkk, closer )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine draws a curved line on the current plot.
c
c  xarr, yarr  coordinates through which curve is drawn
c  n           dimension of xarr, yarr
c  thkk        desired thickness of line; 0.0 => use current thickness
c  closer      .true.  => dashed line
c              .false. => solid line
c----------------------------------------------------------------------- 
       implicit      none
c
       logical       closer
       character*132 cmdstr
       integer       n       , nn
       real          thkk    , curwids , curwid
c
       real          xarr    (   n), yarr    (   n)
c
c      "curwid" is set in routine setlw
c
       common      / lcom     / curwid
       common      / plt1     / cmdstr
c
       external      filler  , setlw   , movet   , linet   , stroke
c----------------------------------------------------------------------- 
       cmdstr = 'Np'
       call filler
c
       if ( thkk .ne. 0.0 ) then
         curwids = curwid
         call setlw ( thkk )
       endif
       call movet ( xarr(1), yarr(1) )
       do 10 nn=2,n
         call linet ( xarr(nn), yarr(nn) )
         if ( mod ( nn, 100 ) .eq. 0 ) then
c
c      stroke now to avoid possible limit
c
           call stroke
           call movet ( xarr(nn), yarr(nn) )
         endif
10     continue
c
       if ( closer ) then
         cmdstr = 'Cs'
       else
         cmdstr = 'S'
       endif
       call filler
       if ( thkk .ne. 0.0 ) call setlw ( curwids )   !Reset linewidth
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine factor ( facc )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ... 
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          facc    , fac     , recipx  , recipy
c
       common      / plt1     / cmdstr
       common      / plt2     / fac
c
       external      filler
c----------------------------------------------------------------------- 
c
c      Unscale previous scaling
c
       recipx = 1.0 / fac
       recipy = 1.0 / fac
       write ( cmdstr, '(2f7.3,'' scale'')' ) recipx, recipy
       call filler
       fac = facc
       write ( cmdstr, '(2F7.3,'' scale'')' ) fac, fac
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine filler
c----------------------------------------------------------------------- 
c  Written by KEK: this routine...
c----------------------------------------------------------------------- 
       implicit      none
c
       logical       ispace
       character*132 cmdstr
       integer       iunit   , ibslash , lc      , lcc     , itot
     1             , icclst  , l       , icc     , ii
c
       character*1   cmdc    ( 132)
c
       common      / plt1     / cmdstr
       common      / pspunit  / iunit
c
       integer       lenstr
       external      lenstr
c----------------------------------------------------------------------- 
       ibslash = 92
       lc      = 0
       lcc     = lenstr ( cmdstr, 132 )
       ispace  = .false.
c
c      "itot" is running total of left/right parentheses in text string
c      If itot=0 then we are not in text mode, i.e. left = right
c
       itot   = 0
       icclst =-999
       do 10 l=1,lcc
         icc = ichar ( cmdstr(l:l) )
c
c      Don't place 2 or more spaces together not in text mode.
c
         if ( (icc .eq. 32) .and. (ispace) .and. (itot .eq. 0) )
     1     go to 10
         if ( (icc .ge. 32) .and. (icc .le. 127) ) then
           lc       = lc + 1
           cmdc(lc) = cmdstr(l:l)
         endif
         if (icc .eq. 32) then
           ispace = .true.
         else
           ispace = .false.
         endif
         if (icclst .ne. ibslash) then
           if (icc .eq. 40) itot = itot + 1
           if (icc .eq. 41) itot = itot - 1
         endif
         icclst = icc
10     continue
c
c      Write cmdstr to output file
c
       write ( iunit, '(132a1)' ) ( cmdc(ii), ii=1,lc )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine keksym ( xp, yp, size, titlein, ang, nchar1, mjus )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ...
c
c  Modified by ABM, 10/03: converted Hollerith strings to character 
c  strings; generalized to accept special characters (fonts 29 and 35)
c----------------------------------------------------------------------- 
       implicit      none
c
       character*1   bslash
       character*132 cmdstr  , curfnt  , scrc    , titleb  , titlec
       character*(*) titlein
       integer       nchar1  , mjus    , nchar   , mchar   , iht
     1             , ifntsz  , nfont   , ititle  , m       , njus
     2             , numc    
       real          xp      , yp      , size    , ang     , conver  
     1             , pi      , high    , rsize   , szrat   , wide    
     2             , xarg    , yarg    , xpos    , ypos    , arg     
     3             , argdeg
c
       character*4  ctitle(20)
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
       common      / kkplot   / szrat
c
       integer       lenstr
       external      filler  , blkstp  , lenstr
c----------------------------------------------------------------------- 
c
c      Stroke previous paths before this write
c
       cmdstr = 'S'
       call filler
       bslash = char(92)
       pi     = acos (-1.0)
c
       if ( nchar1 .eq. -999 ) then        ! octal code
         ctitle(1) = titlein
         read ( ctitle(1), '(I4)' ) ititle
         nchar     = 1
       else
         nchar     = nchar1
         titlec    = titlein
         if ( iabs(nchar) .lt. 132 ) titlec(nchar+1:132) = ' '
         read ( titlec, '(20a4)' ) ctitle
       endif
c
c      Choose proper font height, using current font.
c
       mchar = iabs ( nchar )
c
c      Set character size
c
       iht = int ( size * conver / 0.6 )     !0.6 factor is empirical
       if ( iht .ne. ifntsz ) then
         cmdstr = ' '
         write ( cmdstr, '(I3,'' Setf'')' ) iht
         call filler
         ifntsz = iht
       endif
c
       if ( nchar1 .eq. -999 ) then        !Octal code
         write ( titlec, '(A1,I10)' ) bslash, ititle
         call blkstp ( titlec, 132, titlec, numc )
       else
         write ( titlec, '(20a4)' ) ctitle
c
c      Check if "titlec" contains (, ), or "backslash." These characters 
c  must be treated specially by preceding them with a "backslash". Do 
c  this to ( and ) even though they might be balanced, i.e. () within a 
c  string, which can be treated normally.
c
         titleb = titlec
         numc   = 0
         do 10 m=1,mchar
           if ( (titleb(m:m) .eq. '(') .or. (titleb(m:m) .eq. ')') .or.
     1          (titleb(m:m) .eq. bslash) ) then
             numc              = numc + 1
             titlec(numc:numc) = bslash
           endif
           numc              = numc + 1
           titlec(numc:numc) = titleb(m:m)
10       continue
       endif
 
       mchar = numc
       xpos  = xp
       ypos  = yp
       if ( nchar .lt. 0 ) then
         njus = 0
       else
         njus = mjus
       endif
       rsize = size
c
       if ( xpos .eq. 999.0 ) then
         cmdstr = '/Xpos Xpres def'
         njus   = 0
       else
         cmdstr = ' '
         write ( cmdstr, '(f8.2,'' Xpd'')' ) xp * conver
       endif
       call filler
c
       if ( ypos .eq. 999.0 ) then
         cmdstr = '/Ypos Ypres def'
         njus   = 0
       else
         cmdstr = ' '
         write ( cmdstr, '(f8.2,'' Ypd'')' ) yp * conver
       endif
       call filler
c
       if ( (njus .ne. 0) .and. (njus .ne. 1) .and. (njus .ne. 2) ) then
         write ( *,2010 ) njus
2010     format (1x,'Incorrect justification code ',i5,'found in '
     1          ,'KEKSYMC, zero used')
         njus = 0
       endif
c
c      Strlen has already been "factored" by the choice of font height.
c  Since it will eventually be factored again, we must divide by factor 
c  now.
c
       cmdstr = '('//titlec(1:mchar)//') Le'
       arg  = ang * pi / 180.0
       xarg = 0.5 * cos(arg) * float (njus)
       yarg = 0.5 * sin(arg) * float (njus)
c
       if ( xarg .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(f7.3,'' Xpjd'')') xarg
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
c
       if ( yarg .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(f7.3,'' Ypjd'')' ) yarg
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
c
       if ( nchar .eq. -1 ) then    !centered symbol
         high = rsize
         wide = high * szrat
         xarg = 0.5 * ( high * sin(arg) - wide * cos(arg) )
         yarg =-0.5 * ( high * cos(arg) + wide * sin(arg) )
         xpos = xpos + xarg
         ypos = ypos + yarg
c
         if ( xarg .ne. 0.0 ) then
           scrc   = ' '
           write ( scrc, '(''/Xpos Xpos'',f8.2,'' add def'')' )
     1       xarg * conver
           cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1              //scrc(1:lenstr(scrc,132))
         endif
c
         if ( yarg .ne. 0.0 ) then
           scrc = ' '
           write ( scrc, '(''/Ypos Ypos'',f8.2,'' add def'')' )
     1       yarg * conver
           cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1              //scrc(1:lenstr(scrc,132))
         endif
       endif
c
       call filler
c
c      Move pen to proper coordinates
c
       scrc   = 'xyd mv'
       cmdstr = scrc(1:lenstr(scrc,132))
c
c      Set angle
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write(scrc,'(F7.1,'' rotate'')') ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
c
       scrc   = '('//titlec(1:mchar)//') show'
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1          //scrc(1:lenstr(scrc,132))
c
c      Reset angle
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) -ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
       call filler
c
c      Start next char at 0.5 char width away
c
       argdeg = arg * 180.0 / pi
       cmdstr = ' '
       write ( cmdstr, '(f6.1,'' Xyps'')' ) argdeg
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       function lenstr ( string, ls )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine finds actual length of string by 
c  eliminating trailing blanks
c
c  string      character string
c  ls          declared length of string in calling program
c----------------------------------------------------------------------- 
       implicit      none
c
       character*(*) string
       integer       lenstr  , ls      , i       , is
c----------------------------------------------------------------------- 
       do 10 i=ls,1,-1
         is = i
         if ( string(i:i) .ne. char(32) ) go to 20
10     continue
       is = 0
20     continue
       lenstr = is
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine linet ( xcall, ycall )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine is same as a call to plot with ip=2, but 
c  does not perform the "stroke".
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          xcall   , ycall   , conver
c
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       external      filler
c----------------------------------------------------------------------- 
       write ( cmdstr, '(2f8.2,'' L'')' ) xcall * conver, ycall * conver
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine movet ( xcall, ycall )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine is same as call to plot with ip=3, but 
c  does not perform the "stroke".
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          xcall   , ycall   , conver
c
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       external      filler
c----------------------------------------------------------------------- 
       write ( cmdstr, '(2f8.2,'' M'')' ) xcall * conver, ycall * conver
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine newdev ( iusnb )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine changes the output device/filename to 
c  the user specifications. It must be called before psinit.
c
c  iusnb       new file name ascii string
c----------------------------------------------------------------------- 
       implicit      none
c
       character*80  fileout
       character*(*) iusnb
       integer       inew    , lus
c
       common      / io       / fileout , inew
c
       integer       lenstr
       external      lenstr
c----------------------------------------------------------------------- 
       lus     = lenstr ( iusnb, len(iusnb) )
       fileout = ' '
       fileout = iusnb(1:lus)
c
c      Set inew to 999 so that init knows to use this name
c
       inew = 999
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine ocircle ( xc, yc, rad )
c----------------------------------------------------------------------- 
c  Written by DAC: This routine draws an open circle.
c
c  xc, yc      coordinates of circle centre
c  rad         radius of circle (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , rad
c
       external      circle
c----------------------------------------------------------------------- 
       call circle ( xc, yc, 0.95*rad, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine plot ( xcall, ycall, ip )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ...
c
c  idim        leading dimension of arr in calling program
c  xcall,ycall coordinates of...
c  ip          ...
c----------------------------------------------------------------------- 
       implicit      none
c
       character*80  scr
       character*132 cmdstr
       integer       ip      , iunit   , npage   , npageo  , ipp
     1             , nch
       real          xcall   , ycall   , conver
 
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
       common      / pspunit  / iunit
       common      / pagcom   / npage, npageo
c
       external      filler  , blkstp
c----------------------------------------------------------------------- 
       ipp = iabs(ip)
       if ( ip .eq. 999 ) then   !    Terminate plot session.
         cmdstr = 'stroke showpage'
         call filler
         cmdstr = '%%Trailer'
         call filler
         write ( scr, '(i6)' ) npage
         call blkstp ( scr, 80, scr, nch )
         cmdstr = '%%Pages: '//scr(1:nch)
         call filler
         cmdstr = '%%EOF'
         call filler
         npage  = npageo
         close(iunit)
         return
       endif
c
c      Moving pen
c
       if ( ipp .eq. 3 ) then 
c
c      Stroke to paint previous path, then move to...
c
         write ( cmdstr, '(2f8.2,'' SM'')' )
     1     xcall * conver, ycall * conver
       else
         write ( cmdstr, '(2f8.2,'' LM'')' )
     1     xcall * conver, ycall * conver
       endif
       call filler
c
c      Reset origin if ip < 0
c
       if (ip .lt. 0 ) then
         write ( cmdstr, '(2f8.2,'' translate'')' )
     1     xcall * conver, ycall * conver
         call filler
       endif
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine plus ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 8/08 using dk square as a template: writes a plus 
c  sign as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of plus sign
c  radius      radius of plus sign (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius
c
       real          xa      (2), ya      (2)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       xa(1) = xc - radius
       ya(1) = yc
       xa(2) = xc + radius
       ya(2) = yc
       call drwcrv ( xa, ya, 2, 0.0, .false. )
       xa(1) = xc
       ya(1) = yc - radius
       xa(2) = xc
       ya(2) = yc + radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine psinit ( portrait )
c----------------------------------------------------------------------- 
c  Written by KEK: initializes plot for hp plotter
c
c  portrait    .true. => portrait orientation (.false. => landscape)
c----------------------------------------------------------------------- 
       implicit      none
c
       logical       portrait, prtrt   , iopen
       character*132 cmdstr  , curfnt
       character*80  fileout
       integer       ioffp   , inew    , ientry  , ifntsz  
     1             , nfont   , iunit   , npage   , npageo
       real          spval   , conver  , fac     , szrat   , xsh     
     1             , ysh
c
       common      / conre1   / ioffp   , spval
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
       common      / plt2     / fac
       common      / io       / fileout , inew
       common      / kkplot   / szrat
       common      / chpcom   / ientry  , prtrt
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / pspunit  / iunit
       common      / pagcom   / npage   , npageo
c
       data          npageo   / 0 /
c
       integer       lenstr
       external      lenstr  , filler  , setfnt  , factor  , setlw
     1             , setgry  , setcolr , plot
c----------------------------------------------------------------------- 
       ioffp = 0
       spval = 0.0
c
c      Set conversion factor: conver=72.0 for inches; 72.0/2.54 for cm,
c  etc.
c
       conver = 72.0 / 2.54
       if (npageo .eq. 0) then
         npageo = 1
       else
         npageo = npage
       endif
       npage = 1
       prtrt = portrait
c
c      Use default name unless "newdev" has already been called (inew = 
c  999).
c
       if ( inew .eq. 0 ) then
         fileout = 'psplot.ps'
         inew    = 1
       endif
       if ( inew .eq. 999 )then
         inew    = 1
       endif
c
c      Open output file.
c
       iunit = 60
10     continue
       inquire ( iunit, opened=iopen )    ! get a non-existent unit
       if ( iopen ) then
         iunit = iunit + 1
         go to 10
       endif
c
       open ( iunit, file=fileout, status='unknown', form='formatted')
c
       cmdstr = '%!PS-Adobe-3.0'
       call filler
c
c      Next PS command introduced by Alexander Men'shchikov, Dec 2003.
c  It helps to avoid large white space at the top of a page when
c  displaying it with GhostView. This gives instant view of the entire 
c  plot, without any need to scroll it in the window. There may be a 
c  problem printing such a page, but one can switch Media in GhostView 
c  back to Letter or just delete/comment the line in the PS file.
c
c      cmdstr = '%%DocumentMedia: JustForScreen 585 652 80 white ( )'
c      call filler
c
       cmdstr =  '%%Title: '//fileout(1:lenstr(fileout,80))
       call filler
c
c       cmdstr =  '%%CreationDate: '//cdate//' '//ctime
c       call filler
c
       cmdstr =  '%%Creator: PSPLOT PostScript Plotting Package'
       call filler
c
       cmdstr =  '%%Pages: (atend)'
       call filler
c
       cmdstr = '%%EndComments'
       call filler
c
       cmdstr = '%Library Creator: Kevin E. Kohler <kevin@nova.edu>'
       call filler
c
       cmdstr = '%%BeginProlog'
       call filler
c
       cmdstr = '/inch {72 mul} bind def'
       call filler
c
       cmdstr = '/Ah {moveto lineto lineto stroke} def'
       call filler
       cmdstr = '/Ar {moveto 2 copy lineto 4  - 2 roll'
       call filler
       cmdstr = '     moveto lineto lineto stroke } def'
       call filler
c
       cmdstr = '/arcit {S /A2 exch def /A1 exch def /Rad exch def'
       call filler
       cmdstr = '        /Yc exch def /Xc exch def'
       call filler
       cmdstr = '        Xc Rad A1 cos mul add Yc Rad A1 sin mul add'
       call filler
       cmdstr = '        moveto newpath'
       call filler
       cmdstr = '        Xc Yc Rad A1 A2 arc stroke} def'
       call filler
c
       cmdstr = '/C {/Rad exch def /Yc exch def /Xc exch def'
       call filler
c
c      Closepath needed to avoid notch with fat line width
c
       cmdstr = '         Xc Yc Rad 0 360 arc closepath'
       call filler
       cmdstr = '        } def'
       call filler
c
       cmdstr = '/c0sf {closepath 0 setgray fill} def'
       call filler
c
       cmdstr = ' '
       write (cmdstr,'(''/cc {'',f7.4,'' mul def} def'')') conver
       call filler
c
       cmdstr = '/cd {cc /dy} def'
       call filler
c
       cmdstr = '/cf {closepath fill} def'
       call filler
c
       cmdstr = '/Cs {closepath stroke} def'
       call filler
c
       cmdstr = '/Cln {newpath 3 1 roll'
       call filler
       cmdstr = '      moveto {lineto} repeat clip newpath'
       call filler
       cmdstr = '     } def'
       call filler
c
       cmdstr = '/Cs {closepath stroke} def'
       call filler
c
       cmdstr = '/dd {/Dx dx def /Dy dy def} def'
       call filler
c
       cmdstr = '/Fb {newpath moveto '
       call filler
       cmdstr = ' Dx 0 rlineto 0 Dy rlineto Dx neg 0 rlineto closepath'
       call filler
       cmdstr = ' fill } def'
       call filler
c
       cmdstr = '/Fbn { newpath 3 1 roll moveto {lineto} repeat'
       call filler
       cmdstr = '       closepath fill } def'
       call filler
c
       cmdstr = '/Fbnc { newpath 3 1 roll moveto'
       call filler
       cmdstr = '       {lineto} repeat closepath fill } def'
       call filler
c
       cmdstr = '/L /lineto load def'
       call filler
c
       cmdstr = '/Le {/Strlen exch stringwidth pop def} def'
       call filler
c
c      Define stringlength slightly increased for integrand placement
c
       cmdstr = '/Lei {/Strlen exch stringwidth pop 1.5 mul def} def'
       call filler
c
c      Define stringlength slightly increased for summation placement
c
       cmdstr = '/Les {/Strlen exch stringwidth pop 1.1 mul def} def'
       call filler
c
       cmdstr = '/Lenssd '//
     1        '{/Strlenss exch stringwidth pop 3 mul 4 div def} def'
       call filler
c
       cmdstr = '/LM {2 copy lineto stroke moveto} def'
       call filler
c
       cmdstr = '/lm {Xp Yp lineto stroke mv} def'
       call filler
c
       cmdstr = '/M /moveto load def'
       call filler
c
       cmdstr = '/mv {Xp Yp moveto} def'
       call filler
c
       cmdstr = '/Np {newpath} def'
       call filler
c
       cmdstr = '/S /stroke load def'
       call filler
c
       cmdstr = '/Sc {setrgbcolor} def'
       call filler
c
       cmdstr = '/Sg {setgray} def'
       call filler
c
       cmdstr = '/Setf {Curfnt exch scalefont setfont} def'
       call filler
c
       cmdstr = '/SM {stroke moveto} def'
       call filler
c
       cmdstr = '/sm {stroke mv} def'
       call filler
c
       write ( cmdstr, '(''/Slw {'',f7.4,'' mul setlinewidth} def'')' )
     1   conver
       call filler
c
c      Minimum line width 300 dpi
c
       cmdstr = '/Slw0 {.24 setlinewidth} bind def'
       call filler
c
       cmdstr = '/Xpd {/Xpos exch def} def'
       call filler
c
       cmdstr = '/Xpjd '//
     1        ' {/Xpos exch Xpos exch Strlen mul sub def} def'
       call filler
c
       cmdstr = '/xyd {/Xp Xpos def /Yp Ypos def} def'
       call filler
c
       cmdstr = '/Xyp0d {/Xpos0 Xpres def /Ypos0 Ypres def} def'
       call filler
c
       cmdstr = '/Xyps {dup '//
     1        '/Xpres exch cos Strlen mul Xpos add def'
       call filler
       cmdstr = '              '//
     1        '/Ypres exch sin Strlen mul Ypos add def} def'
       call filler
c
       cmdstr = '/Xyps0 {dup '//
     1        '/Xpres exch cos Strlen mul Xpos0 add def'
       call filler
       cmdstr = '               '//
     1        '/Ypres exch sin Strlen mul Ypos0 add def} def'
       call filler
c
       cmdstr = '/Ypd {/Ypos exch def} def'
       call filler
c
       cmdstr = '/Ypjd '//
     1        ' {/Ypos exch Ypos exch Strlen mul sub def} def'
       call filler
c
       cmdstr = '/space ( ) def'
       call filler
c
       cmdstr = '%%EndProlog'
       call filler
c
       cmdstr = '%%Page: 1 1'
       call filler
c
c      "szrat" is the ratio of width to height of characters, determined
c  empirically to be: szrat = 0.6.  Set initial font to helvetica, 12 
c  point.
c
       szrat  = 0.6
       ifntsz = 12
       call setfnt ( 20 )
c
c      Set factor to 1 for initialization, reset later if CHOPIT called
c
       fac = 1.0
       call factor ( fac )
c
c      Set initial lineweight to 0.
c
       call setlw ( 0.0 )
c
c      Set initial grayscale to 0.
c
       call setgry ( 0.0 )
c
c      Set initial rgb colors to black(0).
c
       call setcolr ( 0.0, 0.0, 0.0 )
c
       if (.not. portrait) then
         cmdstr = '90 rotate 0 -8.5 inch translate '
         call filler
       endif
c
       call plot ( 18.0 / conver, 18.0 / conver, -3 )
c
       if ( portrait ) then
         xsh = 18.0 / conver
         ysh = 0.0
       else
         xsh = 0.0
         ysh = 18.0 / conver
       endif
       call plot ( xsh, ysh, -3 )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setcolr ( red, green, blue )
c----------------------------------------------------------------------- 
c  Written by KEK: this routines sets the current colour
c
c  red, green, blue  rgb saturation ratios between 0 and 1
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          red     , green   , blue    , cred    , cgreen  
     1             , cblue   , cgry    , r       , g       , b
c
       common      / plt1     / cmdstr
       common      / colrcom  / cred    , cgreen  , cblue   , cgry
c
       external      filler
c----------------------------------------------------------------------- 
       r = amin1 ( 1.0, amax1 ( 0.0, red   ) )
       g = amin1 ( 1.0, amax1 ( 0.0, green ) )
       b = amin1 ( 1.0, amax1 ( 0.0, blue  ) )
c
       cmdstr = ' '
       write ( cmdstr, '(3F7.3,'' Sc'')' ) r, g, b
       call filler
       cred   = r
       cgreen = g
       cblue  = b
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setfnt ( numfnt )
c----------------------------------------------------------------------- 
c  Written by KEK: This routines changes the typeface of the current 
c  font.
c
c  Modified by DAC (8/08): fonts described and rationalised
c
c  numfnt         Font            numfnt         Font
c     1    AvantGarde-Book           2    AvantGarde-BookOblique
c     3    AvantGarde-Demi           4    AvantGarde-DemiOblique
c     5    Bookman-Demi              6    Bookman-DemiItalic
c     7    Bookman-Light             8    Bookman-LightItalic
c     9    Courier-Bold             10    Courier-BoldOblique
c    11    Courier-Oblique          12    Courier
c    13    Helvetica-Bold           14    Helvetica-BoldOblique
c    15    Helvetica-Narrow-Bold    16    Helvetica-Narrow-BoldOblique
c    17    Helvetica-Narrow-Oblique 18    Helvetica-Narrow
c    19    Helvetica-Oblique        20    Helvetica
c    21    NewCenturySchlbk-Bold    22    NewCenturySchlbk-BoldItalic
c    23    NewCenturySchlbk-Italic  24    NewCenturySchlbk-Roman
c    25    Palatino-Bold            26    Palatino-BoldItalic
c    27    Palatino-Italic          28    Palatino-Roman
c    29    Symbol                   30    Times-Bold
c    31    Times-BoldItalic         32    Times-Italic
c    33    Times-Roman              34    ZapfChancery-MediumItalic
c    35    ZapfDingbats
c
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr  , scrc    , curfnt
       integer       numfnt  , ifntsz  , nfont
c
       character*40  fntnam  (  35)
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
c
       data          fntnam
     1             / 'AvantGarde-Book'
     2             , 'AvantGarde-BookOblique'
     3             , 'AvantGarde-Demi'
     4             , 'AvantGarde-DemiOblique'
     5             , 'Bookman-Demi'
     6             , 'Bookman-DemiItalic'
     7             , 'Bookman-Light'
     8             , 'Bookman-LightItalic'
     9             , 'Courier-Bold'
     1             , 'Courier-BoldOblique'
     2             , 'Courier-Oblique'
     3             , 'Courier'
     4             , 'Helvetica-Bold'
     5             , 'Helvetica-BoldOblique'
     6             , 'Helvetica-Narrow-Bold'
     7             , 'Helvetica-Narrow-BoldOblique'
     8             , 'Helvetica-Narrow-Oblique'
     9             , 'Helvetica-Narrow'
     1             , 'Helvetica-Oblique'
     2             , 'Helvetica'
     3             , 'NewCenturySchlbk-Bold'
     4             , 'NewCenturySchlbk-BoldItalic'
     5             , 'NewCenturySchlbk-Italic'
     6             , 'NewCenturySchlbk-Roman'
     7             , 'Palatino-Bold'
     8             , 'Palatino-BoldItalic'
     9             , 'Palatino-Italic'
     1             , 'Palatino-Roman'
     2             , 'Symbol'
     3             , 'Times-Bold'
     4             , 'Times-BoldItalic'
     5             , 'Times-Italic'
     6             , 'Times-Roman'
     7             , 'ZapfChancery-MediumItalic'
     8             , 'ZapfDingbats' /
c
       integer        lenstr
       external       filler  , lenstr
c----------------------------------------------------------------------- 
       nfont = numfnt
       if ( (numfnt .lt. 1) .or. (numfnt .gt. 35) ) then
         write (*,'(a)') 'Invalid font number encountered in **setfnt**'
         write (*,'(a)') 'Using Times-Roman default'
         nfont = 30
       endif
       scrc   = fntnam ( nfont )
       cmdstr = '/Curfnt /'//scrc(1:lenstr(scrc,132))//' findfont def'
       call filler
       write ( cmdstr, '(i3,'' Setf'')' ) ifntsz
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setgry ( gry )
c----------------------------------------------------------------------- 
c  Written by KEK: This routines sets the current gray level
c
c  gry         real between 0.0 and 1.0
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          gry     , cred    , cgreen  , cblue   , cgry
     1             , g
c
       common      / plt1     / cmdstr
       common      / colrcom  / cred    , cgreen  , cblue   , cgry
c
       external      filler
c----------------------------------------------------------------------- 
       g      = amax1 ( 0.0, amin1 ( 1.0, gry) )
       cmdstr = ' '
       write ( cmdstr, '(F7.3,'' Sg'')' ) g
       call filler
       cgry   = g
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setlw ( rlwi )
c----------------------------------------------------------------------- 
c  Written by KEK: this routines sets the current linewidth.
c
c  rlwi        desired linewidth in inches
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          rlwi    , curwid
c
       common      / plt1     / cmdstr
       common      / lcom     / curwid
c
       external      filler
c----------------------------------------------------------------------- 
       if ( abs(rlwi) .lt. 1.0e-5 ) then
         cmdstr = 'Slw0'
       else
         cmdstr = ' '
         write ( cmdstr, '(F7.3,'' Slw'')' ) rlwi
       endif
       call filler
       curwid = rlwi
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine sldlin ( x1, y1, x2, y2, thk )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine draws a solid line between specified 
c  points.
c
c  x1, y1      beginning coordinate of line
c  x2, y2      end coordinate of line
c  thk         thickness of the line (inches).  0.0 => line is drawn
c              with current line weight setting.  The width of the line 
c              is centred on the line joining the two points.
c----------------------------------------------------------------------- 
       implicit      none
c
       integer       iset
       real          x1      , y1      , x2      , y2      , thk
     1             , curwid  , curwids
c
       common      / lcom     / curwid
c
       external      setlw   , plot
c----------------------------------------------------------------------- 
       iset = 0
       if ( (thk .gt. 0.0) .and. (thk .ne. curwid) ) then
         iset    = 1
         curwids = curwid
         call setlw ( thk )
       endif
       call plot ( x1, y1, 3 )
       call plot ( x2, y2, 2 )
c
c      Reset linewidth to original value.
c
       if (iset .eq. 1) call setlw ( curwids )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine square ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by KEK: writes a square as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of square
c  radius      radius of square (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius  , rad
c
       real          xa      (   4), ya      (   4)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       rad   = 0.85 * radius
       xa(1) = xc - rad
       ya(1) = yc - rad
       xa(2) = xc + rad
       ya(2) = ya(1)
       xa(3) = xa(2)
       ya(3) = yc + rad
       xa(4) = xa(1)
       ya(4) = ya(3)
       call drwcrv ( xa, ya, 4, 0.0, .true. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine star ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 6/10 using dk square as a template: writes a star 
c  as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of star
c  radius      radius of star (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius
c
       real          xa      (  10), ya      (  10)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       xa( 1) = xc
       ya( 1) = yc + radius
       xa( 2) = xc - 0.225 * radius
       ya( 2) = yc + 0.309 * radius
       xa( 3) = xc - 0.951 * radius
       ya( 3) = ya(2)
       xa( 4) = xc - 0.363 * radius
       ya( 4) = yc - 0.118 * radius
       xa( 5) = xc - 0.588 * radius
       ya( 5) = yc - 0.809 * radius
       xa( 6) = xc
       ya( 6) = yc - 0.382 * radius
       xa( 7) = xc + 0.588 * radius
       ya( 7) = ya(5)
       xa( 8) = xc + 0.363 * radius
       ya( 8) = ya(4)
       xa( 9) = xc + 0.951 * radius
       ya( 9) = ya(3)
       xa(10) = xc + 0.225 * radius
       ya(10) = ya(2)
       call drwcrv ( xa, ya, 10, 0.0, .true. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine stroke
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ... 
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
c
       common      / plt1     / cmdstr
c
       external      filler
c----------------------------------------------------------------------- 
       cmdstr = 'stroke'
       call filler
c
       return
       end
c======================================================================= 
c=======================================================================
       subroutine subber ( titlein, nchar1, size, ang, fact )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine produces subscripted text.
c
c  titlein     character(s) to be subscripted
c  nchar1      number of characters to be subscripted
c  size        height of subscripted variable
c  ang         angle of text
c  fact        parameter for second level of subscripting
c----------------------------------------------------------------------- 
c  Modified by ABM, 10/2003: replaced Hollerith for character strings; 
c  generalized to accept special characters (of fonts 29 and 35).
c
c  Modified by DAC, 7/2010: introduced "fact" for second level of super- 
c  subscripting (fact = 1.0  = > no change from before)
c----------------------------------------------------------------------- 
       implicit      none
c
       character*(*) titlein
       character*132 cmdstr  , curfnt  , scrc
       character*80  titleb  , titlec
       character*1   bslash
       integer       nchar1  , ifntsz  , nfont   , nchar   , mchar   
     1             , numc    , m       , ireset  , iht     , ititle
       real          size    , ang     , fact    , conver  , pi      
     1             , angrad  , ss      , down    , xshift  , yshift  
c
       character*4   ctitle  (  20)
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       integer       lenstr
       external      filler  , blkstp  , lenstr  , setfnt
c----------------------------------------------------------------------- 
       bslash = char(92)
       pi     = acos (-1.0)
       angrad = ang * pi / 180.0
       ss     = 2.0 * size / 3.0
       down   = fact * ss / 2.0
       xshift = sin(angrad) * down
       yshift = cos(angrad) * down
c
       cmdstr = 'Xyp0d'
       call filler
c
       cmdstr = ' '
       write ( cmdstr, '(''/Xpos Xpres '',f8.2,'' add def '')' )
     1   xshift * conver
       call filler
       cmdstr = ' '
       write ( cmdstr, '(''/Ypos Ypres '',F8.2,'' sub def '')' )
     1   yshift * conver
       call filler
c
       if ( nchar1 .eq. -999 ) then
         ctitle(1) = titlein
         read ( ctitle(1), '(I4)' ) ititle
         nchar     = 1
       else
         nchar     = nchar1
         titlec    = titlein
         if ( iabs(nchar) .lt. 80 ) titlec(nchar+1:80) = ' '
         read ( titlec, '(20a4)' ) ctitle
       endif
c
c      Choose font proper height using currently set font
c
       mchar = iabs(nchar)
c
       if ( nchar1. eq. -999 ) then
         write ( titlec, '(A1,I10)' ) bslash, ititle
         call blkstp ( titlec, 80, titlec, mchar )
       else
         write ( titlec, '(20a4)' ) ctitle
c
c      Check if titlec contains (, ), or "backslash". These characters 
c  must be treated specially by preceding them with a "backslash". Do 
c  this to ( and ) even though they might be balanced, i.e. () within a 
c  string, which can be treated normally.
c
         titleb = titlec
         numc   = 0
         do 10 m=1,mchar
           if ( (titleb(m:m) .eq. '(') .or. (titleb(m:m) .eq. ')') .or.
     1          (titleb(m:m) .eq. bslash) ) then
             numc = numc + 1
             titlec(numc:numc) = bslash
           endif
           numc = numc + 1
           titlec(numc:numc) = titleb(m:m)
10       continue
         mchar = numc
       endif
c
       cmdstr = '('//titlec(1:mchar)//') Le'
c
c      Set character size
c
       iht = int ( ss * conver / 0.6 )     !0.6 factor is empirical
c
       ireset = 0
       if ( titlec(1:mchar) .ne. char(39) ) then
         if ( iht .ne. ifntsz ) then
           cmdstr = ' '
           write ( cmdstr, '(I3,'' Setf'')' ) iht
           call filler
           ifntsz = iht
         endif
       else        !Use prime, not apostrophe (do not change ifntsz)
         ireset = 1
         cmdstr = ' '
         write ( cmdstr
     1         ,'(''/Symbol findfont '',I3,'' scalefont setfont'')')iht
         call filler
       endif
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         cmdstr = '('//titlec(1:mchar)//') Le'
       else
         cmdstr = '('//bslash//'242) Le'
       endif
c
c      Move pen to proper coordinates
c
       scrc   = 'xyd mv'
       cmdstr = cmdstr (1:lenstr(cmdstr,132))//' '
     1                 //scrc(1:lenstr(scrc,132))
c
c      Set angle
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         scrc = '('//titlec(1:mchar)//') show'
       else
         scrc = '('//bslash//'242) show'
       endif
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                   //scrc(1:lenstr(scrc,132))
c
c      Reset angle.
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')') -ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
       call filler
c
c      Compute next character position.
c
       cmdstr = ' '
       write ( cmdstr, '(f6.1,'' Xyps0'')' ) ang
       call filler
c
c      Reset font if we drew a prime
c
       if ( ireset .eq. 1 ) call setfnt ( nfont )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine super ( titlein, nchar1, size, ang, fact )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine produces superscripted text.
c
c  titlein     character(s) to be superscripted
c  nchar1      number of characters to be superscripted
c  size        height of superscripted variable
c  ang         angle of text
c  fact        parameter for second level of superscripting
c----------------------------------------------------------------------- 
c  Modified by ABM, 10/2003: replaced Hollerith for character strings; 
c  generalized to accept special characters (of fonts 29 and 35).
c
c  Modified by DAC, 7/2010: introduced "fact" for second level of super- 
c  subscripting (fact = 1.0  = > no change from before)
c----------------------------------------------------------------------- 
       implicit      none
c
       character*(*) titlein
       character*132 cmdstr  , curfnt  , scrc    
       character*80  titlec
       character*1   bslash
       integer       nchar1  , ifntsz  , nfont   , nchar   , mchar   
     1             , iht     , ireset  
       real          size    , ang     , fact    , conver  , pi      
     1             , angrad  , ss      , up      , xshift  , yshift 
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       integer       lenstr
       external      filler  , lenstr  , setfnt
c----------------------------------------------------------------------- 
       bslash = char(92)
       titlec = titlein
c
c      If nchar1 is lt 0, we're calling from oversup and we should
c  lower superscript slightly
c
       nchar  = abs(nchar1)
       pi     = acos(-1.0)
       angrad = ang * pi/180.
       ss     = 2.0 * size / 3.0
       if ( nchar1 .ge. 0 ) then
         up = fact * 0.8 * ss
       else
         up = fact * 0.5 * ss
       endif
       xshift = sin(angrad) * up
       yshift = cos(angrad) * up
       cmdstr = 'Xyp0d'
       call filler
c
       cmdstr = ' '
       write ( cmdstr, '(''/Xpos Xpres '',f8.2,'' sub def '')' )
     1   xshift * conver
       call filler
       cmdstr = ' '
       write ( cmdstr, '(''/Ypos Ypres '',F8.2,'' add def '')' )
     1   yshift * conver
       call filler
c
c      Choose proper height.
c
       mchar = iabs(nchar)
c
c      Set character size.
c
       iht = int ( ss * conver / 0.6 )     !0.6 factor is empirical
c
       ireset = 0
       if ( titlec(1:mchar) .ne. char(39) ) then
         if ( iht .ne. ifntsz ) then
           cmdstr = ' '
           write ( cmdstr, '(I3,'' Setf'')' ) iht
           call filler
           ifntsz = iht
         endif
       else        !Use prime, not apostrophe (do not change ifntsz)
         ireset = 1
         cmdstr = ' '
         write ( cmdstr
     1         ,'(''/Symbol findfont '',I3,'' scalefont setfont'')') iht
         call filler
       endif
c
c      DRAW PRIME, NOT APOSTROPHE
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         cmdstr = '('//titlec(1:mchar)//') Le'
       else
         cmdstr = '('//bslash//'242) Le'
       endif
c
c      Move pen to proper coordinates.
c
       scrc   = 'xyd mv'
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                   //scrc(1:lenstr(scrc,132))
c
c      Set angle.
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         scrc = '('//titlec(1:mchar)//') show'
       else
         scrc = '('//bslash//'242) show'
       endif
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                   //scrc(1:lenstr(scrc,132))
c
c      Set angle.
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) -ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
       call filler
c
c      Compute next character position.
c
       cmdstr = ' '
       write ( cmdstr, '(f6.1,'' Xyps0'')' ) ang
       call filler
c
c      Reset font if we drew a prime.
c
       if ( ireset .eq. 1 ) call setfnt ( nfont )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine times ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 8/08 using dk square as a template: writes a 
c  multiplication sign as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of multiplication sign
c  radius      radius of multiplication sign (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius  , rad
c
       real          xa      (2), ya      (2)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       rad   = 0.9 * radius
       xa(1) = xc - rad
       ya(1) = yc - rad
       xa(2) = xc + rad
       ya(2) = yc + rad
       call drwcrv ( xa, ya, 2, 0.0, .false. )
       ya(1) = yc + rad
       ya(2) = yc - rad
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine triangle ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 8/08 using dk square as a template: writes a triangle
c  as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of triangle
c  radius      radius of triangle (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius  
c
       real          xa      (3), ya      (3)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       xa(1) = xc - radius
       ya(1) = yc - radius
       xa(2) = xc + radius
       ya(2) = ya(1)
       xa(3) = xc
       ya(3) = yc + radius
       call drwcrv ( xa, ya, 3, 0.0, .true. )
c
       return
       end
c======================================================================= 
